clear; 
delete(gcp('nocreate'))
mypool = parpool(4);
format long g



%% CALIBRATION
%%%%%%%%%%%%%%
beta  = 0.99;
alpha = 0.64;
En    = 20;	     % set mean (desired) establishment size
delta = 0;       % attrition rate
c     = 0.013;   % fixed cost as share of frictionless revenue
% NOTE: c = 1.3% if inaction = 52.5%
%       c = 2.7% if inaction = 67%
%       c = 5.8% if inaction = 80%
frisch= 0.5;


%% IDIOSYNCRATIC PRODUCTIVITY SHOCK
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rhox       = 0.70; 
Stdev_epsx = 0.15;	 
%Unconditional moments:
E_x        = 1;
Stdev_logx = Stdev_epsx/sqrt(1-rhox^2);
Var_logx   = Stdev_logx^2;
E_logx     = log(E_x) - 0.5*Var_logx;
mux        = (1-rhox)*E_logx;
Stdev_x    = sqrt( (exp(Var_logx)-1)*exp(2*E_logx + Var_logx) );
%Grid: Legendre nodes
Fxmax   = 0.9999;
Fxmin   = 0.0001;
x1      = exp( E_logx + Stdev_logx*norminv(Fxmin) );
xN      = exp( E_logx + Stdev_logx*norminv(Fxmax) );
xobs    = 90;
[xgrid, xweights] = legendre(x1, xN, xobs);
pdfx    = (1./(xgrid*Stdev_logx*sqrt(2*pi))) .* exp(-0.5*( ((log(xgrid) - E_logx)/Stdev_logx).^2 ));
pdfx	= pdfx / (Fxmax-Fxmin);
xwghts	= xweights.*pdfx;



%% AGGREGATE PRODUCTIVITY SHOCK
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rhop       = 0.95;
E_p        = 1;
Var_logp   = 0.003208; % = (0.024*(1+(1/frisch)-alpha))^2; induces observed var(log(N)) in frictionless model 
Stdev_logp = sqrt(Var_logp);
Stdev_epsp = Stdev_logp*sqrt(1-rhop^2);
E_logp     = log(E_p) - 0.5*Var_logp;
mup        = (1-rhop)*E_logp;
Stdev_p    = sqrt( (exp(Var_logp)-1)*exp(2*E_logp + Var_logp) );
Var_epsp   = Stdev_epsp^2;

% Productivity grid: Rouwenhorst transition matrix
pobs   = 11;
logp0  = -sqrt(pobs-1)*Stdev_logp;
logpT  =  sqrt(pobs-1)*Stdev_logp;
logpgrid_ = (logp0 : (logpT-logp0)/(pobs-1) : logpT)';
logpgrid  = logpgrid_+ E_logp; 
pgrid  = exp(logpgrid);
prob1  = (1+rhop)/2;
prob2  = (1+rhop)/2;
Ptrans = [ prob1, 1-prob1; 1-prob2, prob2];
for j  = 1:pobs-2
    Np = j+2;
    Ptrans = prob1*[Ptrans, zeros(Np-1,1); zeros(1,Np-1), 0]  + ...
            (1-prob1)* [zeros(Np-1,1) Ptrans; 0 zeros(1,Np-1)] + ...
            (1-prob2)* [zeros(1,Np-1) 0; Ptrans zeros(Np-1,1)] + ...
            prob2*[0 zeros(1,Np-1); zeros(Np-1,1) Ptrans];
    Ptrans(2:end-1,:) = Ptrans(2:end-1,:)/2; 
end



%% Discrete Employment grid
%  calculate E[ x^(1/(1-alpha)) ]:
Ex_a   = exp((1/(1-alpha))*E_logx  + 0.5*(((1/(1-alpha))*Stdev_logx)^2));
Ex_a   = Ex_a * ((normcdf((log(xgrid(end))-E_logx-(1/(1-alpha))*Var_logx)/Stdev_logx) - ...
                  normcdf((log(xgrid(1  ))-E_logx-(1/(1-alpha))*Var_logx)/Stdev_logx)) / (Fxmax-Fxmin));
%  set wage so that average employment in frictionless model = En (this
%  does account for Jensen effects):
W      = alpha * ((Ex_a/En)^(1-alpha));

%  We need to span optimal choice, conditional on adjusting. 
%  Again, use frictionless choices as guide and scale up
%  and down from there:
nmin   = 1; 
nmax   = ceil( ((alpha*xgrid(xobs)*pgrid(pobs)/W)^(1/(1-alpha))) ); 
ngrid  = (nmin :1: nmax)';
nobs   = length(ngrid);



    
%% Full steady state (p==1)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Policy
[nreset, L,U,v_no,v_adj, vtomax] = policyss(alpha,delta,beta,c,W,  xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights);
splL = spline(ngrid,L);
splU = spline(ngrid,U);
    
% Sims
rng(508);
N     = 500000;
xlag  = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(E_x) + Stdev_epsx*randn(N,1) )));
nlag  = ceil( (alpha*xlag/W).^(1/(1-alpha)) ); % round up to make integer
T0 = 75;
Nsim  = zeros( T0,1 );
PrNOadj  = zeros( T0,1 );
for t = 1:T0
    %% Draw IDIOSYNCRATIC productivity
    xt    = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(xlag) + Stdev_epsx*randn(N,1) )));
    fixedcost = c*((alpha/W)^(alpha/(1-alpha))) *(xt.^(1/(1-alpha)));
   
    %% Policy rules
     % TRIGGERS:
    triggerl = min(xgrid(xobs),max(xgrid(1),  ppval(splL,nlag)));
    triggerh = min(xgrid(xobs),max(xgrid(1),  ppval(splU,nlag)));
    adjustneg= (xt<triggerl);
    adjustpos= (xt>triggerh);
    adjust   = adjustneg + adjustpos;
     % RESET:
     % Recall, V(n_-1,x) = max_n {v(n,x)}.
     % Fit v(n,x) @ x(t). Then optimize 
     % over n.
    vpfit    = spline( xgrid',vtomax', xt'); % should be nobs x N.
    [~,npos] = max( vpfit );
    nadj     = ngrid(npos);
    nsim     = nadj.*adjust + nlag.*(1-adjust);
    
    %% Moments:
    Nsim(t)    = mean( nsim );
    PrNOadj(t) = mean( (nsim==nlag ) );
    
     % reset:
    xlag  = xt;
    nlag  = nsim;
    %if t/5==floor(t/5); disp(t); end
end
t0 = 25;
Nss = mean(Nsim(end-t0:end));  % Use last 25 periods.
PrNoadjss = mean(PrNOadj(end-t0:end));
disp('Prob of not adjusting'); disp(PrNoadjss);
xss = xlag;
nss = nlag;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%







%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% IMPULSE RESPONSE

% Path of aggregate TFP:
T    = 20;
pirf = zeros(T,1);
epsp = log(1.01) -( E_logp + 0.5*Var_logp  - 0.5*Var_epsp );
for t= 1:T;  pirf(t)= exp(E_logp + 0.5*Var_logp) * exp( (rhop^(t-1))*epsp  - 0.5*(rhop^(2*(t-1)))*Var_epsp ) ; end


 % Bins (for flow balance calculations)
nstart = max(ngrid(1),   min(nss)-10);
nfinal = min(ngrid(nobs),max(nss)+10);
nbins  = (nstart :1: nfinal)';
bins   = length(nbins); 



 % Policy Function
[nreset, L,U,v_no,v_adj, vtomax] = policy(alpha,delta,beta,c,W,  pobs,pgrid,Ptrans,  xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights);
%nrmat = reshape(nreset, xobs,pobs);


 % Simulate:
Sims = 100; 
burnin = 24; % allow time to get near steady state, then raise TFP.

Nsim      = zeros( T+burnin,Sims); % Actual
MNsim     = zeros( T+burnin,Sims); % Mandated
FBNsim    = zeros( T+burnin,Sims); % Flow Balance
parfor i= 1:Sims

    xlag  = xss;
    nlag  = nss;
    Nlag  = Nss;
    for t = 1:T+burnin
        %% Draw agg productivity
        if t<=burnin
            pt = 1;
        else
            tt = t-burnin;
            pt = pirf(tt);
        end
        pos2 = find( pgrid >pt,1,'first');
        pos1 = pos2 -1;
        plo  = pgrid( pos1);
        phi  = pgrid( pos2);


        %% Draw IDIOSYNCRATIC productivity
        xt  = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(xlag) + Stdev_epsx*randn(N,1) )));


        %% Aggregation:
        [nt, nadj] = aggregate(pt,plo,phi,pos1,pos2,pobs,  xt,xobs,xgrid,  nlag,nobs,ngrid,  L,U, vtomax);
        Nsim(t,i)  = mean(nt);
        MNsim(t,i) = mean(nadj); 


        %% Flow balance:
        FBNsim(t,i) = flowbalance( N,bins,nbins,nt,nlag,nadj); 

         % reset
        xlag  = xt;
        nlag  = nt;
        Nlag  = Nsim(t,i);
        %if t/5==floor(t/5); disp( [t,i]); end
    end
end

ENsim     = mean(Nsim,   2);
EMNsim    = mean(MNsim,  2);
EFBNsim   = mean(FBNsim, 2);
%disp( [ENsim, EMNsim, EFBNsim] );
ENstar = Ex_a * ( (alpha*pirf/W).^(1/(1-alpha)) ); % Frictionless

N0 = mean(ENsim(burnin-8+1 : burnin));  % Use 2 years prior to TFP increase
MN0 = mean(EMNsim(burnin-8+1 : burnin));
FBN0 = mean(EFBNsim(burnin-8+1 : burnin));
ENstar0= Ex_a * ( (alpha/W)^(1/(1-alpha)) );

disp(' %% IMPULSE RESPONSES %% ');
disp( 'Actual Employment, Mandated Employment, Flow-balance Employment, Frictionless Employment');
disp( [(1:T)', log(ENsim(burnin+1 : end)/N0), log(EMNsim(burnin+1 : end)/MN0), log(EFBNsim(burnin+1 : end)/FBN0),  log(ENstar/ENstar0)] );



            