function [n,L,U,  v_no,v_adj,  vtomax] = policy(alpha,delta,beta,c,W,   pobs,pgrid,Ptrans,  xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights)



%% Myopic rule is initial guess
gridlo   = 0.01;
gridhi   = 2.00;
grid = (gridlo: 0.001*(gridhi-gridlo): gridhi)';
l_m  = zeros(pobs,1);
u_m  = zeros(pobs,1);
for j= 1:pobs % loop over aggregate productivity
     % Solve value matching ('vmc'):
    fixedcost = c*((alpha/W)^(alpha/(1-alpha))) *((pgrid(j)*grid).^(1/(1-alpha)));    
    vmc  = (1-alpha)*((alpha/W)^((alpha/(1-alpha))))*((pgrid(j)*grid).^(1/(1-alpha))) - ...
            fixedcost  -  pgrid(j)*grid  +  W;
    f    = spline(grid,  vmc);
    g    = unique(fnzeros(f));
    if length(g)>1
        l_m(j)  = g(1);
        u_m(j)  = g(2);
    else
        disp('Error: failure to solve myopic policy');
        pause;
    end
end
L_m    = (ngrid.^(1-alpha))*l_m';
U_m    = (ngrid.^(1-alpha))*u_m';
Xgrid1 = repmat(xgrid(1  ),nobs,pobs);
XgridN = repmat(xgrid(end),nobs,pobs);
L_m = min(XgridN, max( Xgrid1, L_m ));
U_m = max(Xgrid1, min( XgridN, U_m ));



%% Profit of adjusting and not adjusting
%  NOT ADJUSTING:
xmat       = repmat(xgrid,  pobs, 1) * ones(1,nobs);
pmat       = kron(pgrid,ones(xobs,1))* ones(1,nobs);
nmat       = repmat(ngrid', xobs*pobs,  1);
profit_no  = pmat.*xmat.*(nmat.^alpha)  -  W*nmat;  % x(t) x p(t) by n(t)
profit     = profit_no;

%  ADJUSTING:
xmat_ = repmat(xgrid,  pobs, 1);
pmat_ = kron(pgrid,ones(xobs,1));
n_m   = (alpha*pmat_.*xmat_./W).^(1/(1-alpha));     % myopic reset function  
costadjust = c*((alpha./W)^(alpha/(1-alpha))) .*((pmat_.*xmat_).^(1/(1-alpha)));
profit_adj = pmat_.*xmat_.*(n_m.^alpha)  -  W*n_m  - costadjust; % x(t) x p(t) by 1



%% Start policy function iteration
 % Initialize:
v_adj  = profit_adj;
v_no   = profit_no;
V_adj  = repmat( v_adj, nobs, 1 );
V_no   = reshape(v_no,  xobs*pobs*nobs, 1);
v      = max( [V_no,V_adj], [], 2 );

L0     = L_m; 
U0     = U_m;
n0     = n_m;
L      = L0;
U      = U0;


 % Transition matrix (for below)
xnext  = ones(xobs,1)*xgrid';
xnow   = xgrid       *ones(1,xobs);
elogx  = mux + rhox*log(xnow);
Fxmax  = normcdf((log(xgrid(xobs))-elogx)/Stdev_epsx);
Fxmin  = normcdf((log(xgrid(1   ))-elogx)/Stdev_epsx);
pdfx   = (1./(xnext*Stdev_epsx*sqrt(2*pi))) .* exp(-0.5*( ((log(xnext)   -elogx)/Stdev_epsx).^2 ));
pdfx   = pdfx ./ (Fxmax -Fxmin);
xwght  = ones(xobs,1)*xweights';
Pdfx   = xwght .* pdfx;
Pdfx   = Pdfx ./ repmat(sum(Pdfx,2), 1, xobs);

TransMat= zeros(xobs*pobs, xobs*pobs); % (x(t),p(t)) --> (x(t+1), p(t+1))
for ip = 1:pobs % p(t)
    for ix = 1:xobs % x(t)
        ro = ix + (ip-1)*xobs;
        TransMat(ro, :) = repmat(Pdfx(ix, :), 1,pobs) .* kron(Ptrans(ip,:), ones(1,xobs));
    end
end
        
 
iter   = 0;
ptol   = 0.001;
perrcount  = 0;
tic
while perrcount <5
    
    %% Expected value function
    Vprime  = reshape(v, xobs*pobs, nobs);
    EV      = TransMat * Vprime;
    
    
    %% Value of not adjusting:
    v_no  = profit_no  + beta*EV;
    
    
    %% Optimal reset (if adjusting):
    vtomax	= profit + beta*EV;
    [vmax, posn] = max( vtomax,[],2);
    n       = ngrid(posn);
    pnerr   = max(max(abs(n-n0)));
    n0      = n;
    
    %% Take max over regimes, and update v:
    v_adj    = vmax - costadjust;
    V_adj    = repmat(  v_adj, nobs,1);
    V_no     = reshape( v_no,xobs*pobs*nobs,1 );
    V        = max( [V_no,V_adj], [],2 );
    verr     = max(abs(V-v));
    v        = V;
    
    %%  Solve for triggers:
    [L,U]   = triggers(nobs,xobs,xgrid, pobs, v_no,v_adj);
    L       = reshape(L, nobs, pobs);
    U       = reshape(U, nobs, pobs);
    pLerr   = max(max(abs(L -L0 )));
    pUerr   = max(max(abs(U -U0 )));
    L0    = L;
    U0    = U;
    
    %% Policy Errors:
    perr      = max( [pnerr,pLerr,pUerr] );  
    perrcount = (perr <ptol)*(perrcount+1); 
    
    
    iter   = iter +1;
    disp('ITERATION'); disp(iter);
    
    disp( 'Policy errors: reset, L, U');   disp( [pnerr,pLerr,pUerr] );
    
    disp( 'Convergence diagnostics: V, policy, and policy count');
    disp( [iter, verr, perr, perrcount] );
end
        
        
        
        