function [L,U] = triggersss(nobs,xobs,xgrid,  v_no,v_adj)



U = zeros(nobs,1);
L = zeros(nobs,1);
parfor j = 1:nobs
    vj_no = v_no(:,j);
    Delta = v_adj- vj_no; %Net value of adjusting.
    [Dstar, pos] =  min(Delta);

    if pos>1 && pos<xobs
         % If min is strictly interior:
        Dspl          = spline( xgrid, Delta );
        [Dmin,argmin] = fnmin( Dspl );
        if Dmin <0
            % V_adj falls below V_no at some point. 
            % Find triggers.
            if      Delta(1)>0 && Delta(xobs)>0
                 % Delta(1)    >0 indicates firing at smallest x.
                 % Delta(xobs) >0 indicates hiring at highest x.
                 % Find lower and upper triggers 
                Dfunc  = spline( xgrid, Delta );
                trig   = unique(fnzeros(Dfunc));
                L(j) = trig(1);
                U(j) = trig(2);

            elseif  Delta(1)>0 && Delta(xobs)<0
                 % Delta(xobs) <0 indicates hiring is never optimal.
                 % Hiring trigger set to max{x}
                Dfunc  = spline( xgrid(1:pos), Delta(1:pos) );
                trig   = unique(fnzeros(Dfunc));
                L(j) = trig(1);
                U(j) = xgrid(xobs);

            elseif  Delta(1)<0 && Delta(xobs)>0
                 % Firing is never optimal
                Dfunc  = spline( xgrid(pos:xobs), Delta(pos:xobs) );
                trig   = unique(fnzeros(Dfunc));
                L(j) = xgrid(1);
                U(j) = trig(1);

            elseif  Delta(1)<0 && Delta(xobs)<0
                 % Adjusting in any direction is not optimal
                L(j) = xgrid(1);
                U(j) = xgrid(xobs);
            end

        else
            disp('Error: Interior minimum of V*-V0 is positive.');
            % If min of V*-V0 is in interior, then it is declining
            % over some x. Why? Because x is approaching X(n_-1), the 
            % value for which n_-1 is exactly optimal. Thus, if V*-V0
            % never reaches at least zero, there is an error.
            pause;
        end 
    end
    

    if pos == 1
        if      Delta(1)<0 && Delta(xobs)>0 
             % Firing is not optimal
            Dfunc = spline( xgrid, Delta );
            trig  = unique(fnzeros(Dfunc));
            L(j) = xgrid(1);
            U(j) = trig(1);
        elseif  Delta(1)<0 && Delta(xobs)<0 
             % Adjusting in any direction is not optimal
            L(j) = xgrid(1);
            U(j) = xgrid(xobs);
        elseif  Delta(1)>0 && Delta(xobs)>0 
             % In this case, V*-V0 is at its min at min{x} 
             % and increasing everywhere to the right.
             % That is, the hiring trigger lies off
             % the grid to the left.
            L(j) = xgrid(1); 
            U(j) = xgrid(1); % Hire if x>U, so HIRE at all x.
        end
    end

    
    if pos == xobs
        if      Delta(1)>0 && Delta(xobs)<0
             % Hiring is not optimal
            Dfunc = spline( xgrid, Delta );
            trig  = unique(fnzeros(Dfunc));
            L(j) = trig(1);
            U(j) = xgrid(xobs);
        elseif  Delta(1)<0 && Delta(xobs)<0
             % Adjusting in any direction is not optimal
            L(j) = xgrid(1);
            U(j) = xgrid(xobs);
        elseif  Delta(1)>0 && Delta(xobs)>0
             % In this case, Delta is at its min at max{x}.
             % So, it is declining everywhere, but never 
             % in fact reaching the firing trigger,
             % which lies off the grid to the 
             % right.
            L(j) = xgrid(xobs); % Fire if x<L so FIRE at all x.
            U(j) = xgrid(xobs);
        end
    end

end