function fbsim = flowbalance( N,bins,nbins,nsim,nlag)
                  

ht  = zeros( bins,1);
hlag= zeros( bins,1);
adjfrom = zeros( bins,1);
inflows = zeros( bins,1);
inflowsh = zeros( bins,1);
inflowsf = zeros( bins,1);
ssdensity = zeros( bins,1);
ssdensityh = zeros( bins,1);
ssdensityf = zeros( bins,1);

for j = 1:bins
     % h(n) %%%%%%%%%%%%%%
    posn  = find( nsim ==nbins(j));
    if isempty(posn)==1; obs = 0; else obs = length(posn); end
    ht(j) = obs/N;
    
     % inflows %%%%%%%%%%%
    posh = find( nsim== nbins(j) & nlag< nbins(j)); %hire up to nbins(j)
    posf = find( nsim== nbins(j) & nlag> nbins(j)); %fire down to nbins(j)
    inflowsh(j) = length(posh) /N;
    inflowsf(j) = length(posf) /N;
    inflows(j)  = inflowsh(j) + inflowsf(j);
    
     % outflows %%%%%%%%%%
    posn = find( nlag ==nbins(j));
    if isempty(posn)==1; obs = 0; else obs = length(posn); end
    hlag(j) = obs/N;
    if isempty(posn)==0
         % if there is mass at n_-1, 
        posl = find( nsim(posn)< nbins(j));
        posr = find( nsim(posn)> nbins(j));
        adjfrom(j) = (length(posl)+length(posr)) /length(posn);
        ssdensityh(j) = inflowsh(j)/adjfrom(j);
        ssdensityf(j) = inflowsf(j)/adjfrom(j);
        ssdensity(j) = inflows(j)/adjfrom(j);
    else
        ssdensity(j) = 0;
    end
end


 % Because # of firms is finite, there can be zeros in the 
 % density (mass) function. 

 % This program treat zeros as if mass were displaced to nearby
 % locations. It aggregates into a bin that surrounds
 % the point(s) where the zeros are. It treats this 
 % bin as a single location. 
 
 % FIRST, annihilate zeros at very small n, 
 % very large n.
posdrop1 = find( hlag >0,1,'first');
posdrop2 = find( hlag >0,1,'last' );
if posdrop1 ==1;    posdrop1 = []; end
if posdrop2 ==bins; posdrop2 = []; end
posdrop  = [(1:posdrop1-1)';  (posdrop2+1:bins)'];

nt = nbins;
nt(posdrop,:) =[];
ht(posdrop,:) =[];
hlag(posdrop,:) =[];
adjfrom(posdrop,:) =[];
inflowsh(posdrop,:) =[];
inflowsf(posdrop,:) =[];
ht   = ht/sum(ht);
hlag = hlag/sum(hlag);

dn = 1;
dnmax= 5; 
nend = nt(end);
htol = 1e-9; 
nk = nt(1);
k = 1;
while nk <=nend
    pos1 = find( nk== nt ); 
    if hlag(pos1) >htol
        % There is mass in t-1:
        dn = 1;
    else
        stop = 0;
        while stop ==0
            dn   = dn + 1; % advance 1
            pos1 = find( nk<= nt  &  nt <=nk +dn-1 );
            stop = (sum(hlag(pos1))>0) * ( (sum(hlag(pos1))> htol) + (nk +dn-1 >nend) + (dn >dnmax) ) ;
             % the stopping criteria: if there is mass (the outer
             % indicator on left), then stop if 1 of 3 conditions are met:
             % if mass in bin exceeds threshold; if n runs off the grid;
             % if forwarded steps exceeds "dnmax"
        end
        % Encase points of zero mass within larger bin
        if hlag(pos1(1)-2) >0
             % This looks back to see if you replaced 
             % on the last iteration. IE, suppose
             % h(pos(1)-3) > 0
             % h(pos(1)-2) = 0
             % h(pos(1)-1) > 0
             % h(pos(1)  ) = 0
             % h(pos(1)+1) > 0
             % In this case, you will have added up mass across pos(1)-3 to
             % pos(1)-1 and introduced a single bin that spans this
             % interval. In that case you do not want to use 
             % pos(1)-1 again. So if this should happen,
             % just pool together pos(1) and pos(1)+1.
             % Otherwise, encase the zero obs within
             % bin that has positive mass on each
             % end.
            pos1 = [pos1(1)-1; pos1];
            if k >1;  k= k -1; end
        end
        % Now, pos1(1)= nearest element to left w/ mass, 
        % pos1(end)= nearest element to right w/ mass.
        %%%%%%%%%%%%%%%%
    end
    
    adjfrom_(k)  = hlag(pos1)'*adjfrom(pos1)/sum(hlag(pos1));
    inflowsh_(k) = sum( inflowsh(pos1) );
    inflowsf_(k) = sum( inflowsf(pos1) );
    hlag_(k)  = sum(hlag(pos1));
    ht_(k) = sum( ht(pos1));
    if sum(ht(pos1)) >0
        nt_(k) = ht(pos1)'*nt(pos1) /sum(ht(pos1));
    else
        nt_(k) = mean(nt(pos1));
    end 
    nk = nk +dn;
    k  = k+1;
    dn = 1;
end
nt_ = nt_';
ht_ = ht_';
hlag_ = hlag_';
adjfrom_ = adjfrom_';
inflowsh_ = inflowsh_';
inflowsf_ = inflowsf_';
inflows_  = inflowsh_ + inflowsf_; 


 % Last, prob(adj away from n) can be zero (with finite # of firms)
 % EVEN IF there is mass (ht >0). In this case, we impute
 % a prob of adjusting based on nearby obs.
bins_= length(nt_);
k = 1;
while k <= bins_
    if adjfrom_(k) ==0
        
        if k ==1
             % If prob of adj = 0 @ first pt. of grid
             % then move to the left and start
             % grid there.
            apos = find( adjfrom_ >0, 1,'first');
            adjfrom_(k) = adjfrom_(apos);
            k = apos;
        
        elseif k ==bins_
             % if prob of adj = 0 @ last pt. of grid
             % then stop grid at last pt. where 
             % prob of adj >0.
            apos = find( adjfrom_ >0, 1,'last');
            adjfrom_(k) = adjfrom_(apos);
            k = k + 1; 

        else
             % first, look for nearby zeros
            stop =0;
            kk =0;
            while stop ==0
                stop = (adjfrom_(k+1+kk) >0); 
                 % E.G., if prob adj >0 at first pt. to 
                 % right of k, then stop = 1
                 % and we move on.
                if stop ==0 && k+1+kk < bins_ 
                    kk = kk +1;
                elseif stop ==0 && k+1+kk ==bins_
                     % you have advanced to the end
                     % of the grid.
                    stop = 1;
                end
            end
            
            if k+1+kk ==bins_
                 % if zeros stretch to the end of the grid
                 % replace with last positive value.
                adjfrom_(k:end) = adjfrom_(k-1);
            else
                 % This linearly interpolates where there are zeros.
                 % Note - if k is the only zero, then kk=0.
                adjfrom_(k:k+kk) = adjfrom_(k-1) + (1/(2+kk))*(1:kk+1)'*(adjfrom_(k+1+kk) - adjfrom_(k-1));
            end
            k = k + 1 +kk;
        end
    else
        k = k + 1;
    end
end

 % Calculate flow balance density as we do in the QCEW data:
ssdensity  = hlag_ + ((ht_-hlag_)./adjfrom_);
fbsim      = nt_'*ssdensity/sum(ssdensity); 

clear nt_ ht_ hlag_ adjfrom_ inflows_;








