function [nh,nf, v_no,v_adjh,v_adjf, vtomaxh,vtomaxf] = policyks(alpha,beta,W,hiringcost,firingcost,   pobs,pgrid,Ptrans,  xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights, Nvec,Nobs,Wage,Np,Nlag,Ncons)
    

%% Myopic rule is initial guess
% Don't adjust:
Wmat   = kron(Wage,ones(pobs*xobs,1)) * ones(1,nobs);
pmat   = repmat( kron(pgrid,ones(xobs,1)),Nobs,1) * ones(1,nobs);
xmat   = repmat( xgrid,Nobs*pobs,1 ) * ones(1,nobs);
nmat   = ones(xobs*pobs*Nobs,1) * ngrid';               
profit_no  = pmat.*xmat.*(nmat.^alpha)  -  Wmat.*nmat;  

% Adjust UP:
chiring = (1-beta*(1-.06))*hiringcost; % annuitized cost better mimics solution under foresight 
wmat_   = ones(xobs,1)*kron( Wage',ones(1,pobs));
pmat_   = ones(xobs,1)*repmat( pgrid',1,Nobs);
xmat_   = xgrid*ones(1,pobs*Nobs);
nh_m    = (alpha*pmat_.*xmat_./(wmat_+chiring)).^(1/(1-alpha));  
Nh_m    = reshape(nh_m,xobs*pobs*Nobs,1)*ones(1,nobs);                                           
profit_adjh = pmat.*xmat.*(Nh_m.^alpha)  -  Wmat.*Nh_m  - hiringcost*(Nh_m-nmat).*(Nh_m >nmat); 

% Adjust DOWN:
cfiring = (1-beta*(1-.06))*firingcost;
nf_m    = (alpha*pmat_.*xmat_./(wmat_-cfiring)).^(1/(1-alpha));   
Nf_m    = reshape(nf_m,xobs*pobs*Nobs,1)*ones(1,nobs);
profit_adjf = pmat.*xmat.*(Nf_m.^alpha)  -  Wmat.*Nf_m  - firingcost*(nmat-Nf_m).*(nmat >Nf_m); 

% For use in optimization,
profith    = pmat.*xmat.*(nmat.^alpha)  -  Wmat.*nmat  -hiringcost*nmat;
profitf    = pmat.*xmat.*(nmat.^alpha)  -  Wmat.*nmat  +firingcost*nmat;




%% Start value iteration
v_adjh = profit_adjh; 
v_adjf = profit_adjf; 
v_no   = profit_no;   
V_adjh = v_adjh.*(Nh_m >nmat) - 10e6*(Nh_m <=nmat);
V_adjf = v_adjf.*(Nf_m <nmat) - 10e6*(Nf_m >=nmat);
v0     = max( [reshape(V_adjf, xobs*pobs*Nobs*nobs,1),  ...
               reshape(v_no,   xobs*pobs*Nobs*nobs,1),  ...
               reshape(V_adjh, xobs*pobs*Nobs*nobs,1)], [],2 );
v0     = reshape( v0, xobs*pobs*Nobs,nobs );

nh     = nh_m;
nf     = nf_m;
nh0    = nh;
nf0    = nf;

iter   = 0;
ptol   = 0.005;
perrcount  = 0;
while perrcount <5
    
    %% Iterate forward: for given v, compute E[v].
    EV      = evks(nobs,xobs,xgrid,ngrid,xweights,  pobs,pgrid,Ptrans,  v0,  mux,rhox,Stdev_epsx, Nvec,Nobs,Ncons,Nlag,Np); 
    

    %% Update:
    vtomaxh = profith   + beta*EV;
    vtomaxf = profitf   + beta*EV;
     % HIRING:
    [vmaxh, pos] = max(vtomaxh,[],2);  
    nnewh  = ngrid(pos);
    v_adjh = vmaxh*ones(1,nobs) + ones(xobs*pobs*Nobs,1)*hiringcost*ngrid'; %add back in lagged n
    nh     = reshape( nnewh,xobs,pobs*Nobs); 
     % FIRING
    [vmaxf, pos] = max(vtomaxf,[],2);  
    nnewf  = ngrid(pos);
    v_adjf = vmaxf*ones(1,nobs) - ones(xobs*pobs*Nobs,1)*firingcost*ngrid'; 
    nf     = reshape( nnewf,xobs,pobs*Nobs);
     % INACTION:
    v_no   = profit_no + beta*EV;

     % Choose regime:
    Nh     = nnewh*ones(1,nobs);
    Nf     = nnewf*ones(1,nobs);
    V_adjh = v_adjh.*(Nh >nmat) - 10e6*(Nh <=nmat);
    V_adjf = v_adjf.*(Nf <nmat) - 10e6*(Nf >=nmat);
    v      = max( [reshape(V_adjf, xobs*pobs*Nobs*nobs,1),  ...
                   reshape(v_no,   xobs*pobs*Nobs*nobs,1),  ...
                   reshape(V_adjh, xobs*pobs*Nobs*nobs,1)], [],2 );
    v      = reshape( v, xobs*pobs*Nobs,nobs );
    
    
     % Value and policy errors:
    iter   = iter +1;
    perr_h = max( max(abs(nh-nh0)) );
    perr_f = max( max(abs(nf-nf0)) );
    perr   = max( [perr_h,perr_f] );
    verr   = max(max(abs( v-v0)));
    disp('Iteration and value and policy errors');
    disp( [iter,  verr,  perr] );
    perrcount = (perr <ptol).*(perrcount +1);
    nh0 = nh;
    nf0 = nf;
    v0  = v;

end

