function [nh,nf, v_no,v_adjh,v_adjf, vtomaxh,vtomaxf] = policyss(alpha,beta,  xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights,  W,hiringcost,firingcost)
             


%% Initialize with static solution.
 % Hiring and firing costs suitable for infinite-horizon
 % problem are way too large for static problem (static
 % gains too small to warrant adjustment). To better
 % mimic solution under foresight, use 
 % annuitized hiring, firing costs.
ch   = (1-beta*(1-.025))*hiringcost;
cf   = (1-beta*(1-.025))*firingcost;

% Don't adjust:
xmat   = xgrid * ones(1,nobs);
nmat   = ones(xobs,1) * ngrid';             
profit_no  = xmat.*(nmat.^alpha)  -  W*nmat;  

% Adjust UP:
nh_m   = (alpha*xgrid/(W+ch)).^(1/(1-alpha));   
Nh_m   = nh_m*ones(1,nobs);   
profit_adjh = xmat.*(Nh_m.^alpha) -  W*Nh_m  - hiringcost*(Nh_m-nmat).*(Nh_m >nmat); 

% Adjust DOWN:
nf_m   = (alpha*xgrid/(W-cf)).^(1/(1-alpha));               
Nf_m   = nf_m*ones(1,nobs);
profit_adjf = xmat.*(Nf_m.^alpha) -  W*Nf_m  - firingcost*(nmat-Nf_m).*(Nf_m <nmat);    

% For use in optimization,
profith    = xmat.*(nmat.^alpha)  -  W*nmat  - hiringcost*nmat;
profitf    = xmat.*(nmat.^alpha)  -  W*nmat  + firingcost*nmat;




%% Start value iteration
v_adjh = profit_adjh; 
v_adjf = profit_adjf; 
v_no   = profit_no;   
V_adjh = v_adjh.*(Nh_m >nmat) - 10e6*(Nh_m <=nmat);
V_adjf = v_adjf.*(Nf_m <nmat) - 10e6*(Nf_m >=nmat);
vprime = max( [reshape(V_adjf,xobs*nobs,1), ...
               reshape(v_no,  xobs*nobs,1), ...
               reshape(V_adjh,xobs*nobs,1)], [],2 );
v0     = reshape( vprime, xobs,nobs );
nh     = nh_m;
nf     = nf_m;
nh0    = nh;
nf0    = nf;


 % Transition matrix (for use below)
xnext  = ones(xobs,1)*xgrid';
xnow   = xgrid       *ones(1,xobs);
elogx  = mux + rhox*log(xnow);
Fxmax  = normcdf((log(xgrid(xobs))-elogx)/Stdev_epsx);
Fxmin  = normcdf((log(xgrid(1   ))-elogx)/Stdev_epsx);
pdfx   = (1./(xnext*Stdev_epsx*sqrt(2*pi))) .* exp(-0.5*( ((log(xnext)   -elogx)/Stdev_epsx).^2 ));
pdfx   = pdfx ./ (Fxmax -Fxmin);
xwght  = ones(xobs,1)*xweights';
Pdfx   = xwght .* pdfx;

 
    

iter   = 0;
ptol   = 0.005;
perrcount  = 0;
while perrcount <5
    
    %% Compute E[v], given policy rules.
    EV      = Pdfx*v0; 
    
    
    %% Optimize
    vtomaxh = profith   + beta*EV;
    vtomaxf = profitf   + beta*EV;
     % HIRING:
    [vmaxh , pos]= max(vtomaxh,[],2);  
    nh     = ngrid(pos);
    v_adjh = vmaxh*ones(1,nobs) + hiringcost*ones(xobs,1)*ngrid'; %add back in lagged n
     % FIRING
    [vmaxf, pos]= max(vtomaxf,[],2);  
    nf     = ngrid(pos);
    v_adjf = vmaxf*ones(1,nobs) - firingcost*ones(xobs,1)*ngrid'; 
     % INACTION:
    v_no   = profit_no + beta*EV;
    
     % Choose regime:
    Nh     = nh*ones(1,nobs);
    Nf     = nf*ones(1,nobs);
    V_adjh = v_adjh.*(Nh >nmat) - 10e6*(Nh <=nmat);
    V_adjf = v_adjf.*(Nf <nmat) - 10e6*(Nf >=nmat);
    v      = max( [reshape(V_adjf,xobs*nobs,1), ...
                   reshape(v_no,  xobs*nobs,1), ...
                   reshape(V_adjh,xobs*nobs,1)], [],2 );
    v      = reshape( v, xobs,nobs );
    
    
    % Value and policy errors:
    iter   = iter +1;
    verr   = max(max(abs(  v-v0  )));
    perr_h = max( max(abs(nh-nh0)) );
    perr_f = max( max(abs(nf-nf0)) );
    perr   = max( [ perr_h,perr_f] );
    disp('Iteration and value and policy errors');
    disp( [iter,  verr,  perr] );
    
    perrcount = (perr <ptol).*(perrcount +1);
    nh0 = nh;
    nf0 = nf;
    v0  = v;
end


