clear; clc;
plotflag = 0;
poolobj = gcp('nocreate');
delete(poolobj);
format long g
myPool = parpool(20);

%% CALIBRATION
%%%%%%%%%%%%%%
beta  = 0.99;
alpha = 0.64;
En    = 20;	     % set mean (desired) establishment size
delta  = 0;      % attrition rate


%% IDIOSYNCRATIC PRODUCTIVITY SHOCK
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rhox       = 0.70;
Stdev_epsx = 0.15;
%Unconditional moments:
E_x        = 1;
Stdev_logx = Stdev_epsx/sqrt(1-rhox^2);
Var_logx   = Stdev_logx^2;
E_logx     = log(E_x) - 0.5*Var_logx;
mux        = (1-rhox)*E_logx;
Stdev_x    = sqrt( (exp(Var_logx)-1)*exp(2*E_logx + Var_logx) );
Skewn_x    = (exp(Var_logx)+2) * sqrt(exp(Var_logx)-1);
%Conditional moments (recognizing the bounds on x):
Fxmax    = 0.9999;
Fxmin    = 0.0001;
x1       = exp( E_logx + Stdev_logx*norminv(Fxmin) );
xN       = exp( E_logx + Stdev_logx*norminv(Fxmax) );
e_x      = exp( E_logx + 0.5*Var_logx )*((normcdf((log(xN)-E_logx-  Var_logx)/Stdev_logx)-normcdf((log(x1)-E_logx-  Var_logx)/Stdev_logx))/(Fxmax-Fxmin));
var_x    = exp( 2*(E_logx + Var_logx) )*((normcdf((log(xN)-E_logx-2*Var_logx)/Stdev_logx)-normcdf((log(x1)-E_logx-2*Var_logx)/Stdev_logx))/(Fxmax-Fxmin)) ...
    +  E_x^2  -  2*E_x*e_x;
stdev_x  = sqrt(var_x);
skewn_x  = exp(3*E_logx + 4.5*Var_logx)*((normcdf((log(xN)-E_logx-3*Var_logx)/Stdev_logx)-normcdf((log(x1)-E_logx-3*Var_logx)/Stdev_logx))/(Fxmax-Fxmin)) ...
    - 3*E_x*(var_x - (E_x^2) + 2*E_x*e_x)  +  3*(E_x^2)*e_x  - (E_x^3);
skewn_x  = skewn_x / (Stdev_x^3);
% Productivity grid: Legendre nodes
xobs    = 90;
[xgrid, xweights] = legendre(x1, xN, xobs);
pdfx    = (1./(xgrid*Stdev_logx*sqrt(2*pi))) .* exp(-0.5*( ((log(xgrid) - E_logx)/Stdev_logx).^2 ));
pdfx	= pdfx / (Fxmax-Fxmin);
xwghts	= xweights.*pdfx;

%Hit 86% inaction
linearcost  = 0.5;
ENirf = zeros(12,1);
EFBirf = ENirf;
EYirf = ENirf;

%% AGGREGATE PRODUCTIVITY SHOCK
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rhop       = 0.9048;%From Figure 10 calibration
E_p        = 1;
Var_logp   = (0.024*(1-alpha))^2;
Stdev_logp = sqrt(Var_logp);
Stdev_epsp = Stdev_logp*sqrt(1-rhop^2);
E_logp     = log(E_p) - 0.5*Var_logp;
mup        = (1-rhop)*E_logp;
Stdev_p    = sqrt( (exp(Var_logp)-1)*exp(2*E_logp + Var_logp) );
Var_epsp   = Stdev_epsp^2;

% Productivity grid: Rouwenhorst transition matrix
pobs   = 11;
logp0  = -sqrt(pobs-1)*Stdev_logp;
logpT  =  sqrt(pobs-1)*Stdev_logp;
logpgrid_ = (logp0 : (logpT-logp0)/(pobs-1) : logpT)';
logpgrid  = logpgrid_+ E_logp;
pgrid  = exp(logpgrid);
prob1  = (1+rhop)/2;
prob2  = (1+rhop)/2;
Ptrans = [ prob1, 1-prob1; 1-prob2, prob2];
for j  = 1:pobs-2
    Np = j+2;
    Ptrans = prob1*[Ptrans, zeros(Np-1,1); zeros(1,Np-1), 0]  + ...
        (1-prob1)* [zeros(Np-1,1) Ptrans; 0 zeros(1,Np-1)] + ...
        (1-prob2)* [zeros(1,Np-1) 0; Ptrans zeros(Np-1,1)] + ...
        prob2*[0 zeros(1,Np-1); zeros(Np-1,1) Ptrans];
    Ptrans(2:end-1,:) = Ptrans(2:end-1,:)/2;
end
%  calculate E[ x^(1/(1-alpha)) ]:
Ex_a   = exp((1/(1-alpha))*E_logx  + 0.5*(((1/(1-alpha))*Stdev_logx)^2));
Ex_a   = Ex_a * ((normcdf((log(xgrid(end))-E_logx-(1/(1-alpha))*Var_logx)/Stdev_logx) - ...
    normcdf((log(xgrid(1  ))-E_logx-(1/(1-alpha))*Var_logx)/Stdev_logx)) / (Fxmax-Fxmin));

%% FIXED EFFECTS DISTRIBUTION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Assume pareto fixed effects
alphap = 2.9; 
xm = .7;
probs = [.01 0.05 .1:.1:.9 .91:.01:0.99 .995];
zgrid = xm*(1-probs).^(-1/alphap);
zobs = length(zgrid);

W = .29;

hiringcost   = ones(1,zobs).*linearcost*W;
firingcost  = ones(1,zobs).*linearcost*W;
%% Employment grid
%We will eliminate employment above 1000, but keep some room in policy function
nmin   = 1;
nmax   = 1050;
ngrid  = [(1:1:99)';(100:5:250)';(260:10:nmax)'];
nobs   = length(ngrid);


%% First, full steady state
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
bargpower = 0;
threatpoint = W;
scaleprodfn = 1;
% Function written so that it can accommodate a SZ wage bargain. To
% shut off, set "0" for zero bargaining power,  insert W for the outside
% option, or threat point, and do not scale the production function (set
% this to 1).

[ssnh,ssnf, ssv_no,ssv_adjh,ssv_adjf,EVss]=policylineardiscretess(alpha,beta,  xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights,  bargpower,threatpoint,scaleprodfn,hiringcost, firingcost,zgrid,zobs);

%Grid for choice of n over integer
ngridc = (1:1:ngrid(end))';
nobsc = length(ngridc);

%Interpolate EVss onto finer employment grid first
EVssi = permute(EVss,[2 1 3]);
EVssi = interp1(ngrid,EVssi,ngridc);
EVssi = permute(EVssi,[2 1 3]);

% Simulate invariant (steady-state) distribution of n.
rng( 508);
N     = 100000;
T0 = 50;

hiringcostsim = W*linearcost;firingcostsim = W*linearcost;
mobs = 10;
Fzmax    = 0.995;
Fzmin    = 0.01;
[zsims, zweights] = legendre(zgrid(1), zgrid(end), mobs);
pdfz    = (alphap*xm^alphap)./(zsims.^(alphap+1));
pdfz	= pdfz / (Fzmax-Fzmin);
pdfz	= (zweights.*pdfz)';
pdfz    = pdfz./sum(pdfz);

nmat = repmat(ngridc',[N 1]);
nmatalpha = nmat.^alpha;
fbdensitymat = zeros(nobsc,mobs);
htmat        = zeros(nobsc,mobs);
nlagmat      = zeros(N,mobs);
xfbmat       = zeros(N,mobs);
PrNOadj      = zeros( T0,mobs );
En           = zeros(T0,mobs);
Sh100        = En;
Sh200        = En;
Sh500        = En;
Sh10         = En;
Sh5          = En;
Sh1000       = En;

for i = 1:mobs
    
    zsim  = zsims(i);
    xlag  = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(E_x) + Stdev_epsx*randn(N,1) )));
    nlag  = ceil( (alpha.*xlag.*zsim/(W)).^(1/(1-alpha)) ); % round up.
    zsimm  = zsim;
    EVi   = interpn(xgrid,ngridc,zgrid,EVssi,xgrid,ngridc,zsim);
    nlagm = repmat(nlag,[1 nobsc]);
 

    for t = 1:T0
        %% Draw IDIOSYNCRATIC productivity
        xt    = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(xlag) + Stdev_epsx*randn(N,1) )));
        xtm = repmat(xt,[1 nobsc]);
        
        EVt = interp1(xgrid,EVi,xt);
        %%Now maximize
        vh =  scaleprodfn.*zsimm.*xtm.*nmatalpha  -  threatpoint*nmat  - hiringcostsim.*(nmat - nlagm).*(nmat>nlagm) + beta*EVt;
        [~,npos] = max( vh,[],2 );
        reseth   = ngridc(npos);
        adjustpos= (nlag <reseth);
        % Firing:
        vf =  scaleprodfn*zsimm.*xtm.*nmatalpha  -  threatpoint*nmat  -  firingcostsim.*(nlagm-nmat).*(nmat<nlagm) + beta*EVt;
        [~,npos] = max( vf , [],2);
        resetf   = ngridc(npos);
        adjustneg= (nlag >resetf);
        % New Employment:
        adjust   = adjustneg + adjustpos;
        nsim     = reseth.*adjustpos + resetf.*adjustneg + nlag.*(1-adjust);
        
        %Discard above 1000
        ros1000c = find(nsim >= 1000);
        ros1000l = find(nlag >= 1000);
        %create union of ros1000 and ros1000l
        ros1000 = union(ros1000c,ros1000l);
        %eliminate
        nsim1 = nsim;
        nlag1 = nlag;
        nsim1(ros1000) = [];
        nlag1(ros1000) = [];
        N1 = length(nsim1);
        Sh1000(t,i)  = mean(nsim>=1000);    
        %% Moments on data excluding 1000
        PrNOadj(t,i) = mean( (nsim1==nlag1 ) );
        En(t,i)      = mean(nsim1);
        Sh100(t,i)   = mean(nsim1>=100);
        Sh200(t,i)   = mean(nsim1>=200);
        Sh500(t,i)   = mean(nsim1>=500);
        Sh10(t,i)    = mean(nsim1<=10);
        Sh5(t,i)     = mean(nsim1<=5);
        
        if t > T0 - 10
            [nhatsim,fbdensity,nt,ht] = flowbalance_size(nsim1,nlag1);
            fbdensitymat(nt,i) = fbdensity/sum(fbdensity);
            %saves only last
            htmat(nt,i)        = ht;
            nlagmat(:,i)       = nlag;
            xfbmat(:,i)        = xt;
        end
        % reset: save the unredacted variables
        xlag = xt;
        nlag  = nsim;
        nlagm = repmat(nlag,[1 nobsc]);
        
    end
    disp(i)
end
t0 = 25;

%Integrate over z's
disp('Mean employment');
En_ = mean(En(end-t0:end,:),1);
En_ = pdfz*En_';
disp(En_)

disp('Prob of not adjusting');
PrNoadj_ = mean(PrNOadj(end-t0:end,:),1);
PrNoadj_ = pdfz*PrNoadj_';
disp(PrNoadj_);

disp('Share above 100');
Sh100_ = mean(Sh100(end-t0:end,:),1);
Sh100_ = pdfz*Sh100_';
disp(Sh100_);

disp('Share above 200');
Sh200_ = mean(Sh200(end-t0:end,:),1);
Sh200_ = pdfz*Sh200_';
disp(Sh200_);

disp('Share above 500');
Sh500_ = mean(Sh500(end-t0:end,:),1);
Sh500_ = pdfz*Sh500_';
disp(Sh500_);

disp('Share below 10');
Sh10_ = mean(Sh10(end-t0:end,:),1);
Sh10_ = pdfz*Sh10_';
disp(Sh10_);

disp('Share below 5');
Sh5_ = mean(Sh5(end-t0:end,:),1);
Sh5_ = pdfz*Sh5_';
disp(Sh5_);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Policy Function Out of Steady State
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[nh,nf, v_no,v_adjh,v_adjf, EVp] = policylineardiscrete(alpha,beta,W,hiringcost,firingcost,   pobs,pgrid,Ptrans,  xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights,zgrid,zobs);
%EVp is pobs x xobs x nobs x zobs
%Interpolate EVss onto finer employment grid first
EVpi = permute(EVp,[3 1 2 4]);
EVpi = interp1(ngrid,EVpi,ngridc);
EVpi = permute(EVpi,[2 3 1 4]);

T    = 100;
Sims = 24;
horizon = 12;
%   Path of aggregate TFP:
psim = zeros(T,Sims);
epsp = Stdev_epsp*randn(T,Sims);
for s= 1:Sims
    plag = 1;
    for t= 1:T
        logpsim  = -0.5*Var_logp + rhop*log(plag) + epsp(t,s);
        psim(t,s)= max(pgrid(1), min(pgrid(pobs),exp(logpsim)) );
        plag   = psim(t,s);
    end
end

%  Path of idiosyncratic TFP:
xsim  = zeros( T,N,mobs);

for t = 1:T
    for i = 1:mobs
    xt  = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(xfbmat(:,i)) + Stdev_epsx*randn(N,1) )));
    xsim(t,:,i) = xt;
    xlag= xt;
    end
    
end
clear xt;

Nsim = zeros( T,Sims);
Ysim  = zeros( T,Sims);
YIRF  = zeros(horizon,Sims);
NIRF  = zeros(horizon,Sims);
FBIRF = zeros(horizon,Sims);

lags  = 4;
lags2 = 3; %allow different lags for innovation. Must have lags2+1 >=lags
Lags2 = max(lags2, lags);

nmatc = repmat(ngridc',[N 1]);
nmatalpha = nmatc.^alpha;

for i = 1:Sims
    
    fbdensitymat = zeros(nobsc,mobs);
    Nsimi=zeros(T,mobs);
    Ysimi=zeros(T,mobs);
    FBNsimv_normi= zeros(T,mobs);    
    for j = 1:mobs

     zsim = zsims(j);
     nlag  = nlagmat(:,j);
     zsimm  = zsim;
     EVi   = interpn(pgrid,xgrid,ngridc,zgrid,EVpi,pgrid,xgrid,ngridc,zsim);
    
    % initialize
    nlagm = repmat(nlag,[1 nobsc]);
    
    for t = 1:T
        %% Draw agg productivity
        pt  = psim(t,i);
        
        %% Draw IDIOSYNCRATIC productivity
        xt   = xsim(t,:,j)';
        xtm  = repmat(xt,[1 nobsc]);
        %% Interpolate the EV function over p first
        pos2 = find( pgrid >=pt,1,'first');
        pos1 = max(1,pos2 -1);
        plo  = pgrid( pos1);
        phi  = pgrid( pos2);
        
        EVt      = EVi(pos1,:,:) + ((pt-plo)/(phi-plo+1e-10))*( EVi(pos2,:,:) -EVi(pos1,:,:));
        EVt1      = squeeze(EVt);
        EVt2 = interp1(xgrid,EVt1,xt);
        
        % Hiring:
        vh =  pt.*zsimm.*xtm.*nmatalpha  -  W.*nmatc  - hiringcostsim.*(nmatc - nlagm).*(nmatc>nlagm) + beta*EVt2;
        [~,npos] = max( vh,[],2 );
        nhire   = ngridc(npos);
        % Firing:
        vf =  pt*zsimm.*xtm.*nmatalpha  -  W*nmatc  - firingcostsim.*(nlagm-nmatc).*(nmatc<nlagm) + beta*EVt2;
        [~,npos] = max( vf , [],2);
        nfire  = ngridc(npos);
        % New Employment:
        nt     = nlag.*(nhire<=nlag).*(nlag<=nfire) + nhire.*(nlag<nhire) + nfire.*(nlag>nfire);
        
        %Discard above 1000
        ros1000c = find(nt >= 1000);
        ros1000l = find(nlag >= 1000);
        %create union of ros1000 and ros1000l
        ros1000 = union(ros1000c,ros1000l);
        %eliminate
        nt1 = nt;
        nlag1 = nlag;
        xt1   = xt;
        nt1(ros1000) = [];
        nlag1(ros1000) = [];
        xt1(ros1000)   = [];
        [nhatsim,fbdensity,nts] = flowbalance_size(nt1,nlag1);
        fbdensitymat(nts,j) = fbdensity/sum(fbdensity);
        
        FBNsimv_normi(t,j) = nhatsim/sum(fbdensity);
        Nsimi(t,j) = mean(nt1);
        Ysimi(t,j) = pt*mean( zsim.*xt1.*(nt1.^alpha) );
        
        % reset:
        nlag  = nt;
        nlagm = repmat(nlag,[1 nobsc]);
        
    end
        disp(j);
    end
    %Integrate over z
    FBNsimv_normi = (pdfz*(FBNsimv_normi'))';
    Nsimi = (pdfz*(Nsimi'))';
    Ysimi = (pdfz*(Ysimi'))';
    % IRFs %%%%%%%%%%%%%%%%%%%%%
    Ndev=log(Nsimi);
    FBNdev=log(FBNsimv_normi);
    YLdev = log(Ysimi./Nsimi);
    
    
    %% 0)   Estimate surprises to output per worker.
    ylmat =[];
    nmat = [];
    
    for j = 1:lags;
        ylmat = [ylmat, YLdev(lags-(j-1): end-j)];
        nmat  = [nmat,  Ndev( lags-(j-1): end-j)];
    end
    ymatrix  = [ylmat, nmat];
    ymatrix  = ymatrix  -  repmat(mean(ymatrix),length(ymatrix),1); %DEMEAN.
    ylcoeff  = (ymatrix'*ymatrix)\(ymatrix'*YLdev(lags+1:end));
    ylresid  = YLdev(lags+1:end) - ymatrix*ylcoeff;
    
    %%  I)   Estimate impulse responses with surprises
    ylmat = [];
    fbnmat = [];
    nmat  = [];
    Ndev_ = Ndev( lags+1:end, 1);
    FBNdev_ = FBNdev(lags+1:end,1);
    
    
    for j = 1:lags;
        ylmat = [ylmat,  ylresid(Lags2+1 -(j-1) : end-j+1)];   %include contemporaneous Y/L
        fbnmat = [fbnmat,  FBNdev_( Lags2   -(j-1) : end-j  )];
        nmat  = [nmat,   Ndev_(  Lags2   -(j-1) : end-j  )];
    end
    if lags2 >=lags;
        % add additional lags of y/n
        for j = 1:lags2-lags+1;
            ylmat = [ylmat, ylresid(Lags2-lags+2-j : end-lags+1-j)];
        end
    end
    % EMPLOYMENT:
    nmatrix  = [ylmat, nmat]; 
    nmatrix  = nmatrix - repmat( mean(nmatrix), length(nmatrix),1);
    ncoeff   = (nmatrix'*nmatrix)\(nmatrix'*Ndev_(Lags2+1:end));
    nresid   = Ndev_(Lags2+1:end) - nmatrix*ncoeff;
    
    % FB EMPLOYMENT:
    fbnmatrix  = [ylmat, fbnmat]; 
    fbnmatrix  =fbnmatrix - repmat( mean(fbnmatrix), length(fbnmatrix),1);
    fbncoeff   = (fbnmatrix'*fbnmatrix)\(fbnmatrix'*FBNdev_(Lags2+1:end));
    
    
    % IRFs:
    yirf  = zeros(horizon,1);
    nirf  = zeros(horizon,1);
    fbnirf = zeros(horizon,1);
    
    ysurp = 0.01;
    ylmat = [ysurp, zeros(1,lags2)];
    nmat  = zeros(1,lags);
    fbnmat = zeros(1,lags);
    
    
    ylmat2= zeros(1,lags);
    for t = 1:horizon;
        %Employment:
        xmatrix = [ylmat,nmat];
        nirf(t) = xmatrix*ncoeff;
        xmatrixs = [ylmat,fbnmat];
        fbnirf(t) = xmatrixs*fbncoeff;
        
        %Output per worker:
        if t==1;
            yirf(t) = ysurp;
        else
            xmatrix = [ylmat2,nmat];
            yirf(t) = xmatrix*ylcoeff;
        end
        
        % Reset:
        nmat  = [nirf(t),   nmat(1:lags-1)]; %reset n.
        fbnmat = [fbnirf(t), fbnmat(1:lags-1)];
        
        ylmat2= [yirf(t), ylmat2(1:lags-1)];
        
        if t <lags2;
            %ylmat is vector of current and past residuals.
            ylmat= [zeros(1,t), ysurp, zeros(1,lags2-t)];
        elseif t==lags2;
            ylmat= [zeros(1,t), ysurp];
        else
            ylmat= zeros(1,lags2+1);
        end
    end
    YIRF(:,i) =yirf;
    NIRF(:,i) =nirf;
    FBNIRF(:,i) =fbnirf;
    disp(i);
end
clear xt nt prodt;
EYirf(:,1)  = mean(YIRF,2);
ENirf(:,1)  = mean(NIRF,2);
EFBirf(:,1) = mean(FBNIRF,2);
moments = zeros(12,1);
moments(1:11,1)  = [En_;PrNoadj_;Sh5_;Sh10_;Sh100_;Sh200_;Sh500_;linearcost;alphap;xm;W];
dlmwrite('IRF_size.txt', [ENirf,EFBirf,moments]);







