function [nh,nf, v_no,v_adjh,v_adjf, EVp] = policylineardiscrete(alpha,beta,W,hiringcost,firingcost,   pobs,pgrid,Ptrans,  xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights,zgrid,zobs)
    

%% Myopic rule is initial guess
% Don't adjust:
pmat   = kron(pgrid,ones(xobs,1)) * ones(1,nobs);
xmat   = repmat( xgrid,pobs,1 ) * ones(1,nobs);
nmat   = ones(xobs*pobs,1) * ngrid';	% ngrid is interpreted as labor IN USE in period t, in event of nonadjustment.
pmat_  = ones(xobs,1)*pgrid';
xmat_  = xgrid*ones(1,pobs);

ngridc = (1:1:ngrid(end))';
nobsc = length(ngridc);
pmatc   = kron(pgrid,ones(xobs,1)) * ones(1,nobsc);
xmatc   = repmat( xgrid,pobs,1 ) * ones(1,nobsc);
nmatc   = ones(xobs*pobs,1) * ngridc';	% ngrid is interpreted as labor IN USE in period t, in event of nonadjustment.

profit_no = zeros(xobs*pobs,nobs,zobs);
profit_adjh  = zeros(xobs*pobs,nobs,zobs);
profit_adjf = zeros(xobs*pobs,nobs,zobs);
nh_m        = zeros(xobs,pobs,zobs);
nf_m        = zeros(xobs,pobs,zobs);
Nh_m        = zeros(xobs*pobs,nobs,zobs);
Nf_m        = zeros(xobs*pobs,nobs,zobs);
parfor i = 1:zobs
    profit_no(:,:,i)  = pmat.*zgrid(i).*xmat.*(nmat.^alpha)  -  W*nmat;
    
    % Adjust UP:
    % profits are used as initial guess for value of HIRE regime.
    ch     = (1-beta*(1-.06))*hiringcost(i); %again, the annuitized cost for the sake of initializing policy.
    nh_m(:,:,i)   = (alpha*pmat_.*xmat_./(W+ch)).^(1/(1-alpha));   % xobs by pobs by zobs
    Nh_m(:,:,i)   = reshape(nh_m(:,:,i),xobs*pobs,1)*ones(1,nobs); %xobs*pobs by nobs by zobs
    profit_adjh(:,:,i) = pmat.*zgrid(i).*xmat.*(Nh_m(:,:,i).^alpha)  -  W*Nh_m(:,:,i)  - hiringcost(i)*(Nh_m(:,:,i)-nmat).*(Nh_m(:,:,i) >nmat);
    
    % Adjust DOWN:
    cf     = (1-beta*(1-.06))*firingcost(i);
    nf_m(:,:,i)   = (alpha*pmat_.*xmat_./(W-cf)).^(1/(1-alpha));
    Nf_m(:,:,i)   = reshape(nf_m(:,:,i),xobs*pobs,1)*ones(1,nobs);
    profit_adjf(:,:,i) = pmat.*zgrid(i).*xmat.*(Nf_m(:,:,i).^alpha)  -  W*Nf_m(:,:,i)  - firingcost(i)*(nmat-Nf_m(:,:,i)).*(nmat >Nf_m(:,:,i));
    
end


%% Start value iteration
v_adjh = profit_adjh; 
v_adjf = profit_adjf; % pobs blocs of xobs by nobs by zobs. 
v_no   = profit_no;   % pobs blocs of xobs by nobs by zobs.
V_adjh = v_adjh.*(Nh_m >nmat) - 10e4*(Nh_m <=nmat);
V_adjf = v_adjf.*(Nf_m <nmat) - 10e4*(Nf_m >=nmat);

v0 = zeros(xobs*pobs,nobs,zobs);
parfor i = 1:zobs;
    v0z     = max( [reshape(V_adjf(:,:,i),xobs*pobs*nobs,1), reshape(v_no(:,:,i),xobs*pobs*nobs,1), reshape(V_adjh(:,:,i),xobs*pobs*nobs,1)], [],2 );
    v0(:,:,i)     = reshape( v0z, xobs*pobs,nobs );
end

nh     = nh_m;
nf     = nf_m;
nh0    = nh;
nf0    = nf;

profith = zeros(xobs*pobs,nobsc);
profitf = zeros(xobs*pobs,nobsc);
% For use in optimization,
parfor i = 1:zobs;
    profith(:,:,i)    = pmatc.*zgrid(i).*xmatc.*(nmatc.^alpha)  -  W*nmatc  -hiringcost(i)*nmatc;
    profitf(:,:,i)    = pmatc.*zgrid(i).*xmatc.*(nmatc.^alpha)  -  W*nmatc  +firingcost(i)*nmatc;
end

EVp = zeros(xobs*pobs,nobs,zobs);
v   = zeros(xobs*pobs,nobs,zobs);
iter   = 0;
ptol   = 0.005;
perrcount  = 0;
while perrcount <5
    parfor i = 1:zobs
        %% Iterate forward: for given v, compute E[v].
        EV     = evfunlinear(nobs,xobs,xgrid,ngrid,xweights,  pobs,pgrid,Ptrans,  nf(:,:,i),nh(:,:,i),v_no(:,:,i),v_adjh(:,:,i),v_adjf(:,:,i),  mux,rhox,Stdev_epsx); % pobs blocs of xobs by nobs.
        EVi    = spline(ngrid,EV,ngridc);
        %% Update:
        vtomaxh = profith(:,:,i)   + beta*EVi;
        vtomaxf = profitf(:,:,i)   + beta*EVi;
        % HIRING:
        [vmaxh_, pos]= max(vtomaxh,[],2);
        nnewh = ngridc(pos);
        vmaxh = vmaxh_*ones(1,nobs) + ones(xobs*pobs,1)*hiringcost(i)*ngrid'; %add back in adj cost
        nh(:,:,i)    = reshape( nnewh,xobs,pobs);
        % FIRING
        [vmaxf_, pos]= max(vtomaxf,[],2);
        nnewf = ngridc(pos);
        vmaxf = vmaxf_*ones(1,nobs) - ones(xobs*pobs,1)*firingcost(i)*ngrid';
        nf(:,:,i)    = reshape( nnewf,xobs,pobs);
              
        v_no(:,:,i)   = profit_no(:,:,i) + beta*EV;
        v_adjh(:,:,i) = vmaxh;
        v_adjf(:,:,i) = vmaxf;
        Nh(:,:,i)     = nnewh*ones(1,nobs);
        Nf(:,:,i)     = nnewf*ones(1,nobs);
        V_adjh = v_adjh(:,:,i).*(Nh(:,:,i) >nmat) - 10e6*(Nh(:,:,i) <=nmat);
        V_adjf = v_adjf(:,:,i).*(Nf(:,:,i) <nmat) - 10e6*(Nf(:,:,i) >=nmat);
        vz      = max( [reshape(V_adjf,xobs*pobs*nobs,1), reshape(v_no(:,:,i),xobs*pobs*nobs,1), reshape(V_adjh,xobs*pobs*nobs,1)], [],2 );
        v(:,:,i)      = reshape( vz, xobs*pobs,nobs );
        EVp(:,:,i)    = EV;
    end
    
    % Compare:
    iter   = iter +1;
    perr_h = max(max( max(abs(nh-nh0)) ));
    perr_f = max(max( max(abs(nf-nf0)) ));
    perr   = max( [perr_h,perr_f] );
    verr   = max(max(max(abs( v-v0))));
    disp('Iteration,  dV,    max{dhiring,dfiring}');
    disp( [iter,      verr,  perr] );
    perrcount = (perr <ptol).*(perrcount +1);
    nh0 = nh;
    nf0 = nf;
    v0  = v;

end

%Turn EVp into pobs,xobs,nobs,zobs
EVp = reshape(EVp,[xobs pobs nobs zobs]);
EVp = permute(EVp,[2 1 3 4]);



