function [nh,nf, v_no,v_adjh,v_adjf,EVss] = policylineardiscretess(alpha,beta,  xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights,  eta,Omega0,prodconst,HireCost0, FireCost0,zgrid,zobs)



%% Initialize: Guess static policy rules.
 % Some parameters you will want in order to initialize:
alpha2 = alpha*(1-eta)/(1-eta*(1-alpha));
ch0    = (1-beta*(1-.025))*HireCost0; 
cf0    = (1-beta*(1-.025))*FireCost0; 

%Fine choice grid 
ngridc = (1:1:ngrid(end))';
nobsc = length(ngridc);
xmatc   = xgrid * ones(1,nobsc);
nmatc   = ones(xobs,1) * ngridc';   

xmat   = xgrid * ones(1,nobs);
nmat   = ones(xobs,1) * ngrid';

profit_no = zeros(xobs,nobs,zobs);
profit_adjh = zeros(xobs,nobs,zobs);
profit_adjf = zeros(xobs,nobs,zobs);
nh_m      = zeros(xobs,zobs);
Nh_m      = zeros(xobs,nobs,zobs);
nf_m      = zeros(xobs,zobs);
Nf_m      = zeros(xobs,nobs,zobs);
% Don't adjust:
parfor i = 1:zobs
        
    profit_no(:,:,i)  = prodconst*zgrid(i).*xmat.*(nmat.^alpha)  -  Omega0*nmat;  

    % Adjust UP:
    nh_m(:,i)   = (alpha2*xgrid./(Omega0+ch0(i))).^(1/(1-alpha));
    % To mimic solution under foresight, use annuitized hiring cost.
    % Almost as if you solved static problem scaled by (1/(1-beta*(1-attrition rate)))
    Nh_m(:,:,i)   = nh_m(:,i)*ones(1,nobs);
    profit_adjh(:,:,i) = prodconst*zgrid(i).*xmat.*(Nh_m(:,:,i).^alpha) -  Omega0*Nh_m(:,:,i)  - HireCost0(i)*(Nh_m(:,:,i)-nmat).*(Nh_m(:,:,i) >nmat);
    
    % Adjust DOWN:
    nf_m(:,i)   = (alpha2*xgrid/Omega0-cf0(i)).^(1/(1-alpha));    % likewise, use annuitized firing cost.
    Nf_m(:,:,i)   = nf_m(:,i)*ones(1,nobs);
    profit_adjf(:,:,i) = prodconst*zgrid(i).*xmat.*(Nf_m(:,:,i).^alpha) -  Omega0*Nf_m(:,:,i)  - FireCost0(i)*(nmat-Nf_m(:,:,i)).*(Nf_m(:,:,i) <nmat);
    
end

%% Start value iteration
v_adjh = profit_adjh; 
v_adjf = profit_adjf; 
v_no   = profit_no;   
v0 = zeros(xobs,nobs,zobs);

parfor i = 1:zobs
    V_adjh = v_adjh(:,:,i).*(Nh_m(:,:,i) >nmat) - 10e4*(Nh_m(:,:,i) <=nmat);
    V_adjf = v_adjf(:,:,i).*(Nf_m(:,:,i) <nmat) - 10e4*(Nf_m(:,:,i) >=nmat);
    v0z     = max( [reshape(V_adjf,xobs*nobs,1), reshape(v_no(:,:,i),xobs*nobs,1), reshape(V_adjh,xobs*nobs,1)], [],2 );
    v0(:,:,i)     = reshape( v0z, xobs,nobs );
end

nh     = nh_m; % initial policies are myopic ("m") policies
nf     = nf_m;
nh0    = nh;
nf0    = nf;
Nh    = Nh_m;
Nf    = Nf_m;
EVss  = zeros(xobs,nobs,zobs);
v     = zeros(xobs,nobs,zobs);
% For use in optimization,
profith = zeros(xobs,nobsc,zobs);
profitf = zeros(xobs,nobsc,zobs);
parfor i = 1:zobs
    profith(:,:,i)    = prodconst*zgrid(i)*xmatc.*(nmatc.^alpha)  -  Omega0*nmatc  - HireCost0(i)*nmatc;
    profitf(:,:,i)    = prodconst*zgrid(i)*xmatc.*(nmatc.^alpha)  -  Omega0*nmatc  + FireCost0(i)*nmatc;
end

iter   = 0;
ptol   = 0.005;
perrcount  = 0;
while perrcount <5 % will stop when max policy error is < "ptol" for "perrcount" number of times.
    
    parfor i = 1:zobs
    %% Iterate forward: for given v, compute E[v].
    EV     = evfunlinearss(nobs,xobs,xgrid,ngrid,xweights,  nf(:,i),nh(:,i),v_no(:,:,i),v_adjh(:,:,i),v_adjf(:,:,i),  mux,rhox,Stdev_epsx);
    EVi     = spline(ngrid,EV,ngridc);
    %% Update:
    vtomaxh = profith(:,:,i)   + beta*EVi;
    vtomaxf = profitf(:,:,i)   + beta*EVi;
     % HIRING:
    [vmaxh , pos]= max(vtomaxh,[],2);  
    nh(:,i)     = ngridc(pos);
    v_adjh(:,:,i) = vmaxh*ones(1,nobs) + HireCost0(i)*ones(xobs,1)*ngrid'; %add back in adj cost 
     % FIRING
    [vmaxf, pos]= max(vtomaxf,[],2);  
    nf(:,i)     = ngridc(pos);
    v_adjf(:,:,i) = vmaxf*ones(1,nobs) - FireCost0(i)*ones(xobs,1)*ngrid'; 
    
     % Reset:
    v_no(:,:,i)   = profit_no(:,:,i) + beta*EV;
    Nh(:,:,i)     = nh(:,i)*ones(1,nobs);
    Nf(:,:,i)     = nf(:,i)*ones(1,nobs);
    V_adjh = v_adjh(:,:,i).*(Nh(:,:,i) >nmat) - 10e6*(Nh(:,:,i) <=nmat);
    V_adjf = v_adjf(:,:,i).*(Nf(:,:,i) <nmat) - 10e6*(Nf(:,:,i) >=nmat);
    vz      = max( [reshape(V_adjf,xobs*nobs,1), reshape(v_no(:,:,i),xobs*nobs,1), reshape(V_adjh,xobs*nobs,1)], [],2 );
    v(:,:,i)      = reshape( vz, xobs,nobs );
    EVss(:,:,i) = EV;
    
    end
    % Compare:
    iter   = iter +1;
    verr   = max(max(max(abs( v-v0))));
    perr_h = max(max( max(abs(nh-nh0)) ));
    perr_f = max(max( max(abs(nf-nf0)) ));
    perr   = max( [ perr_h,perr_f] );
    disp('Iteration,  dV,    max{dhiring,dfiring}');
    disp( [iter,      verr,  perr] );
    perrcount = (perr <ptol).*(perrcount +1);
    nh0 = nh;
    nf0 = nf;
    v0  = v;
end

