function EV = evks(nobs,xobs,xgrid,xweights,  pobs,pgrid,Ptrans,  v_no,v_adj,  mux,rhox,Stdev_epsx,  Nvec,Nobs,OLSp,OLSlag,OLScons)



 % Transition matrix (for below)
xnext  = ones(xobs,1)*xgrid';
xnow   = xgrid       *ones(1,xobs);
elogx  = mux + rhox*log(xnow);
Fxmax  = normcdf((log(xgrid(xobs))-elogx)/Stdev_epsx);
Fxmin  = normcdf((log(xgrid(1   ))-elogx)/Stdev_epsx);
pdfx   = (1./(xnext*Stdev_epsx*sqrt(2*pi))) .* exp(-0.5*( ((log(xnext)   -elogx)/Stdev_epsx).^2 ));
pdfx   = pdfx ./ (Fxmax -Fxmin);
xwght  = ones(xobs,1)*xweights';
Pdfx   = xwght .* pdfx;
Pdfx   = Pdfx ./ repmat(sum(Pdfx,2), 1, xobs);


 % Reshape value functions -
vMAT_no = reshape(v_no,  xobs,pobs,Nobs, nobs);
vMAT_adj= reshape(v_adj, xobs,pobs,Nobs);

 % Grids -
NVEC = kron(Nvec, ones(pobs,1));
pVEC = repmat(pgrid, Nobs, 1);


tol    = 1e-8;
EV     = zeros(xobs*pobs*Nobs, nobs);
parfor h  = 1:nobs %n(t)
    Ev = zeros( xobs, pobs*Nobs);
    
    for i = 1:xobs %x(t+1)
        vi_no  = reshape(vMAT_no( i,:,:,h), pobs*Nobs, 1); % a function of p(t+1) and N(t+1)
        vi_adj = reshape(vMAT_adj(i,:,:),   pobs*Nobs, 1);
        V      = max( [vi_no,vi_adj],[],2 );  
         
         % Interpolate forward value function, V, at N'(p', N)
        Nprime = exp( OLScons + OLSlag*log(NVEC) + OLSp*log(pVEC) ); 
        dN     = Nvec*ones(1,pobs*Nobs) -ones(Nobs,1)*Nprime';	% increasing along rows, starting negative and turning positive.
        [~,pos2] = max( dN.*(dN<=0) );  % pos2 will be first where dN >0
        pos2 = max( 2,pos2');
        pos1 = pos2 -1;
        Nlo  = Nvec( pos1);
        Nhi  = Nvec( pos2);
        vfit = V(pos1) + ((Nprime-Nlo)./(Nhi-Nlo +tol)).*( V(pos2) - V(pos1) );
        Vfit = reshape(vfit, pobs, Nobs);
         
         % Integrate over p'
        ev      = Ptrans*Vfit; %pobs by Nobs;
        ev_     = reshape(ev, pobs*Nobs, 1);
        Ev(i,:) = ev_';
    end
     % Integrate over x'
    ev      = Pdfx*Ev; % xobs by pobs*Nobs
    EV(:,h) = reshape( ev, xobs*pobs*Nobs, 1);
end
        

        
        