clear; 
delete(gcp('nocreate'))
mypool = parpool(4);
format long g



%% CALIBRATION
%%%%%%%%%%%%%%
beta  = 0.99;
alpha = 0.64;
En    = 20;	     % set mean (desired) establishment size
delta = 0;       % attrition rate
c     = 0.013;   % fixed cost as share of frictionless revenue
% NOTE: c = 1.3% if inaction = 52.5%
%       c = 2.7% if inaction = 67%
%       c = 5.8% if inaction = 80%
frisch= 0.5;


%% IDIOSYNCRATIC PRODUCTIVITY SHOCK
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rhox       = 0.70; 
Stdev_epsx = 0.15;	 
%Unconditional moments:
E_x        = 1;
Stdev_logx = Stdev_epsx/sqrt(1-rhox^2);
Var_logx   = Stdev_logx^2;
E_logx     = log(E_x) - 0.5*Var_logx;
mux        = (1-rhox)*E_logx;
Stdev_x    = sqrt( (exp(Var_logx)-1)*exp(2*E_logx + Var_logx) );
%Grid: Legendre nodes
Fxmax   = 0.9999;
Fxmin   = 0.0001;
x1      = exp( E_logx + Stdev_logx*norminv(Fxmin) );
xN      = exp( E_logx + Stdev_logx*norminv(Fxmax) );
xobs    = 90;
[xgrid, xweights] = legendre(x1, xN, xobs);
pdfx    = (1./(xgrid*Stdev_logx*sqrt(2*pi))) .* exp(-0.5*( ((log(xgrid) - E_logx)/Stdev_logx).^2 ));
pdfx	= pdfx / (Fxmax-Fxmin);
xwghts	= xweights.*pdfx;



%% AGGREGATE PRODUCTIVITY SHOCK
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rhop       = 0.95;
E_p        = 1;
Var_logp   = 0.003208; % = (0.024*(1+(1/frisch)-alpha))^2; induces observed var(log(N)) in frictionless model 
Stdev_logp = sqrt(Var_logp);
Stdev_epsp = Stdev_logp*sqrt(1-rhop^2);
E_logp     = log(E_p) - 0.5*Var_logp;
mup        = (1-rhop)*E_logp;
Stdev_p    = sqrt( (exp(Var_logp)-1)*exp(2*E_logp + Var_logp) );
Var_epsp   = Stdev_epsp^2;

% Productivity grid: Rouwenhorst transition matrix
pobs   = 11;
logp0  = -sqrt(pobs-1)*Stdev_logp;
logpT  =  sqrt(pobs-1)*Stdev_logp;
logpgrid_ = (logp0 : (logpT-logp0)/(pobs-1) : logpT)';
logpgrid  = logpgrid_+ E_logp; 
pgrid  = exp(logpgrid);
prob1  = (1+rhop)/2;
prob2  = (1+rhop)/2;
Ptrans = [ prob1, 1-prob1; 1-prob2, prob2];
for j  = 1:pobs-2
    Np = j+2;
    Ptrans = prob1*[Ptrans, zeros(Np-1,1); zeros(1,Np-1), 0]  + ...
            (1-prob1)* [zeros(Np-1,1) Ptrans; 0 zeros(1,Np-1)] + ...
            (1-prob2)* [zeros(1,Np-1) 0; Ptrans zeros(Np-1,1)] + ...
            prob2*[0 zeros(1,Np-1); zeros(Np-1,1) Ptrans];
    Ptrans(2:end-1,:) = Ptrans(2:end-1,:)/2; 
end



%% Discrete Employment grid
%  calculate E[ x^(1/(1-alpha)) ]:
Ex_a   = exp((1/(1-alpha))*E_logx  + 0.5*(((1/(1-alpha))*Stdev_logx)^2));
Ex_a   = Ex_a * ((normcdf((log(xgrid(end))-E_logx-(1/(1-alpha))*Var_logx)/Stdev_logx) - ...
                  normcdf((log(xgrid(1  ))-E_logx-(1/(1-alpha))*Var_logx)/Stdev_logx)) / (Fxmax-Fxmin));
%  set wage so that average employment in frictionless model = En (this
%  does account for Jensen effects):
W      = alpha * ((Ex_a/En)^(1-alpha));

%  We need to span optimal choice, conditional on adjusting. 
%  Again, use frictionless choices as guide and scale up
%  and down from there:
nmin   = 1; 
nmax   = ceil( ((alpha*xgrid(xobs)*pgrid(pobs)/W)^(1/(1-alpha))) ); 
ngrid  = (nmin :1: nmax)';
nobs   = length(ngrid);



    
%% Full steady state (p==1)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Policy
[nreset, L,U,v_no,v_adj, vtomax] = policyss(alpha,delta,beta,c,W,  xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights);
splL = spline(ngrid,L);
splU = spline(ngrid,U);
    
% Sims
rng(508);
N     = 500000;
xlag  = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(E_x) + Stdev_epsx*randn(N,1) )));
nlag  = ceil( (alpha*xlag/W).^(1/(1-alpha)) ); % round up to make integer
T0 = 75;
Nsim  = zeros( T0,1 );
PrNOadj  = zeros( T0,1 );
for t = 1:T0
    %% Draw IDIOSYNCRATIC productivity
    xt    = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(xlag) + Stdev_epsx*randn(N,1) )));
    fixedcost = c*((alpha/W)^(alpha/(1-alpha))) *(xt.^(1/(1-alpha)));
   
    %% Policy rules
     % TRIGGERS:
    triggerl = min(xgrid(xobs),max(xgrid(1),  ppval(splL,nlag)));
    triggerh = min(xgrid(xobs),max(xgrid(1),  ppval(splU,nlag)));
    adjustneg= (xt<triggerl);
    adjustpos= (xt>triggerh);
    adjust   = adjustneg + adjustpos;
     % RESET:
     % Recall, V(n_-1,x) = max_n {v(n,x)}.
     % Fit v(n,x) @ x(t). Then optimize 
     % over n.
    vpfit    = spline( xgrid',vtomax', xt'); % should be nobs x N.
    [~,npos] = max( vpfit );
    nadj     = ngrid(npos);
    nsim     = nadj.*adjust + nlag.*(1-adjust);
    
    %% Moments:
    Nsim(t)    = mean( nsim );
    PrNOadj(t) = mean( (nsim==nlag ) );
    
     % reset:
    xlag  = xt;
    nlag  = nsim;
    if t/5==floor(t/5); disp(t); end
end
t0 = 25;
Nss = mean(Nsim(end-t0:end));  % Use last 25 periods.
PrNoadjss = mean(PrNOadj(end-t0:end));
disp('Prob of not adjusting'); disp(PrNoadjss);
xss = xlag;
nss = nlag;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%







%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% OUT OF STEADY STATE ANALYSIS

 % Set Krusell-Smith loop or 
 %     Impulse responses or
 %     both:
ks = 0;
irf= 1;


if ks == 1
    %% Krusell-Smith Loop
     % Aggregate Labor supply 
    Nl   = 0.9*Nss;
    Nr   = 1.1*Nss;
    Nobs = 12;
    Nvec = (Nl  : (Nss-Nl)/(floor(Nobs*0.5)-1) : Nss)'; % Ensures Nss is part of grid.
    Nvec = [Nvec(1:end-1);  ...
           (Nss : (Nr-Nss)/floor(Nobs*0.5) : Nr)']; 
    Wage = W*((Nvec/Nss).^(1/frisch));

     % Conjecture coefficients (these are nearly equil. values in 52.5 pct inaction case)
    OLSp   = 0.4170517; 
    OLSlag = 0.0128178; 
    OLScons= log(Nss)*(1-OLSlag);
    KScoef = [OLScons; OLSp; OLSlag];

    % Path of aggregate TFP:
    rng(694);
    T    = 225;
    psim = zeros(T,1);
    plag = 1;
    for t= 1:T
        psim(t) = max(pgrid(1), min(pgrid(pobs),  exp( mup + rhop*log(plag) + Stdev_epsp*randn)));
        plag    = psim(t);
    end
    % Idiosyncratic TFP
    xlag  = xss;
    xsim  = zeros(N,T);
    for t = 1:T
        xt  = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(xlag) + Stdev_epsx*randn(N,1) )));
        xsim(:,t) = xt;
        xlag      = xt;
    end
    xlag  = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(E_x) + Stdev_epsx*randn(N,1) )));


    KSerr= 1;
    KStol= .001;
    burnin = 25; % discard first burnin draws when run regression.
    iter = 1;
    while KSerr >KStol

        %% Policy Function
        [nreset, L,U,v_no,v_adj, vtomax] = policyks(alpha,delta,beta,c,W,  pobs,pgrid,Ptrans,  xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights, Nvec,Nobs,Wage,OLSp,OLSlag,OLScons);


        %% Simulate out of steady state, iterate on law of motion 
        nlag  = nss;
        Nlag  = Nss;
        Nsim  = zeros( T,1);
        Ysim  = zeros( T,1);
        for t = 1:T
            %% Draw agg productivity
            pt   = psim(t);
            pos2 = find( pgrid >pt,1,'first');
            pos1 = pos2 -1;
            plo  = pgrid( pos1);
            phi  = pgrid( pos2);

            %% Draw IDIOSYNCRATIC productivity
            xt  = xsim(:,t);

            %% Fixed point problem in Aggregate Employment
            [nt, ~] = fixedptN(pt,plo,phi,pos1,pos2,pobs,  Nlag,Nobs,Nvec,  xt,xobs,xgrid,  nlag,nobs,ngrid,  L,U, vtomax);

             % save and reset:
            Nsim(t) = mean(nt);
            Ysim(t) = mean(pt.*xt.*(nt.^alpha));
            xlag  = xt;
            nlag  = nt;
            Nlag  = Nsim(t);
            if t/5==floor(t/5); disp(t); end
        end
        disp('Diagnostic: bounds of N, min and max of sims');
        disp( [Nvec(1),Nvec(Nobs), min(Nsim),max(Nsim)] );

        disp('Std dev of log employment'); disp( std(log(Nsim(burnin+1:T))) );
        disp('Std dev of log output per worker'); disp( std(log(Ysim(burnin+1:T)./Nsim(burnin+1:T))) );


         %% Run Krusell-Smith regression and update coefficients:
        xx = [ones(T-burnin,1), log(psim(burnin+1:T)), log(Nsim(burnin:T-1))];
        yy = log(Nsim(burnin+1:T));
        KScoef_ = (xx'*xx)\(xx'*yy);   
        yresid  = yy - xx*KScoef_;
        R2      = 1 - ((yresid'*yresid)/( (yy-mean(yy))'*(yy-mean(yy))));
        KSerr   = max(abs( KScoef-KScoef_));

        KScoef  = KScoef - 0.5*(KScoef-KScoef_);
        OLScons = KScoef(1);
        OLSp    = KScoef(2);
        OLSlag  = KScoef(3);
        disp('KS iteration, error, R^2, and new coefficients');
        disp( [iter, KSerr,R2, KScoef']);
        iter = iter+1;
    end
    
    disp( [Nsim, Ysim] );

    auxn = reshape(nreset, xobs, pobs*Nobs);
    dlmwrite('Lowertrigger.txt', L);
    dlmwrite('Uppertrigger.txt', U);
    dlmwrite('Reset.txt', auxn);
end



if irf ==1
    %% Impulse Response
    
     % Wage (Labor supply) grid:
    Nl   = 0.9*Nss;
    Nr   = 1.1*Nss;
    Nobs = 12;
    Nvec = (Nl  : (Nss-Nl)/(floor(Nobs*0.5)-1) : Nss)'; % Ensures Nss is part of grid.
    Nvec = [Nvec(1:end-1);  ...
           (Nss : (Nr-Nss)/floor(Nobs*0.5) : Nr)']; 
    Wage = W*((Nvec/Nss).^(1/frisch));

     % Krusell-Smith coefficients
     % Inaction = 52.5 pct -->
    OLSp   = 0.417051730276344; 
    OLSlag = 0.012817837382578; 
    OLScons= 2.95226839617543;
     % Inaction = 67 pct -->
%     OLSp   = 0.406391354669990; 
%     OLSlag = 0.036409836543216;
%     OLScons= 2.874063613067060;         
     % Inaction = 80 pct -->
%     OLSp   = 0.3780688853447460;
%     OLSlag = 0.0993610193493026;
%     OLScons= 2.6722516164201100;

     
     
     % Policy Function
    [nreset, L,U,v_no,v_adj, vtomax] = policyks(alpha,delta,beta,c,W,  pobs,pgrid,Ptrans,  xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights, Nvec,Nobs,Wage,OLSp,OLSlag,OLScons);

 
     % Bins (for flow balance calculations)
    nstart = max(ngrid(1),   min(nss)-10);
    nfinal = min(ngrid(nobs),max(nss)+10);
    nbins  = (nstart :1: nfinal)';
    bins   = length(nbins); 
    
    
     % Path of aggregate TFP:
    T    = 20;
    pirf = zeros(T,1);
    epsp = log(1.01) -( E_logp + 0.5*Var_logp  - 0.5*Var_epsp );
    for t= 1:T;  pirf(t)= exp(E_logp + 0.5*Var_logp) * exp( (rhop^(t-1))*epsp  - 0.5*(rhop^(2*(t-1)))*Var_epsp ) ; end


     % Calculate IRFs of aggregate employment
    Sims = 100; 
    burnin = 24; % allow time to get near steady state, then raise TFP.
    
    Nsim      = zeros( T+burnin,Sims); % Actual
    MNsim     = zeros( T+burnin,Sims); % Mandated
    FBNsim    = zeros( T+burnin,Sims); % Flow Balance
    parfor i= 1:Sims
    
        xlag  = xss;
        nlag  = nss;
        Nlag  = Nss;
        for t = 1:T+burnin
            %% Draw agg productivity
            if t<=burnin
                pt = 1;
            else
                tt = t-burnin;
                pt = pirf(tt);
            end
            pos2 = find( pgrid >pt,1,'first');
            pos1 = pos2 -1;
            plo  = pgrid( pos1);
            phi  = pgrid( pos2);

            
            %% Draw IDIOSYNCRATIC productivity
            xt  = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(xlag) + Stdev_epsx*randn(N,1) )));
            
            
            %% Fixed point problem in Aggregate Employment
            [nt, nadj] = fixedptN(pt,plo,phi,pos1,pos2,pobs,  Nlag,Nobs,Nvec,  xt,xobs,xgrid,  nlag,nobs,ngrid,  L,U, vtomax);
            Nsim(t,i)  = mean(nt);
            MNsim(t,i) = mean(nadj); 
            
        
            %% Flow balance:
            FBNsim(t,i) = flowbalance( N,bins,nbins,nt,nlag,nadj); 
        
             % reset
            xlag  = xt;
            nlag  = nt;
            Nlag  = Nsim(t,i);
            %if t/5==floor(t/5); disp( [t,i]); end
        end
    end
    
    ENsim     = mean(Nsim,   2);
    EMNsim    = mean(MNsim,  2);
    EFBNsim   = mean(FBNsim, 2);
    %disp( [ENsim, EMNsim, EFBNsim] );
    ENstar    = (Ex_a^((1-alpha)/(1+(1/frisch)-alpha))) * ( (alpha*pirf*(Nss^(1/frisch))/W).^(1/(1+(1/frisch)-alpha)) );


    N0 = mean(ENsim(burnin-8+1 : burnin));  % Use 2 years prior to TFP increase
    MN0 = mean(EMNsim(burnin-8+1 : burnin));
    FBN0 = mean(EFBNsim(burnin-8+1 : burnin));
    ENstar0  = (Ex_a^((1-alpha)/(1+(1/frisch)-alpha))) * ( (alpha*1*(Nss^(1/frisch))/W)^(1/(1+(1/frisch)-alpha)) );


    disp(' %% IMPULSE RESPONSES %% ');
    disp( 'Actual Employment, Mandated Employment, Flow-balance Employment, Frictionless Employment');
    disp( [(1:T)', log(ENsim(burnin+1 : end)/N0), log(EMNsim(burnin+1 : end)/MN0), log(EFBNsim(burnin+1 : end)/FBN0),  log(ENstar/ENstar0)] );
end


            