function [n,L,U,  v_no,v_adj,  vtomax] = policyks(alpha,delta,beta,c,W,   pobs,pgrid,Ptrans,  xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights, Nvec,Nobs,Wage,OLSp,OLSlag,OLScons)



%% Partial Equ. Myopic rule is initial guess
gridlo   = 0.01;
gridhi   = 2.00;
grid = (gridlo: 0.001*(gridhi-gridlo): gridhi)';
l_m  = zeros(pobs,Nobs);
u_m  = zeros(pobs,Nobs);
for i= 1:Nobs % loop over wage (a function of aggregate labor supply)
    Wi = Wage(i);
    for j= 1:pobs % loop over aggregate productivity
         % Solve value matching ('vmc'):
        fixedcost = c*((alpha/Wi)^(alpha/(1-alpha))) *((pgrid(j)*grid).^(1/(1-alpha)));    
        vmc  = (1-alpha)*((alpha/Wi)^((alpha/(1-alpha))))*((pgrid(j)*grid).^(1/(1-alpha))) - ...
                fixedcost  -  pgrid(j)*grid  +  Wi;
        f    = spline(grid,  vmc);
        g    = unique(fnzeros(f));
        if length(g)>1
            l_m(j)  = g(1);
            u_m(j)  = g(2);
        else
            disp('Error: failure to solve myopic policy');
            pause;
        end
    end
end
L_m    = (ngrid.^(1-alpha))*reshape(l_m,pobs*Nobs,1)';
U_m    = (ngrid.^(1-alpha))*reshape(u_m,pobs*Nobs,1)';
Xgrid1 = repmat(xgrid(1  ),nobs,pobs*Nobs);
XgridN = repmat(xgrid(end),nobs,pobs*Nobs);
L_m = min(XgridN, max( Xgrid1, L_m ));
U_m = max(Xgrid1, min( XgridN, U_m ));



%% Profit of adjusting and not adjusting
%  NOT ADJUSTING:
xmat       = repmat(xgrid,  pobs*Nobs, 1) *ones(1,nobs);
pmat       = repmat( kron(pgrid,ones(xobs,1)), Nobs, 1) * ones(1,nobs);
Wmat       = kron( Wage,ones(xobs*pobs,1)) *ones(1,nobs);
nmat       = repmat(ngrid', xobs*pobs*Nobs,  1);
profit_no  = pmat.*xmat.*(nmat.^alpha)  -  Wmat.*nmat;	% x(t) x p(t) x N(t) by n(t)
profit     = profit_no;

%  ADJUSTING:
xmat_ = repmat(xgrid,  pobs*Nobs, 1);
pmat_ = repmat( kron(pgrid,ones(xobs,1)), Nobs, 1);
Wmat_ = kron( Wage,ones(xobs*pobs,1));
n_m   = (alpha*pmat_.*xmat_./Wmat_).^(1/(1-alpha));     % myopic reset function  
costadjust = c*((alpha./Wmat_).^(alpha/(1-alpha))) .*((pmat_.*xmat_).^(1/(1-alpha)));
profit_adj = pmat_.*xmat_.*(n_m.^alpha)  -  Wmat_.*n_m  - costadjust; % x(t) x p(t) x N(t) by 1



%% Start policy function iteration
 % Initialize:
v_adj  = profit_adj;
v_no   = profit_no;
V_adj  = repmat( v_adj, nobs, 1 );
V_no   = reshape(v_no,  xobs*pobs*Nobs*nobs, 1);
v      = max( [V_no,V_adj], [], 2 );

L0     = L_m; 
U0     = U_m;
n0     = n_m;
L      = L0;
U      = U0;

iter   = 0;
ptol   = 0.001;
perrcount  = 0;
tic
while perrcount <5
    
    %% Expected value function
    EV    = evks(nobs,xobs,xgrid,xweights,  pobs,pgrid,Ptrans,  v_no,v_adj,  mux,rhox,Stdev_epsx,  Nvec,Nobs,OLSp,OLSlag,OLScons);
    
    %% Value of not adjusting:
    v_no  = profit_no  + beta*EV;
    
    %% Optimal reset (if adjusting):
    vtomax	= profit + beta*EV;
    [vmax, posn] = max( vtomax,[],2);
    n       = ngrid(posn);
    pnerr   = max(max(abs(n-n0)));
    n0      = n;
    
    %% Take max over regimes, and update v:
    v_adj    = vmax - costadjust;
    V_adj    = repmat(  v_adj, nobs,1);
    V_no     = reshape( v_no,xobs*pobs*Nobs*nobs,1 );
    V        = max( [V_no,V_adj], [],2 );
    verr     = max(abs(V-v));
    v        = V;
    
    %%  Solve for triggers:
    [L,U]   = triggersks(nobs,xobs,xgrid, pobs, Nobs,  v_no,v_adj);
    L       = reshape(L, nobs, pobs*Nobs);
    U       = reshape(U, nobs, pobs*Nobs);
    pLerr   = max(max(abs(L -L0 )));
    pUerr   = max(max(abs(U -U0 )));
    L0    = L;
    U0    = U;
    
    %% Policy Errors:
    perr      = max( [pnerr,pLerr,pUerr] );  
    perrcount = (perr <ptol)*(perrcount+1); 
    
    
    iter   = iter +1;
    disp('ITERATION'); disp(iter);
    
    disp( 'Policy errors: reset, L, U');   disp( [pnerr,pLerr,pUerr] );
    
    disp( 'Convergence diagnostics: V, policy, and policy count');
    disp( [iter, verr, perr, perrcount] );
end
        
        
        
        