function [n,L,U,  v_no,v_adj,  vtomax] = policyss(alpha,delta,beta,c,W,   xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights)


%% Myopic rule is initial guess. 
%  Solve value matching condition ('vmc') for triggers.
n_m  = (alpha*xgrid/W).^(1/(1-alpha));
gridlo   = 0.01;
gridhi   = 2.00;
grid = (gridlo: 0.001*(gridhi-gridlo): gridhi)';
fixedcost = c*((alpha/W)^(alpha/(1-alpha))) *(grid.^(1/(1-alpha)));
vmc  = (1-alpha)*((alpha/W)^((alpha/(1-alpha))))*(grid.^(1/(1-alpha))) - ...
        fixedcost  -  grid  +  W;
f    = spline(grid,  vmc);
g    = unique(fnzeros(f));
if length(g)>1
    l_m  = g(1);
    u_m  = g(2);
else
    disp('Error: failure to solve myopic policy');
    pause;
end
L_m    = (ngrid.^(1-alpha))*l_m;
U_m    = (ngrid.^(1-alpha))*u_m;
Xgrid1 = repmat(xgrid(1  ),nobs,1);
XgridN = repmat(xgrid(end),nobs,1);
L_m = min(XgridN, max( Xgrid1, L_m ));
U_m = max(Xgrid1, min( XgridN, U_m ));



%% Profit of adjusting and not adjusting
%  ADJUSTING:
fixedcost  = c*((alpha/W)^(alpha/(1-alpha))) *(xgrid.^(1/(1-alpha)));
profit_adj = xgrid.*(n_m.^alpha)  -  W*n_m  - fixedcost;
%  NOT ADJUSTING:
xmat       = repmat(xgrid,  1,    nobs);
nmat       = repmat(ngrid', xobs,    1);
profit_no  = xmat.*(nmat.^alpha)  -  W*nmat;
profit     = profit_no;



%% Start value function iteration
 % Initialize:
v_adj  = profit_adj; %xobs by 1
v_no   = profit_no;  %xobs by nobs
V_adj  = repmat( v_adj,nobs,1 );
V_no   = reshape(v_no, xobs*nobs,1);
v      = max( [V_no,V_adj], [], 2 );
L0     = L_m; 
U0     = U_m;
n0     = n_m;
L      = L0;
U      = U0;

 % Transition matrix (for use below)
xnext  = ones(xobs,1)*xgrid';
xnow   = xgrid       *ones(1,xobs);
elogx  = mux + rhox*log(xnow);
Fxmax  = normcdf((log(xgrid(xobs))-elogx)/Stdev_epsx);
Fxmin  = normcdf((log(xgrid(1   ))-elogx)/Stdev_epsx);
pdfx   = (1./(xnext*Stdev_epsx*sqrt(2*pi))) .* exp(-0.5*( ((log(xnext)   -elogx)/Stdev_epsx).^2 ));
pdfx   = pdfx ./ (Fxmax -Fxmin);
xwght  = ones(xobs,1)*xweights';
Pdfx   = xwght .* pdfx;
Pdfx   = Pdfx ./ repmat(sum(Pdfx,2), 1, xobs);


iter   = 0;
ptol   = 0.001;
perrcount =0;   % counts # of consecutive iterations in which policy errors
                % are small (< ptol).
while perrcount <5
    
    %% Expected value function
    V_adj = v_adj*ones(1,nobs);
    V     = max( V_adj, v_no );
    EV    = Pdfx*V; %integrate over x(t+1).

    
    %% Value of not adjusting
    v_no  = profit_no  + beta*EV;
    
    
    %% Optimal reset policy (if adjusting)
    vtomax = profit + beta*EV;
    [vmax, posn] = max( vtomax,[],2);
    n = ngrid( posn);
    pnerr  = max(max(abs(n-n0)));
    n0     = n;
    
    
    %% Take max over regimes, and update v:
    v_adj    = vmax - fixedcost;
    V_adj    = repmat( v_adj,nobs,1 );
    V_no     = reshape(v_no, xobs*nobs,1 );
    V        = max( [V_no,V_adj], [],2 );
    verr     = max(abs(V-v));
    v        = V;
    
    
    %%  Solve for triggers
    [L,U]   = triggersss(nobs,xobs,xgrid, v_no,v_adj);
    pLerr   = max(max(abs(L -L0 )));
    pUerr   = max(max(abs(U -U0 )));
    L0    = L;
    U0    = U;
    
    %% Policy Errors
    perr      = max( [pnerr,pLerr,pUerr] );  
    perrcount = (perr <ptol)*(perrcount+1); 
    
    
    iter   = iter +1;
    disp('ITERATION'); disp(iter);
    
    disp( 'Policy errors: reset, L, U');   disp( [pnerr,pLerr,pUerr] );
    
    disp( 'Convergence diagnostics: V, policy, and policy count');
    disp( [iter, verr, perr, perrcount] );
    
end
        
        
        
        