function EV = evks(nobs,xobs,xgrid,ngrid,xweights,  pobs,pgrid,Ptrans,  vprime,  mux,rhox,Stdev_epsx, Nvec,Nobs,Ncons,Nlag,Np)


 % Transition matrix (for below)
xnext  = ones(xobs,1)*xgrid';
xnow   = xgrid       *ones(1,xobs);
elogx  = mux + rhox*log(xnow);
Fxmax  = normcdf((log(xgrid(xobs))-elogx)/Stdev_epsx);
Fxmin  = normcdf((log(xgrid(1   ))-elogx)/Stdev_epsx);
pdfx   = (1./(xnext*Stdev_epsx*sqrt(2*pi))) .* exp(-0.5*( ((log(xnext)   -elogx)/Stdev_epsx).^2 ));
pdfx   = pdfx ./ (Fxmax -Fxmin);
xwght  = ones(xobs,1)*xweights';
Pdfx   = xwght .* pdfx;
Pdfx   = Pdfx ./ repmat(sum(Pdfx,2), 1, xobs);


 % Reshape value functions -
Vprime   = reshape(vprime, xobs,pobs,Nobs, nobs);

 % Grids -
NVEC = kron(Nvec, ones(pobs,1));
pVEC = repmat(pgrid, Nobs, 1);


tol    = 1e-8;
EV     = zeros(xobs*pobs*Nobs, nobs);
parfor h  = 1:nobs %n(t)
    
    Ev = zeros( xobs, pobs*Nobs);
    for i = 1:xobs %x(t+1)
        V      = reshape(Vprime(  i,:,:,h), pobs, Nobs); % V(p', N')
         
         % Interpolate forward value function, V, at N'(p', N)
        Nprime = exp( Ncons + Nlag*log(NVEC) + Np*log(pVEC) ); 
        dN     = Nvec*ones(1,pobs*Nobs) -ones(Nobs,1)*Nprime';	% increasing along rows, starting negative and turning positive.
        [~,pos2] = max( dN.*(dN<=0) );  % pos2 will be first where dN >0
        pos2 = max( 2,pos2');
        pos1 = pos2 -1;
        Nlo  = Nvec( pos1);
        Nhi  = Nvec( pos2);
        Vlo  = V( repmat((1:pobs)',Nobs,1) + (pos1-1)*pobs);
        Vhi  = V( repmat((1:pobs)',Nobs,1) + (pos2-1)*pobs);
        vfit = Vlo + ((Nprime-Nlo)./(Nhi-Nlo +tol)).*( Vhi - Vlo );
        Vfit = reshape(vfit, pobs, Nobs);
        
        ev      = Ptrans*Vfit; %pobs by Nobs;
        ev_     = reshape( ev, pobs*Nobs, 1);
        Ev(i,:) = ev_';
    end
    
    ev      = Pdfx*Ev;
    EV(:,h) = reshape( ev,xobs*pobs*Nobs,1);
end
        
