clear;
delete(gcp('nocreate'))
mypool = parpool(4);
format long g




%% CALIBRATION
%%%%%%%%%%%%%%
beta  = 0.99;
alpha = 0.64;
En    = 20;      % set mean (desired) establishment size
delta  = 0;      % attrition rate
frisch = 0.5; 



%% IDIOSYNCRATIC PRODUCTIVITY SHOCK
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rhox       = 0.70; 
Stdev_epsx = 0.15;	 
%Unconditional moments:
E_x        = 1;
Stdev_logx = Stdev_epsx/sqrt(1-rhox^2);
Var_logx   = Stdev_logx^2;
E_logx     = log(E_x) - 0.5*Var_logx;
mux        = (1-rhox)*E_logx;
Stdev_x    = sqrt( (exp(Var_logx)-1)*exp(2*E_logx + Var_logx) );
%Grid: Legendre nodes
Fxmax   = 0.9999;
Fxmin   = 0.0001;
x1      = exp( E_logx + Stdev_logx*norminv(Fxmin) );
xN      = exp( E_logx + Stdev_logx*norminv(Fxmax) );
xobs    = 90;
[xgrid, xweights] = legendre(x1, xN, xobs);
pdfx    = (1./(xgrid*Stdev_logx*sqrt(2*pi))) .* exp(-0.5*( ((log(xgrid) - E_logx)/Stdev_logx).^2 ));
pdfx	= pdfx / (Fxmax-Fxmin);
xwghts	= xweights.*pdfx;



%% AGGREGATE PRODUCTIVITY SHOCK
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rhop       = 0.95;
E_p        = 1;
Var_logp   = 0.003208; % = (0.024*(1+(1/frisch)-alpha))^2; induces observed var(log(N)) in frictionless model 
Stdev_logp = sqrt(Var_logp);
Stdev_epsp = Stdev_logp*sqrt(1-rhop^2);
E_logp     = log(E_p) - 0.5*Var_logp;
mup        = (1-rhop)*E_logp;
Stdev_p    = sqrt( (exp(Var_logp)-1)*exp(2*E_logp + Var_logp) );
Var_epsp   = Stdev_epsp^2;

% Productivity grid: Rouwenhorst transition matrix
pobs   = 11;
logp0  = -sqrt(pobs-1)*Stdev_logp;
logpT  =  sqrt(pobs-1)*Stdev_logp;
logpgrid_ = (logp0 : (logpT-logp0)/(pobs-1) : logpT)';
logpgrid  = logpgrid_+ E_logp; 
pgrid  = exp(logpgrid);
prob1  = (1+rhop)/2;
prob2  = (1+rhop)/2;
Ptrans = [ prob1, 1-prob1; 1-prob2, prob2];
for j  = 1:pobs-2
    Np = j+2;
    Ptrans = prob1*[Ptrans, zeros(Np-1,1); zeros(1,Np-1), 0]  + ...
            (1-prob1)* [zeros(Np-1,1) Ptrans; 0 zeros(1,Np-1)] + ...
            (1-prob2)* [zeros(1,Np-1) 0; Ptrans zeros(Np-1,1)] + ...
            prob2*[0 zeros(1,Np-1); zeros(Np-1,1) Ptrans];
    Ptrans(2:end-1,:) = Ptrans(2:end-1,:)/2; 
end


%% Employment grid
%  calculate E[ x^(1/(1-alpha)) ]:
Ex_a   = exp((1/(1-alpha))*E_logx  + 0.5*(((1/(1-alpha))*Stdev_logx)^2));
Ex_a   = Ex_a * ((normcdf((log(xgrid(end))-E_logx-(1/(1-alpha))*Var_logx)/Stdev_logx) - ...
                  normcdf((log(xgrid(1  ))-E_logx-(1/(1-alpha))*Var_logx)/Stdev_logx)) / (Fxmax-Fxmin));
%  set wage so that average employment in frictionless model = En (this
%  does account for Jensen effects):
W  = alpha * ((Ex_a/En)^(1-alpha));

% Adjustment costs are made proportional to frictionless steady state wage.
% For SYMMETRIC case: hiring cost= firing cost = 
%                     16% of W for inaction = 52.5%
%                     28% of W for inaction = 67%
%                     49% of W for inaction = 80%
% For HIRING COST ONLY case: hiring cost = 
%                     32% of W for inaction = 52.5%
%                     56% of W for inaction = 67%
%                     96% of W for inaction = 80%
% For FIRING COST ONLY case: firing cost = 
%                     32% of W for inaction = 52.5%
%                     56% of W for inaction = 67%
%                     96% of W for inaction = 80%
hiringcost = 0.16*W; 
firingcost = 0.16*W; 

%  We need to span optimal choice, conditional on adjusting. 
%  Again, use frictionless choices as guide and scale up
%  and down from there:
nmin   = 1; 
nmax   = ceil( ((alpha*xgrid(xobs)*pgrid(pobs)/W)^(1/(1-alpha))) ); 
ngrid  = (nmin :1: nmax)';
nobs   = length(ngrid);



%% First, full steady state (p==1)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[ssnh,ssnf, ssv_no,ssv_adjh,ssv_adjf, ssvh,ssvf] = policyss(alpha,beta,  xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights,  W,hiringcost,firingcost);   
splh = spline(xgrid', ssvh');
splf = spline(xgrid', ssvf');
% NOTE: ssvh (ssvf) is value of the firm to be maximized if
% the firm is hiring (firing)


rng( 508);
N     = 250000;
xlag  = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(E_x) + Stdev_epsx*randn(N,1) )));
nlag  = ceil( (alpha*xlag/W).^(1/(1-alpha)) ); % round up to integer.
T0 = 75;
Nsim  = zeros( T0,1);
PrNOadj = zeros( T0,1 );
for t = 1:T0
    %% Draw IDIOSYNCRATIC productivity
    xt    = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(xlag) + Stdev_epsx*randn(N,1) )));
    
    %% Policy rules
     % RESET:
     % Recall, V(n_-1,x) = max_n {v(n,x)}. Fit
     % v(n,x) @ x(t). Then optimize over n.
     % Hiring:
    vpfit    = ppval( splh, xt'); % should be nobs x N.
    [~,npos] = max( vpfit );
    reseth   = ngrid(npos);
    adjustpos= (nlag <reseth);
     % Firing:
    vpfit    = ppval( splf, xt'); % should be nobs x N.
    [~,npos] = max( vpfit );
    resetf   = ngrid(npos);
    adjustneg= (nlag >resetf);
     % New Employment:
    adjust   = adjustneg + adjustpos;
    nsim     = reseth.*adjustpos + resetf.*adjustneg + nlag.*(1-adjust);
    
    %% Moments:
    Nsim(t) = mean( nsim );
    PrNOadj(t) = mean( (nsim==nlag ) );
    
     % reset:
    xlag  = xt;
    nlag = nsim; 
    if t/5==floor(t/5); disp( [t,mean(Nsim(5:t))] ); end
end
t0 = 25;
Nss  = mean(Nsim(end-t0:end)); 
PrNoadjss = mean(PrNOadj(end-t0:end));
disp('Prob of not adjusting'); disp(PrNoadjss);
xss = xlag;
nss = nlag;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% OUT OF STEADY STATE ANALYSIS

 % Set Krusell-Smith loop or 
 %     Impulse responses or
 %     both:
ks = 0;
irf= 1;

if ks == 1
    %% Krusell-Smith Loop
     % Aggregate Labor supply 
    Nl   = 0.9*Nss;
    Nr   = 1.1*Nss;
    Nobs = 12;
    Nvec = (Nl  : (Nss-Nl)/(floor(Nobs*0.5)-1) : Nss)'; % Ensures Nss is part of grid.
    Nvec = [Nvec(1:end-1);  ...
           (Nss : (Nr-Nss)/floor(Nobs*0.5) : Nr)']; 
    Wage = W*((Nvec/Nss).^(1/frisch));

     % Conjecture coefficients (these are close to equil. values in the 52.5 pct inaction case)
    OLSp   = 0.35606;
    OLSlag = 0.19045;
    OLScons= log(Nss)*(1-OLSlag);
    KScoef = [OLScons; OLSp; OLSlag];

    % Path of aggregate TFP:
    rng(694);
    T    = 225;
    psim = zeros(T,1);
    plag = 1;
    for t= 1:T
        psim(t) = max(pgrid(1), min(pgrid(pobs),  exp( mup + rhop*log(plag) + Stdev_epsp*randn)));
        plag    = psim(t);
    end
    % Idiosyncratic TFP
    xlag  = xss;
    xsim  = zeros(N,T);
    for t = 1:T
        xt  = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(xlag) + Stdev_epsx*randn(N,1) )));
        xsim(:,t) = xt;
        xlag      = xt;
    end
    xlag  = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(E_x) + Stdev_epsx*randn(N,1) )));


    KSerr= 1;
    KStol= .001;
    burnin = 25; % discard first burnin draws when run regression.
    iter = 1;
    while KSerr >KStol

        %% Policy rules:
        [nh,nf, v_no,v_adjh,v_adjf, vtomaxh,vtomaxf] = policyks(alpha,beta,W,hiringcost,firingcost,   pobs,pgrid,Ptrans,  xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights, Nvec,Nobs,Wage,OLSp,OLSlag,OLScons);
        nhmat = reshape(nh,xobs,pobs,Nobs);
        nfmat = reshape(nf,xobs,pobs,Nobs);
        
        
        %% Simulate
        nlag  = nss;
        Nlag  = Nss;
        Nsim  = zeros( T,1);
        Ysim  = zeros( T,1);
        for t = 1:T
            %% Draw agg productivity
            pt   = psim(t);
            pos2 = find( pgrid >pt,1,'first');
            pos1 = pos2 -1;
            plo  = pgrid( pos1);
            phi  = pgrid( pos2);

            %% Draw IDIOSYNCRATIC productivity
            xt  = xsim(:,t);

            %% Fixed point problem in Aggregate Employment
            nt  = fixedptN(pt,plo,phi,pos1,pos2,pobs,  Nlag,Nobs,Nvec,  xt,xobs,xgrid,  nlag,nobs,ngrid, vtomaxh, vtomaxf);

             % save and reset:
            Nsim(t) = mean(nt);
            Ysim(t) = mean(pt.*xt.*(nt.^alpha));
            xlag  = xt;
            nlag  = nt;
            Nlag  = Nsim(t);
            if t/5==floor(t/5); disp(t); end
        end
        disp('Diagnostic: bounds of N, min and max of sims');
        disp( [Nvec(1),Nvec(Nobs), min(Nsim),max(Nsim)] );

        
         %% Run Krusell-Smith regression and update coefficients:
        xx = [ones(T-burnin,1), log(psim(burnin+1:T)), log(Nsim(burnin:T-1))];
        yy = log(Nsim(burnin+1:T));
        KScoef_ = (xx'*xx)\(xx'*yy);   
        yresid  = yy - xx*KScoef_;
        R2      = 1 - ((yresid'*yresid)/( (yy-mean(yy))'*(yy-mean(yy))));
        KSerr   = max(abs( KScoef-KScoef_));

        KScoef  = KScoef - 0.5*(KScoef-KScoef_);
        OLScons = KScoef(1);
        OLSp    = KScoef(2);
        OLSlag  = KScoef(3);
        disp('KS iteration, error, R^2, and new coefficients');
        disp( [iter, KSerr,R2, KScoef']);
        iter = iter+1;
    end
    
    disp( [Nsim, Ysim] );

    dlmwrite('firingrule.txt', nf);
    dlmwrite('hiringrule.txt', nh);
end
        


if irf ==1
    %% Impulse Response
    
     % Aggregate Labor supply 
    Nl   = 0.9*Nss;
    Nr   = 1.1*Nss;
    Nobs = 12;
    Nvec = (Nl  : (Nss-Nl)/(floor(Nobs*0.5)-1) : Nss)'; % Ensures Nss is part of grid.
    Nvec = [Nvec(1:end-1);  ...
           (Nss : (Nr-Nss)/floor(Nobs*0.5) : Nr)']; 
    Wage = W*((Nvec/Nss).^(1/frisch));

     % KS coefficients
     % For hiring cost = firing cost --> 52.5 pct inaction:
    OLScons= 2.411687446;
    OLSp   = 0.349555154;
    OLSlag = 0.174100821;
%      % For hiring cost = firing cost --> 67 pct inaction:
%     OLScons= 2.007295720;
%     OLSp   = 0.288948877;
%     OLSlag = 0.308018787;
%      % For hiring cost = firing cost --> 80 pct inaction:
%     OLScons= 1.430682654;
%     OLSp   = 0.199859660;
%     OLSlag = 0.504569795;
%     % For firing cost = 0 and hiring cost --> 52.5 pct inaction:
%     OLScons= 2.408676722;
%     OLSp   = 0.35014295;
%     OLSlag = 0.173908563;
%     % For firing cost = 0 and hiring cost --> 67 pct inaction:
%     OLScons= 2.002957371;
%     OLSp   = 0.289810598;
%     OLSlag = 0.307671021;
%     % For firing cost = 0 and hiring cost --> 80 pct inaction:
%     OLScons= 1.451549670;
%     OLSp   = 0.204980818;
%     OLSlag = 0.495058085;
%     % For hiring cost = 0 and firing cost --> 52.5 pct inaction:
%     OLScons= 2.415322554;
%     OLSp   = 0.349070062;
%     OLSlag = 0.174079955;
%     % For hiring cost = 0 and firing cost --> 67 pct inaction:
%     OLScons= 2.011731302;
%     OLSp   = 0.288113113;
%     OLSlag = 0.308336053;
%     % For hiring cost = 0 and firing cost --> 80 pct inaction:
%     OLScons= 1.4566872216357;
%     OLSp   = 0.2020308165789;
%     OLSlag = 0.4979780079423;


    
     % Policy rules:
    [nh,nf, v_no,v_adjh,v_adjf, vtomaxh,vtomaxf] = policyks(alpha,beta,W,hiringcost,firingcost,   pobs,pgrid,Ptrans,  xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights, Nvec,Nobs,Wage,OLSp,OLSlag,OLScons);
        
    
     % Bins (for flow balance calculations)
    nstart = max(ngrid(1),   min(nss)-10);
    nfinal = min(ngrid(nobs),max(nss)+10);
    nbins  = (nstart :1: nfinal)';
    bins   = length(nbins); 
    
    
     % Path of aggregate TFP:
    T    = 20;
    pirf = zeros(T,1);
    epsp = log(1.01) -( E_logp + 0.5*Var_logp  - 0.5*Var_epsp );
    for t= 1:T;  pirf(t)= exp(E_logp + 0.5*Var_logp) * exp( (rhop^(t-1))*epsp  - 0.5*(rhop^(2*(t-1)))*Var_epsp ) ; end


     % Calculate IRFs of aggregate employment
    Sims = 100; 
    burnin = 24; % allow time to get near steady state, then raise TFP.
    Nsim      = zeros( T+burnin,Sims); % Actual
    FBNsim    = zeros( T+burnin,Sims); % Flow Balance
    parfor i= 1:Sims
    
        xlag  = xss;
        nlag  = nss;
        Nlag  = Nss;
        for t = 1:T+burnin
            %% Draw agg productivity
            if t<=burnin
                pt = 1;
            else
                tt = t-burnin;
                pt = pirf(tt);
            end
            pos2 = find( pgrid >pt,1,'first');
            pos1 = pos2 -1;
            plo  = pgrid( pos1);
            phi  = pgrid( pos2);

            
            %% Draw IDIOSYNCRATIC productivity
            xt  = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(xlag) + Stdev_epsx*randn(N,1) )));
            
            
            %% Fixed point problem in Aggregate Employment
            nt  = fixedptN(pt,plo,phi,pos1,pos2,pobs,  Nlag,Nobs,Nvec,  xt,xobs,xgrid,  nlag,nobs,ngrid, vtomaxh, vtomaxf);
            Nsim(t,i)  = mean(nt);
            
        
            %% Flow balance:
            FBNsim(t,i) = flowbalance( N,bins,nbins,nt,nlag); 
        
             % reset
            xlag  = xt;
            nlag  = nt;
            Nlag  = Nsim(t,i);
            %if t/5==floor(t/5); disp( [t,i]); end
        end
    end
    
    ENsim     = mean(Nsim,   2);
    EFBNsim   = mean(FBNsim, 2);
    %disp( [ENsim, EFBNsim] );
    ENstar    = (Ex_a^((1-alpha)/(1+(1/frisch)-alpha))) * ( (alpha*pirf*(Nss^(1/frisch))/W).^(1/(1+(1/frisch)-alpha)) );

    
    N0 = mean(ENsim(burnin-8+1 : burnin));  % Use 2 years prior to TFP increase
    FBN0 = mean(EFBNsim(burnin-8+1 : burnin));
    ENstar0  = (Ex_a^((1-alpha)/(1+(1/frisch)-alpha))) * ( (alpha*1*(Nss^(1/frisch))/W)^(1/(1+(1/frisch)-alpha)) );

    disp(' %% IMPULSE RESPONSES %% ');
    disp( 'Actual Employment, Flow-balance Employment, Frictionless Employment');
    disp( [(1:T)', log(ENsim(burnin+1 : end)/N0), log(EFBNsim(burnin+1 : end)/FBN0),  log(ENstar/ENstar0)] );
end