function [nh,nf, v_no,v_adjh,v_adjf, vtomaxh,vtomaxf,  Xh,Xf] = policysearch(alpha,beta,   pobs,pgrid,Ptrans,  xobs,xgrid,xweights,mux,rhox,Stdev_epsx,   nobs,ngrid,  eta,omega,prodconst,HiringCost, Nvec,Nobs,NOLSp,NOLSlag,NOLScons, Thvec,Thobs,ThOLSp,ThOLSn,ThOLScons)
    


%% Initialize with static policy
% Don't adjust:
Wmat   = repmat( kron(omega,ones(pobs*xobs,1)), Nobs,1) * ones(1,nobs); %order is x, p, Theta, Agg. Emp on rows
pmat   = repmat( kron(pgrid,ones(xobs,1)),Thobs*Nobs,1) * ones(1,nobs);
xmat   = repmat( xgrid,pobs*Thobs*Nobs,1 ) * ones(1,nobs);
nmat   = ones(xobs*pobs*Thobs*Nobs,1) * ngrid';             
profit_no  = prodconst*pmat.*xmat.*(nmat.^alpha)  -  Wmat.*nmat;  

% Adjust UP:
hiringcost_ = ones(xobs,1)*repmat( kron(HiringCost',ones(1,pobs)), 1,Nobs);
wmat_  = ones(xobs,1)*repmat( kron( omega',ones(1,pobs)), 1,Nobs);
pmat_  = ones(xobs,1)*repmat( pgrid',1,Thobs*Nobs);
xmat_  = xgrid*ones(1,pobs*Thobs*Nobs);
% Static solution is: (Alpha*p*x) *(n^(alpha-1)) =omega + ch;
Alpha  = alpha*(1-eta)/(1-eta*(1-alpha));
ch_    = (1-beta)*hiringcost_;
nh_m   = (Alpha*pmat_.*xmat_./(wmat_+ch_)).^(1/(1-alpha));   
Nh_m   = reshape(nh_m,xobs*pobs*Thobs*Nobs,1)*ones(1,nobs);   
HireCostMat = repmat( kron(HiringCost,ones(xobs*pobs,1)), Nobs,1) * ones(1,nobs);
profit_adjh = prodconst*pmat.*xmat.*(Nh_m.^alpha) -  Wmat.*Nh_m  - HireCostMat.*(Nh_m-nmat).*(Nh_m >nmat); 

% Adjust DOWN:
nf_m   = (Alpha*pmat_.*xmat_./wmat_).^(1/(1-alpha));               
Nf_m   = reshape(nf_m,xobs*pobs*Thobs*Nobs,1)*ones(1,nobs);
profit_adjf = prodconst*pmat.*xmat.*(Nf_m.^alpha) -  Wmat.*Nf_m;   

% For use in optimization,
profith    = prodconst*pmat.*xmat.*(nmat.^alpha)  -  Wmat.*nmat  - HireCostMat.*nmat;
profitf    = prodconst*pmat.*xmat.*(nmat.^alpha)  -  Wmat.*nmat;   




%% Start value iteration
v_adjh = profit_adjh; 
v_adjf = profit_adjf; 
v_no   = profit_no;   
V_adjh = v_adjh.*(Nh_m >nmat) - 10e6*(Nh_m <=nmat);
V_adjf = v_adjf.*(Nf_m <nmat) - 10e6*(Nf_m >=nmat);
v0     = max( [reshape(V_adjf, xobs*pobs*Thobs*Nobs*nobs, 1),  ...
               reshape(v_no,   xobs*pobs*Thobs*Nobs*nobs, 1),  ...
               reshape(V_adjh, xobs*pobs*Thobs*Nobs*nobs, 1)], [],2 );
v0     = reshape( v0, xobs*pobs*Thobs*Nobs,nobs );

nh     = nh_m;
nf     = nf_m;
nh0    = nh;
nf0    = nf;

iter   = 0;
ptol   = 0.005;
perrcount  = 0;
itermax= 150;   % fine tune so very few elements of state space display policy revisions if iter=itermax  
while perrcount <5 && iter <=itermax
    
    %% Iterate forward: for given v, compute E[v].
    EV      = evsearch(nobs,ngrid,  xobs,xgrid,xweights,mux,rhox,Stdev_epsx,  pobs,pgrid,Ptrans,  v0, Nvec,Nobs,NOLScons,NOLSlag,NOLSp, Thvec,Thobs,ThOLScons,ThOLSn,ThOLSp);
    
    
    %% Optimal policy:
    vtomaxh = profith   + beta*EV;
    vtomaxf = profitf   + beta*EV;
     % HIRING:
    [vmaxh_, pos]= max(vtomaxh,[],2);  
    nnewh = ngrid(pos);
    vmaxh = vmaxh_*ones(1,nobs) + repmat( kron(HiringCost,ones(xobs*pobs,1)), Nobs,1)*ngrid'; %add back in lagged n 
    nh    = reshape( nnewh,xobs,pobs*Thobs*Nobs); 
     % FIRING
    [vmaxf_, pos]= max(vtomaxf,[],2);  
    nnewf = ngrid(pos);
    vmaxf = vmaxf_*ones(1,nobs); 
    nf    = reshape( nnewf,xobs,pobs*Thobs*Nobs);
     % INACTION
    v_no  = profit_no + beta*EV;
 
     % Update v:
    v_adjh = vmaxh; 
    v_adjf = vmaxf; 
    Nh     = nnewh*ones(1,nobs);
    Nf     = nnewf*ones(1,nobs);
    V_adjh = v_adjh.*(Nh >nmat) - 10e6*(Nh <=nmat);
    V_adjf = v_adjf.*(Nf <nmat) - 10e6*(Nf >=nmat);
    v      = max( [reshape(V_adjf, xobs*pobs*Thobs*Nobs*nobs, 1), ...
                   reshape(v_no,   xobs*pobs*Thobs*Nobs*nobs, 1), ...
                   reshape(V_adjh, xobs*pobs*Thobs*Nobs*nobs, 1)], [],2 );
    v      = reshape( v, xobs*pobs*Thobs*Nobs,nobs );
    
    % Value and policy errors:
    iter   = iter +1;
    verr   = max(max(abs( v-v0)));
    perr_h = max( max(abs(nh-nh0)) );
    perr_f = max( max(abs(nf-nf0)) );
    perr   = max( [ perr_h,perr_f] );
    perrcount = (perr <ptol).*(perrcount +1);
    disp('Iteration and value and policy errors');
    disp( [iter,  verr,  perr, perrcount] );
    
    % reset:
    if iter <=itermax
        nh0 = nh;
        nf0 = nf;
        v0  = v;    
    end
end
clear nnewh nnewf Nh Nf V_adjh V_adjf v;

if iter >=itermax 
    disp('Max iterations reached on policy problem');
    disp('Share of state space where hire, fire policy differences >0:');
    nhvec  = reshape(nh,  xobs*pobs*Thobs*Nobs, 1);
    nfvec  = reshape(nf,  xobs*pobs*Thobs*Nobs, 1);
    nh0vec = reshape(nh0, xobs*pobs*Thobs*Nobs, 1);
    nf0vec = reshape(nf0, xobs*pobs*Thobs*Nobs, 1);
    posh = find(abs(nhvec - nh0vec) >0);
    posf = find(abs(nfvec - nf0vec) >0);
    disp( [(length(posh)/(xobs*pobs*Thobs*Nobs)),  (length(posf)/(xobs*pobs*Thobs*Nobs))] );
    
    clear nh0 nf0 v0 nhvec nfvec nh0vec nf0vec posh posf;
end
    

%% Recover productivity thresholds
 % Fit value functions in each 'regime' 
 % on a very fine grid of x. Then select hire, fire rules
 % and piece together labor demand decision policy 
 % (from hire, fire, inaction regions).
 
 % If productivity grid is very fine, then first (last) x at which
 % firm opts for inaction rather than fire (hire) is a very good
 % approximation to the fire (hire) threshold. 

maxlongxobs = 2500; % put lid on # of grid points for finer x grid, or else run out of memory.
longxobs = 10000;
xstep = 0;
dx = .01;
while longxobs > maxlongxobs
    xstep    = xstep + dx;
    longx    = (xgrid(1) :xstep: xgrid(xobs))';  
    if longx(end) <xgrid(xobs)
        longx= [longx; ...
                xgrid(xobs)];
    end
    longxobs = length(longx);
end
disp('Increment between grid points for productivity thresholds'); 
disp(xstep);

vtomaxH = reshape(vtomaxh, xobs, pobs*Thobs*Nobs*nobs);
vtomaxF = reshape(vtomaxf, xobs, pobs*Thobs*Nobs*nobs);
vHfit   = spline(xgrid',vtomaxH',longx);
vFfit   = spline(xgrid',vtomaxF',longx);
vhfit   = reshape( vHfit', longxobs*pobs*Thobs*Nobs, nobs);
vffit   = reshape( vFfit', longxobs*pobs*Thobs*Nobs, nobs);
[~,posh] =  max(vhfit, [], 2);
[~,posf] =  max(vffit, [], 2);
nhlong= ngrid(posh);
nflong= ngrid(posf);
nhmat = reshape(nhlong, longxobs, pobs*Thobs*Nobs);
nfmat = reshape(nflong, longxobs, pobs*Thobs*Nobs);


tol= 1e-12;
Xh = zeros(pobs*Thobs*Nobs, nobs);
Xf = zeros(pobs*Thobs*Nobs, nobs);
parfor h = 1:nobs %n(t)
    nt   = ngrid(h);
    
    %% HIRE:
    [~,pos2] = max((nhmat-nt - tol) .* (nhmat<=nt)); % minus tol so pos2 will pick first element where nh >nt (strictly).
    pos2 = pos2';
    pos1 = max(1, pos2-1);
    x1   = longx(pos1);
    x2   = longx(pos2);
    dn   = nhmat - nt;
    dn1  = dn( pos1 + ((1:1:pobs*Thobs*Nobs)' -1)*longxobs );
    dn2  = dn( pos2 + ((1:1:pobs*Thobs*Nobs)' -1)*longxobs );
     % threshold solves 0 = dn1 + ((threshold-x1) /(x2-x1))*( dn2 - dn1 ) -->
    xh   = x1 - ((x2-x1).*(dn1./(dn2-dn1+tol)));
     % NOTE: If nh never reaches nt, then should set pos2=xobs (xh = max{x}).
    xh   = xh.*(dn2 >0) + xgrid(xobs).*(dn2 <=0);
            
    
    %% FIRE:
    [~,pos2] = max((nfmat-nt) .* (nfmat< nt));	% pos2 will be first point where nf >=nt
    pos2 = pos2';
    pos1 = max(1, pos2-1);
    x1   = longx(pos1);
    x2   = longx(pos2);
    dn   = nfmat - nt;
    dn1  = dn( pos1 + ((1:1:pobs*Thobs*Nobs)' -1)*longxobs );
    dn2  = dn( pos2 + ((1:1:pobs*Thobs*Nobs)' -1)*longxobs );
    xf   = x1 - ((x2-x1).*(dn1./(dn2 - dn1 +tol)));
    xf   = xf.*(dn2>=0) + xgrid(xobs).*(dn2 <0); % if dn2 < (strictly) 0, then always fire.

    
    Xh(:,h) = xh;
    Xf(:,h) = xf;
end
Xh = reshape(Xh', nobs, pobs,Thobs,Nobs);
Xf = reshape(Xf', nobs, pobs,Thobs,Nobs);

            
            
            


 