*Create flow balance according to equation 20 of page 25
*Uses data created from QCEW microdata in "Main.sas"

clear all
set more off
cd "path"
insheet using adjustprobdataset.csv, clear

#delimit ;
reshape long totadjustdown totadjustup totadjustin
		totadjust totadjustinup totadjustindown numestabs1 numestabs2 dist1 dist2, i(binlst) j(yrqrt);
rename dist1 h1;
rename dist2 h2;
drop *f*;
sort yrqrt binlst;

*Renormalize the distributions.  This is unnecessary, but an insurance policy;
bys yrqrt: egen sumdist1=total(h1);
bys yrqrt: egen sumdist2=total(h2);
replace h1=h1/sumdist1;
replace h2=h2/sumdist2;
sort yrqrt binlst;
gen H1=.;
by yrqrt: replace H1=h1 if _n==1;
by yrqrt: replace H1=H1[_n-1]+h1 if _n>1;
gen H2=.;
by yrqrt: replace H2=h2 if _n==1;
by yrqrt: replace H2=H2[_n-1]+h2 if _n>1;

*Generate CDF of employment in quarter 2 of each pair;
gen employ=numestabs2*binmid;
bys yrqrt: egen totemp=total(employ);
gen h_emp=employ/totemp;
drop totemp;

sort binlst yrqrt;
by binlst: gen hhat=(h2-h1)/totadjust + h1;
sort yrqrt binlst;
bys yrqrt: egen hhattot=total(hhat);
*Renormalize
replace hhat=hhat/hhattot;

gen avenhat=hhat*binmid;
gen avenact=h2*binmid;

gen phi=totadjust;
gen tau=totadjustin;

collapse (rawsum) avenact avenhat numestabs2 [aw=h2], by(yrqrt);
sort yrqrt;
gen qdate=tq(1990q2)+_n-1;
tsset qdate, q;

gen totalemp_act=avenact*numestabs;
gen totalemp_hat=avenhat*numestabs;

drop *yrqrt;
save Flowbalance, replace;
