*Program to generate quarter to quarter employment changes and distributions;
*Employment above 1000 is deleted;

libname input "path";
libname output "path";

*Number of bins;
%let BinNumMac=351;
*Macro to Bin Data;

%macro quartdistdata(YrBeg=,YrEnd=,BinNum=);


proc delete data=input.totalempdataset;
proc delete data=input.distribution;
proc delete data=input.adjustprobdataset;run;

%do z=&YrBeg %to &YrEnd;

                %do j=1 %to 4;

                        %let yrqrt=&z&j;

                        %if &z=1990 and &j=1 %then %do; %end;
                        %else %if &z=2014 and &j>2 %then %do; %end;
                        %else %do;

                %if &j=1 %then %do;
                        %let yrlst=%eval(&z-1);
                        %let four=4;
                        %let yrqrtlst=&yrlst&four;
                %end;
                %if &j>1 %then %do;
                        %let qrtlst=%eval(&j-1);
                        %let yrqrtlst=&z&qrtlst;
                %end;
%put &yrqrt;
%put &yrqrtlst;
       
data ldb&yrqrt (keep=emp ldb_num); set input.ldb&yrqrt (keep=m3_empl ldb_num own_code succ_break_consol_flag pred_break_consol_flag);
emp=m3_empl;

if own_code~=5 then delete;
if succ_break_consol_flag~=0 or pred_break_consol_flag~=0 then delete;
                                
run;


data ldb&yrqrt; set ldb&yrqrt;
	%bindata;
run;



	%if &z=1990 and &j=2 %then %do;
data ldb&yrqrtlst (keep=emp ldb_num); set input.ldb&yrqrtlst (keep=m3_empl ldb_num own_code succ_break_consol_flag pred_break_consol_flag);
emp=m3_empl;

if own_code~=5 then delete;
if succ_break_consol_flag~=0 or pred_break_consol_flag~=0 then delete;
                                
run;

data ldb&yrqrtlst; set ldb&yrqrtlst;

%bindata;
run;

	%end;

data bindataset;
%do jj=1 %to %eval(&BinNum+1);

        
        if &jj>=1 and &jj<=250 then do;
                        binlst=&jj;binleft=&jj;binright=&jj;
                                binmid=(binleft+binright)/2;
                        output;
        end;

        else if &jj>=251 and &jj<=300 then do;
                        binlst=&jj;binleft=251+5*(%eval(&jj-251));
                        binright=251+5*(%eval(&jj-250))-1;
                        binmid=(binleft+binright)/2;
                        output;
        end;

        else if &jj>=301 and &jj<=350 then do;
                        binlst=&jj;binleft=500+10*(%eval(&jj-301));binright=500+10*(%eval(&jj-300))-1;
                                binmid=(binleft+binright)/2;
                        output;
        end;



%end;

run;


********MERGING DATA TOGETHER;

proc sort data=ldb&yrqrt;by ldb_num;
proc sort data=ldb&yrqrtlst; by ldb_num;

data Merge_&yrqrt;

        merge ldb&yrqrt 
        ldb&yrqrtlst (rename=(emp=emplst bin=binlst));
        by ldb_num;

                if emp>0 & emplst>0 then continuing=1;
                else continuing=0;

		if emp<1000 & emplst<1000 then less1000=1;
		else less1000=0;

proc means data=Merge_&yrqrt noprint nway;
class continuing less1000;
output out=totalemp sum(emp emplst)=totalemp totalemplst;

proc means data=Merge_&yrqrt noprint nway;
class continuing less1000;
output out=samplestats mean(emp)=totalemp;

data samplestats;
set samplestats;

qrt=&j;
yr=&z;

proc append base=input.samplestats data=samplestats;

data totalemp;
set totalemp;

qrt=&j;
yr=&z;

proc append base=input.totalempdataset data=totalemp;

*Get adjustment probabilities;
proc sort data=Merge_&yrqrt; by bin;
proc freq data=Merge_&yrqrt (where=(continuing=1 and less1000=1)) noprint;
tables bin*binlst/out=matrix&yrqrt;
run;
proc sort data=matrix&yrqrt;by bin;

*Get distribution of first quarter;
proc freq data=Merge_&yrqrt (where=(continuing=1 and less1000=1)) noprint;
tables binlst/out=distribution1&yrqrt (rename=(percent=dist1&yrqrt count=numestabs1&yrqrt));
run;

*Get distribution of second quarter;
proc freq data=Merge_&yrqrt (where=(continuing=1 and less1000=1)) noprint;
tables bin/out=distribution2&yrqrt (rename=(bin=binlst percent=dist2&yrqrt count=numestabs2&yrqrt));
run;

data adjustprobs&yrqrt;
set matrix&yrqrt;

%do zz=1 %to %eval(&BinNum+1);

        *Outflows: calculated as fraction that were at bin zz last quarter;
        *and either strictly below, or strictly above zz this quarter;
        /***************************************************************/;
 	if binlst=&zz & bin<&zz then adjustdown&zz=count;
	if binlst=&zz then adjustdownc&zz=count;

        if binlst=&zz & bin>&zz then adjustup&zz=count;
        if binlst=&zz then adjustupc&zz=count;
        /***************************************************************/;
	*Inflows: Calculated as fraction not at bin zz last quarter
		  and at bin zz this quarter, divided by entire mass of establishments;
        if binlst~=&zz & bin=&zz then adjustin&zz=count;
	adjustinc&zz=count;

	*Up;
	if binlst<&zz & bin=&zz then adjustinup&zz=count;
	adjustinupc&zz=count;
	*Down;
	if binlst>&zz & bin=&zz then adjustindown&zz=count;
	adjustindownc&zz=count;
	/****************************************************************/;
	
%end;

run;

proc means data=adjustprobs&yrqrt noprint nway;
output out=adjustprobs&yrqrt sum=;
run;
 
data adjustprobs&yrqrt (keep=binlst totadjustdownf&yrqrt totadjustupf&yrqrt);
set adjustprobs&yrqrt;
%do zzz=1 %to %eval(&BinNum+1);
    
        binlst=&zzz;

        totadjustdownf&yrqrt=adjustdownf&zzz/adjustdownfc&zzz;
        totadjustupf&yrqrt=adjustupf&zzz/adjustupfc&zzz;



        output;
%end;
run;


data adjustprobs&yrqrt;
        set adjustprobs&yrqrt;
        if totadjustdownf&yrqrt=. then totadjustdownf&yrqrt=0;
        if totadjustupf&yrqrt=. then totadjustupf&yrqrt=0;
      
run;

data Merge_&yrqrt;

        merge ldb&yrqrt 
        ldb&yrqrtlst (rename=(emp=emplst bin=binlst));
        by ldb_num;


data distribution&yrqrt;
	merge distribution1&yrqrt distribution2&yrqrt;
	by binlst;
        dist1&yrqrt=dist1&yrqrt/100;
	dist2&yrqrt=dist2&yrqrt/100;
run;

        %if &yrqrt=19902 %then %do;


                data input.adjustprobdataset;
                        merge bindataset adjustprobs&yrqrt distribution&yrqrt;
                        by binlst;
  
                        if dist1&yrqrt=. then dist1&yrqrt=0;
 			if dist2&yrqrt=. then dist2&yrqrt=0;
                run;
                data input.adjustprobdataset;
                        set input.adjustprobdataset;
                       
                       
                        run;
		data input.distribution;
			set distribution&yrqrt;
			run;


        %end;
        %else %do;

                data input.adjustprobdataset;
                        merge input.adjustprobdataset adjustprobs&yrqrt distribution&yrqrt;by binlst;
                   
                       	if dist1&yrqrt=. then dist1&yrqrt=0;
 			if dist2&yrqrt=. then dist2&yrqrt=0;

                data input.adjustprobdataset;
                        set input.adjustprobdataset;
                
		data input.distribution;
			merge input.distribution distribution&yrqrt; by binlst;
			run;      
                
                        run;

%end;

data output.adjustprobdataset;set input.adjustprobdataset;
data output.totalempdataset;set input.totalempdataset;
data output.samplestatistics;set input.samplestats;    
proc delete data=ldb&yrqrtlst; 
run;

data output.adjustprobdataset;
     merge output.adjustprobdataset bindataset;
     by binlst;
run;

PROC EXPORT DATA= output.Adjustprobdataset
            OUTFILE= "/path/AdjustProbDataset.csv" 
            DBMS=CSV REPLACE;
RUN;

PROC EXPORT DATA= output.totalempdataset
            OUTFILE= "/path/TotalEmpDataset.csv" 
            DBMS=CSV REPLACE;
RUN;

PROC EXPORT DATA= output.samplestatistics
            OUTFILE= "/path/samplestatistics.csv" 
            DBMS=CSV REPLACE;
RUN;
%end;
%end;
%end;





%mend;


***************************;
%macro bindata;
%do jj=1 %to &BinNum;

        if &jj>=1 and &jj<=250 then do;

                if emp>=&jj and emp<=&jj then do;
                        bin=&jj;binleft=&jj;binright=&jj;
                end;

        end;
       

        else if &jj>=251 and &jj<=300 then do;

                if emp>=251+5*(%eval(&jj-251)) & emp<251+5*(%eval(&jj-250)) then do;
                        bin=&jj;binleft=251+5*(%eval(&jj-251));
                        binright=251+5*(%eval(&jj-250))-1;

                end;

        end;

        else if &jj>=301 and &jj<=350 then do;

                if emp>=500+10*(%eval(&jj-301)) & emp<500+10*(%eval(&jj-300)) then do;
                        bin=&jj;binleft=500+10*(%eval(&jj-301));binright=500+10*(%eval(&jj-300));
                end;

        end;
	
	else if &jj=351 then do;
	end;
    

%end;
%mend;
**************************;


%quartdistdata(YrBeg=1992,YrEnd=2014,BinNum=&BinNumMac);

*******************Output two datasets--totalempdataset and adjustprobdataset;
PROC EXPORT DATA= output.Adjustprobdataset_int
            OUTFILE= "/path/AdjustProbDataset.csv" 
            DBMS=CSV REPLACE;
RUN;


PROC EXPORT DATA= output.totalempdataset_int
            OUTFILE= "/path/TotalEmpDataset.csv" 
            DBMS=CSV REPLACE;
RUN;


PROC EXPORT DATA= output.samplestatistics 
            OUTFILE= "/path/samplestatistics.csv" 
            DBMS=CSV REPLACE;
RUN;





