%Estimates IRFs from equations 24-25 and used in Figure 10A
clear; clc;

datamat = dlmread('RawData.txt');
emp  = datamat(:,1); %Employment
nfb  = datamat(:,2); %flow balance employment
prod_emp = datamat(:,3); %output per emp

%driving force
df=[prod_emp];
    
%% I) AR(4) in output
output = df(:,1);

p=4;
intercept=1;
y=output(p+1:end,:);
ydm=y-mean(y);

X1=[];
X2=[];
for i=1:p;
    X1=[X1 output(p+1-i:end-i,:)];
    X2=[X2 emp(p+1-i:end-i,:)];
end;
time=[1:length(X2)]';
X=[X1 X2 time time.^2];
%X=[X1 X2];
Xdm=X-repmat(mean(X,1),length(X),1);
[bhat_y,~,~,~,yres,bhat_y_cov]=OLS(ydm,[Xdm]);
ypred=y-yres;


%% II) AR model of (Actual) Employment

y_emp=[emp(2*p:end,:)];
%X1_emp=[ones(length(y_emp),1)];
X1_emp=[];
X2_emp=[];
for i=1:p;
    X1_emp=[X1_emp emp(2*p-i:end-i,:)];
    X2_emp = [X2_emp yres(p-i+1:end-i+1,:)];
end;

%MUST DEMEAN
time=[1:length(X2_emp)]';
X_emp=[X1_emp X2_emp time time.^2];    
%X_emp=[X1_emp X2_emp];
X_emp = X_emp - repmat(mean(X_emp,1),length(X_emp),1);
y_emp = y_emp - mean(y_emp);
[bhat_emp,~,~,~,nres2,bhat_emp_cov]=OLS(y_emp,X_emp);

%% III) AR model of flow balance Employment
y_fb=[nfb(2*p:end,:)];

X1_fb = [];
X2_fb=[];
for i=1:p;
    X1_fb=[X1_fb nfb(2*p-i:end-i,:)];
    X2_fb = [X2_fb yres(p-i+1:end-i+1,:)];
end;
time=[1:length(X2_fb)]';
X_fb=[X1_fb X2_fb time time.^2];    


%MUST DEMEAN
X_fb = X_fb - repmat(mean(X_fb,1),length(X_fb),1);
y_fb = y_fb - mean(y_fb);
[bhat_fb,~,~,~,nfbres2,bhat_fb_cov]=OLS(y_fb,X_fb);

T=25;
irfy=zeros(T+p,1);
irfn=zeros(T+p,1);
irfnfb=zeros(T+p,1);
yeps=zeros(T+p,1);
yeps(p+1,1)=.01;

xvec=[irfn(1:p)' yeps(1:p)'];
xvecfb=[irfnfb(1:p)' yeps(1:p)'];
yvec=[irfy(1:p)'];

for i=p+1:T+p;
        irfnvec=[];irfyvec=[];irfnfbvec=[];
    for j=1:p;
        irfnvec=[irfnvec irfn(i-j)];
        irfnfbvec=[irfnfbvec irfnfb(i-j)];
        irfyvec=[irfyvec irfy(i-j)];
    end;
    for j=1:p;
        irfyvec=[irfyvec irfn(i-j)];
    end;
    
xvec=[irfnvec];
xvecfb=[irfnfbvec];
for j=1:p;    
xvec=[xvec yeps(i-j+1)]; 
xvecfb=[xvecfb yeps(i-j+1)];
end;
xvec=[xvec 0 0];
xvecfb=[xvecfb 0 0];
yvec=[irfyvec 0 0];

irfn(i)=xvec*bhat_emp;
irfnfb(i)=xvecfb*bhat_fb;
irfy(i)= yvec*bhat_y+ yeps(i);
end

irfmat=[irfn(5:16) irfnfb(5:16)];

File=strcat('Stock_IRFs','.txt');
dlmwrite(File,irfmat);
