*Seasonally adjust and create data for Figure 6;
clear
cd "path"

#delimit ;
insheet using "/path/BLS code/samplestatistics.csv", clear;
gen totalemp=numestabs*averageemp;
egen type=group(continuing less1000);
*1 is not contin, greater 1000;
*2 is not contin, less 1000;
*3 is contin, greater 1000;
*4 is contin, less 1000 (our sample);
drop less1000 continuing;
sort year qrt;
egen yrqrt=group(year qrt);
drop year qrt;
drop averageemp;
reshape wide totalemp numestabs, i(yrqrt) j(type);
gen qdate=tq(1990q2)+_n-1;

/*
Create the following variables
totalemp=sum of all employment
totalemp_cont --sum of all emp at contin estabs
totalemp_1000 --sum of all emp at estabs less 1000 (in consec quarters)
totalemp_sample -- sum of all emp at contin and less 1000 estabs;
*/;
gen totalemp=totalemp1+totalemp2+totalemp3+totalemp4;
gen totalemp_cont=totalemp3+totalemp4;
gen totalemp_1000=totalemp2+totalemp4;
gen totalemp_sample=totalemp4;

gen totalest=numestabs1+numestabs2+numestabs3+numestabs4;
gen totalest_cont=numestabs3+numestabs4;
gen totalest_1000=numestabs2+numestabs4;
gen totalest_sample=numestabs4;
drop *1 *2 *3 *4;
drop yrqrt;
tsset qdate, q;

merge 1:1 qdate using "qcew_published", nogen;
rename totestabs_pub totalest_pub;
tsset qdate, q;
sort qdate;
keep if qdate>=tq(1992q1);
local salist="totalemp totalemp_cont totalemp_1000 totalemp_sample totalemp_pub totalest totalest_cont totalest_1000 totalest_sample totalest_pub";

keep qdate `salist';

foreach var of local salist{;
  rename `var' `var'_n;
};
save totalemp_n, replace;

foreach var of local salist{;
  rename `var'_n `var';
};
gen int year = year(dofq(qdate));
gen byte qrt = quarter(dofq(qdate));
cd sa;

file open metafile using metafile.dta, write text replace;
foreach var of local salist {;
  outfile year qrt `var' using `var'.dat, nolabel replace;
  file write metafile _char(034)"`var'.dat"_char(034) _n;
};
file close metafile;
shell x12a-0.3 stocks-q -d metafile;
!cat metafile.log >> seasadj-s.log;
foreach var of local salist {;
  infix 3 first int year 1-4 byte qrt 5-6 double `var' 10-35 using `var'.d11, clear;
  gen int qdate = yq(year,qrt);
  keep qdate `var';
  tsset qdate, q;
  compress;
  save `var'.dta, replace;
  erase `var'.dat;
  erase `var'.err;
  erase `var'.d11;
};
erase metafile.dta;
erase metafile.log;
cd ..;

use totalemp_n, clear;
foreach var of local salist{;
  merge 1:1 qdate using ./sa/`var', nogen;
};

drop *n;
foreach var of local salist{;
    gen l`var'=log(`var');
};

keep qdate ltotalemp_pub ltotalemp ltotalemp_cont ltotalemp_sample;
export excel "Figure6.xlsx", firstrow(variables) replace sheet("Total Employment") datestring("%tq");
