function [hdensity,  fbdensity,nhatsim, fbdensity_,nhatsim_, fbdensity2,nhatsim2, lognhatsim] = flowbalance( N,bins,nbins,nsim,nlag...
    ,reseth,resetf)
 
ht  = zeros( bins,1);
hlag= zeros( bins,1);
adjtoh = zeros( bins,1);
adjtof = zeros( bins,1);
adjfrom = zeros( bins,1);
inflows = zeros( bins,1);
inflowsh = zeros( bins,1);
inflowsf = zeros( bins,1);
fbdensity = zeros( bins,1);
fbdensityh = zeros( bins,1);
fbdensityf = zeros( bins,1);
hmand = zeros( bins,1);
hmandh= zeros( bins,1);
hmandf= zeros( bins,1);
for j = 1:bins
     % h(n) %%%%%%%%%%%%%%
    posn  = find( nsim ==nbins(j));
    if isempty(posn)==1; obs = 0; else obs = length(posn); end
    ht(j) = obs/N;
    
     % Mandated %%%%%%%%%%
   %pos  = find( nbins(j)== reset0);
   %if isempty(pos) ==1; obs = 0; else obs = length(pos);  end
   %hmand(j) = obs /N;
   
     % inflows %%%%%%%%%%%
     % first, share of firms that hire (fire) to bin j
    posh = find( nsim== nbins(j) & nlag< nbins(j)); %hires
    posf = find( nsim== nbins(j) & nlag> nbins(j)); %fires
    inflowsh(j) = length(posh) /N;
    inflowsf(j) = length(posf) /N;
    inflows(j)  = inflowsh(j) + inflowsf(j);
     % second, probability that firm hires (fires) 
     % to bin j GIVEN THAT bin j is optimal.
    mposh = find( nbins(j)== reseth ); %hires
    mposf = find( nbins(j)== resetf ); %fires
    mobsh = length(mposh);
    mobsf = length(mposf);
    if mobsh >0 
        adjtoh(j) = length(posh)/mobsh;
        hmandh(j) = mobsh/N; %memo - since we know share for whom j is optimal, easy to compute mandated density.
    end
    if mobsf >0 
        adjtof(j) = length(posf)/mobsf;
        hmandf(j) = mobsf/N;
    end
    
     % outflows %%%%%%%%%%
    posn = find( nlag ==nbins(j));
    if isempty(posn)==1; obs = 0; else obs = length(posn); end
    hlag(j) = obs/N;
    if isempty(posn)==0
         % if there is mass at n_-1, 
        posl = find( nsim(posn)< nbins(j));
        posr = find( nsim(posn)> nbins(j));
        adjfrom(j) = (length(posl)+length(posr)) /length(posn);
        fbdensityh(j) = inflowsh(j)/adjfrom(j);
        fbdensityf(j) = inflowsf(j)/adjfrom(j);
        fbdensity(j) = inflows(j)/adjfrom(j);
    else
        fbdensity(j) = 0;
    end
end

 % save
hdensity= ht; 
xadjtoh = adjtoh;
xadjtof = adjtof;
xinflows= inflows; %save tau and phi
xadjfrom= adjfrom;

 % This treats zero as "true" holes in the density.
apos    = find( adjfrom >0);
fbdensityh = fbdensityh(apos);
fbdensityf = fbdensityf(apos);
fbdensity = fbdensity(apos);
nhatsimh  = nbins(apos)'*fbdensityh;
nhatsimf  = nbins(apos)'*fbdensityf;
nhatsim   = nbins(apos)'*fbdensity;

 % Next, we treat zeros as if mass were displaced to nearby
 % locations. It aggregates into a bin that surrounds
 % the point(s) where the zeros are. It treats this 
 % bin as a single location. 
 % For instance, if there is mass at n = 49 and 51
 % but no mass at 50, then we create a bin that
 % encompasses n = 49 to 51.
 
 % FIRST, before we start, annihilate zeros at very small n, 
 % very large n.
posdrop1 = find( hlag >0,1,'first');
posdrop2 = find( hlag >0,1,'last' );
if posdrop1 ==1;    posdrop1 = []; end
if posdrop2 ==bins; posdrop2 = []; end
posdrop  = [(1:posdrop1-1)';  (posdrop2+1:bins)'];
nt = nbins;
nt(posdrop,:) =[];
ht(posdrop,:) =[];
hlag(posdrop,:) =[];
adjfrom(posdrop,:) =[];
inflowsh(posdrop,:) =[];
inflowsf(posdrop,:) =[];
ht   = ht/sum(ht);
hlag = hlag/sum(hlag);
 
dn = 1;
dnmax= 5; % calibrate!!
nend = nt(end);

htol = 1e-9; 
nk = nt(1);
k = 1;
clear nt_ ht_ hlag_ adjfrom_ inflows_;
while nk <=nend
    pos1 = find( nk== nt ); 
    if hlag(pos1) >htol
        % There is mass in t-1:
        dn = 1;
    else
        % There is no mass. 
        % Find next n at 
        % which there is
        % mass.
        stop = 0;
        while stop ==0
            dn   = dn + 1;
            pos1 = find( nk<= nt  &  nt <=nk +dn-1 );
            stop = ( (sum(hlag(pos1))> htol) + (nk +dn-1 >nend) + (dn >dnmax) ) *(sum(hlag(pos1))>0);
             % the criteria: if there is mass (the outer
             % indicator), then stop if 1 of 3 conditions are met:
             % if mass in bin exceeds threshold; if n runs off the grid;
             % if forwarded steps exceeds "dnmax"
        end
        % ENCASE ZEROS %
        if hlag(pos1(1)-2) >0
             % this looks back to make sure you didn't
             % replace on the last iteration. (If you
             % don't do this check, you will over-
             % write the last iteration.)
            pos1 = [pos1(1)-1; pos1];
            if k >1;  k= k -1; end
        end
        % Now, pos1(1)= nearest element to left w/ mass, 
        % pos1(end)= nearest element to right w/ mass.
        %%%%%%%%%%%%%%%%
    end
    
    adjfrom_(k)  = hlag(pos1)'*adjfrom(pos1)/sum(hlag(pos1)); 
    inflowsh_(k) = sum( inflowsh(pos1) );
    inflowsf_(k) = sum( inflowsf(pos1) );
    hlag_(k)  = sum(hlag(pos1));
    ht_(k) = sum( ht(pos1));
    if sum(ht(pos1)) >0
        nt_(k) = ht(pos1)'*nt(pos1) /sum(ht(pos1));
    else
        nt_(k) = mean(nt(pos1));
    end 
    nk = nk +dn;
    k  = k+1;
    dn = 1;
end
nt_ = nt_';
ht_ = ht_';
hlag_ = hlag_';
adjfrom_ = adjfrom_';
inflowsh_ = inflowsh_';
inflowsf_ = inflowsf_';
inflows_  = inflowsh_ + inflowsf_; 


 % LAST step: Because of finite sample, prob(adj from n) 
 % may be zero Even if there IS mass
 % there.  In this case, we IMPUTE.
bins_= length(nt_);
k = 1;
while k <= bins_
    if adjfrom_(k) ==0
        if k ==1
            apos = find( adjfrom_ >0,1,'first');
            adjfrom_(k) = adjfrom_(apos);
            k = apos;
        elseif k ==bins_
            apos = find( adjfrom_ >0,1,'last');
            adjfrom_(k) = adjfrom_(apos);
            k = k + 1; 
        else
             % catch other adjacent zeros
             % We do this so we can then (see below)
             % interpolate between two points where
             % prob(adj) is >0.
            stop =0;
            kk =0;
            while stop ==0
                stop = (adjfrom_(k+1+kk) >0);
                if stop ==0 && k+1+kk < bins_
                    kk = kk +1;
                elseif stop ==0 && k+1+kk ==bins_
                    stop = 1;
                end
            end
            if k+1+kk ==bins_
                adjfrom_(k:end) = adjfrom_(k-1);
            else
                 % this linearly interpolates the zeros.
                 % (note - if k is the only zero, then kk=0)
                adjfrom_(k:k+kk) = adjfrom_(k-1) + (1/(2+kk))*(1:kk+1)'*(adjfrom_(k+1+kk)-adjfrom_(k-1));
            end
            k = k + 1 +kk;
        end
    else
        k = k + 1;
    end
end
fbdensityh_ = inflowsh_./adjfrom_; 
fbdensityf_ = inflowsf_./adjfrom_; 
fbdensity_ = inflows_./adjfrom_; 
nhatsimh_  = nt_'*fbdensityh_;
nhatsimf_  = nt_'*fbdensityf_;
nhatsim_   = nt_'*fbdensity_;
lognhatsim = log(nt_)'*fbdensity_;

 % Last, invert law of motion
fbdensity2 = hlag_ + ((ht_-hlag_)./adjfrom_);
nhatsim2   = nt_'*fbdensity2; 

