clear; clc;
plotflag = 0;
poolobj = gcp('nocreate');
delete(poolobj);
format long g
myPool = parpool(20);


%Load Quad-Detrended data for target
data=dlmread('QuadDetrend.txt');
empdata=data(:,1);
fbdata=data(:,2);
for i=2:length(empdata)-2;
    empsmooth(i,1) = .25*empdata(i-1) + .25*empdata(i) + .25*empdata(i+1) ...
        +.25*empdata(i+2);
    
    fbsmooth(i,1) = .25*fbdata(i-1) + .25*fbdata(i)  +.25*fbdata(i+1) ...
        + .25*fbdata(i+2);
end;
fbsmooth=fbsmooth(2:end,1);
empsmooth=empsmooth(2:end,1);
Tmatch=length(empsmooth);


%% CALIBRATION
%%%%%%%%%%%%%%
beta  = 0.99;
alpha = 0.64;
En    = 20;	     % set mean (desired) establishment size
delta  = 0;      % attrition rate

%% IDIOSYNCRATIC PRODUCTIVITY SHOCK
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rhox       = 0.70;
Stdev_epsx = 0.15;
%Unconditional moments:
E_x        = 1;
Stdev_logx = Stdev_epsx/sqrt(1-rhox^2);
Var_logx   = Stdev_logx^2;
E_logx     = log(E_x) - 0.5*Var_logx;
mux        = (1-rhox)*E_logx;
Stdev_x    = sqrt( (exp(Var_logx)-1)*exp(2*E_logx + Var_logx) );
Skewn_x    = (exp(Var_logx)+2) * sqrt(exp(Var_logx)-1);
%Conditional moments (recognizing the bounds on x):
Fxmax    = 0.9999;
Fxmin    = 0.0001;
x1       = exp( E_logx + Stdev_logx*norminv(Fxmin) );
xN       = exp( E_logx + Stdev_logx*norminv(Fxmax) );
e_x      = exp( E_logx + 0.5*Var_logx )*((normcdf((log(xN)-E_logx-  Var_logx)/Stdev_logx)-normcdf((log(x1)-E_logx-  Var_logx)/Stdev_logx))/(Fxmax-Fxmin));
var_x    = exp( 2*(E_logx + Var_logx) )*((normcdf((log(xN)-E_logx-2*Var_logx)/Stdev_logx)-normcdf((log(x1)-E_logx-2*Var_logx)/Stdev_logx))/(Fxmax-Fxmin)) ...
    +  E_x^2  -  2*E_x*e_x;
stdev_x  = sqrt(var_x);
skewn_x  = exp(3*E_logx + 4.5*Var_logx)*((normcdf((log(xN)-E_logx-3*Var_logx)/Stdev_logx)-normcdf((log(x1)-E_logx-3*Var_logx)/Stdev_logx))/(Fxmax-Fxmin)) ...
    - 3*E_x*(var_x - (E_x^2) + 2*E_x*e_x)  +  3*(E_x^2)*e_x  - (E_x^3);
skewn_x  = skewn_x / (Stdev_x^3);
% Productivity grid: Legendre nodes
xobs    = 90;
[xgrid, xweights] = legendre(x1, xN, xobs);
pdfx    = (1./(xgrid*Stdev_logx*sqrt(2*pi))) .* exp(-0.5*( ((log(xgrid) - E_logx)/Stdev_logx).^2 ));
pdfx	= pdfx / (Fxmax-Fxmin);
xwghts	= xweights.*pdfx;

rhop=.98;
Var_logp   = .00065;
iter=1;
errz=1;
load('FminOut','x');
linearc = x(2);

while errz>.0001;
    
    
    %% AGGREGATE PRODUCTIVITY SHOCK
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    E_p = 1;
    Stdev_logp = sqrt(Var_logp);
    Stdev_epsp = Stdev_logp*sqrt(1-rhop^2);
    E_logp     = log(E_p) - 0.5*Var_logp;
    mup        = (1-rhop)*E_logp;
    Stdev_p    = sqrt( (exp(Var_logp)-1)*exp(2*E_logp + Var_logp) );
    Var_epsp   = Stdev_epsp^2;
    
    % Productivity grid: Rouwenhorst transition matrix
    pobs   = 11;
    logp0  = -sqrt(pobs-1)*Stdev_logp;
    logpT  =  sqrt(pobs-1)*Stdev_logp;
    logpgrid_ = (logp0 : (logpT-logp0)/(pobs-1) : logpT)';
    logpgrid  = logpgrid_+ E_logp;
    pgrid  = exp(logpgrid);
    prob1  = (1+rhop)/2;
    prob2  = (1+rhop)/2;
    Ptrans = [ prob1, 1-prob1; 1-prob2, prob2];
    for j  = 1:pobs-2
        Np = j+2;
        Ptrans = prob1*[Ptrans, zeros(Np-1,1); zeros(1,Np-1), 0]  + ...
            (1-prob1)* [zeros(Np-1,1) Ptrans; 0 zeros(1,Np-1)] + ...
            (1-prob2)* [zeros(1,Np-1) 0; Ptrans zeros(Np-1,1)] + ...
            prob2*[0 zeros(1,Np-1); zeros(Np-1,1) Ptrans];
        Ptrans(2:end-1,:) = Ptrans(2:end-1,:)/2;
    end
    
    
    
    %% Employment grid
    %  calculate E[ x^(1/(1-alpha)) ]:
    Ex_a   = exp((1/(1-alpha))*E_logx  + 0.5*(((1/(1-alpha))*Stdev_logx)^2));
    Ex_a   = Ex_a * ((normcdf((log(xgrid(end))-E_logx-(1/(1-alpha))*Var_logx)/Stdev_logx) - ...
        normcdf((log(xgrid(1  ))-E_logx-(1/(1-alpha))*Var_logx)/Stdev_logx)) / (Fxmax-Fxmin));
    %  set wage so that average employment in frictionless model = En (this
    %  does account for Jensen effects):
    W  = alpha * ((Ex_a/En)^(1-alpha));
    hiringcost = linearc*W;
    firingcost = linearc*W;
    conc     = 0.0; %constant in front of convex cost
    cone     = 0;    %exponent on convex cost
    %  We need to span optimal choice, conditional on adjusting.
    %  Again, use frictionless choices as guide and scale up
    %  and down from there:
    nmin   = 1; 
    nmax   = ceil( ((alpha*xgrid(xobs)*pgrid(pobs)/W)^(1/(1-alpha))) );
    ngrid  = (nmin :1: nmax)';
    nobs   = length(ngrid);
    
    
    %
    
    %% First, full steady state
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    bargpower = 0;
    threatpoint = W;
    scaleprodfn = 1;
    % Function written so that it can accommodate a SZ wage bargain. To
    % shut off, set "0" for zero bargaining power,  insert W for the outside
    % option, or threat point, and do not scale the production function (set
    % this to 1).
    
    [ssnh,ssnf, ssv_no,ssv_adjh,ssv_adjf, ssvh,ssvf,EVss] = policylinearconvexdiscretess(alpha,beta,  xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights,  bargpower,threatpoint,scaleprodfn, hiringcost,firingcost,conc,cone);
    
    % Simulate invariant (steady-state) distribution of n.
    rng( 508);
    N     = 250000;
    xlag  = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(E_x) + Stdev_epsx*randn(N,1) )));
    nlag  = ceil( (alpha*xlag/(W+conc/2)).^(1/(1-alpha)) ); % round up.
    T0 = 50;
    Nsim  = zeros( T0,1);
    PrNOadj = zeros( T0,1 );
    stdDlogn = zeros(T0,1);
   
    avecost  = zeros(T0,1);
    convcost = zeros(T0,1);
    lincost   = zeros(T0,1);
    En25     = zeros(T0,1);
    En75     = zeros(T0,1);
    %Some matrices
    nmat = repmat(ngrid',[N 1]);
    nmatalpha = nmat.^alpha;
    nlagm = repmat(nlag,[1 nobs]);
    for t = 1:T0
        %% Draw IDIOSYNCRATIC productivity
        xt    = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(xlag) + Stdev_epsx*randn(N,1) )));
        xtm  = repmat(xt,[1 nobs]);
        
        EVt = interp1(xgrid,EVss,xt);
        
        % Hiring:
        vh =  scaleprodfn.*xtm.*nmatalpha  -  threatpoint*nmat  - hiringcost*(nmat - nlagm).*(nmat>nlagm) -(conc)*abs(nmat - nlagm).^cone + beta*EVt;
        [~,npos] = max( vh,[],2 );
        reseth   = ngrid(npos);
        adjustpos= (nlag <reseth);
        % Firing:
        vf =  scaleprodfn*xtm.*nmatalpha  -  threatpoint*nmat  - firingcost*(nlagm-nmat).*(nmat<nlagm) -(conc)*abs(nmat - nlagm).^cone + beta*EVt;
        [~,npos] = max( vf , [],2);
        resetf   = ngrid(npos);
        adjustneg= (nlag >resetf);
        % New Employment:
        adjust   = adjustneg + adjustpos;
        nsim     = reseth.*adjustpos + resetf.*adjustneg + nlag.*(1-adjust);
        
        
        %% Moments:
        Nsim(t) = mean( nsim );
        PrNOadj(t) = mean( (nsim==nlag ) );
        Dlogn     = log(nsim./nlag);
        stdDlogn(t) = std(Dlogn(adjust==1));
        gt   = 200*((nsim-nlag)./(nsim + nlag));
        Dn      = abs(nsim(adjust==1) - nlag(adjust==1));
        avecost(t) = mean(((conc).*Dn.^(cone))./(nsim(adjust==1)-nlag(adjust==1)))/W;
        convcost(t) = sum((conc).*Dn.^(cone));
        lincost(t)  = sum(hiringcost.*(nsim-nlag).*(nsim>nlag)) - sum(firingcost.*(nsim-nlag).*(nsim<nlag));
        En25(t) = prctile(nsim,25);
        En75(t) = prctile(nsim,75);
        
        % reset:
        xlag = xt;
        nlag = nsim;
        nlagm = repmat(nlag,[1 nobs]);
        if t/5==floor(t/5); disp( [t,mean(Nsim(5:t))] ); end
    end
    t0 = 25;
    N0  = mean(Nsim(end-t0:end));
    PrNoadj0 = mean(PrNOadj(end-t0:end));
    disp('Mean employment'); disp( N0);
    disp('Prob of not adjusting'); disp(PrNoadj0);
    xss = xlag;
    nss = nlag;
    
    En25 = mean(En25(end-t0:end));
    En75 = mean(En75(end-t0:end));
    
    nstart = max(ngrid(1),   min(nss)-10);
    nfinal = min(ngrid(nobs),max(nss)+10);
    nbins  = (nstart :1: nfinal)';
    bins   = length(nbins);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
    
    
    %% Policy Function Out of Steady State
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    [nh,nf, v_no,v_adjh,v_adjf, vtomaxh,vtomaxf,EV] = policylinearconvexdiscrete(alpha,beta,W,hiringcost,firingcost,   pobs,pgrid,Ptrans,  xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights,conc,cone);
    EV = reshape(EV,[pobs xobs nobs]);
    
    T    = 100;
    Sims = 1;
    
    %   Path of aggregate TFP:
    if iter==1
        psim = zeros(T,1);
        epsp = Stdev_epsp*randn(T,1);
        for s= 1:Sims
            plag = 1;
            for t= 1:T
                logpsim  = -0.5*Var_logp + rhop*log(plag) + epsp(t,1);
                psim(t,s)= max(pgrid(1), min(pgrid(pobs),exp(logpsim)) );
                plag   = psim(t,1);
            end
        end
    end
    
    %  Path of idiosyncratic TFP:
    xsim  = zeros( T,N);
    for t = 1:T
        xt  = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(xlag) + Stdev_epsx*randn(N,1) )));
        xsim(t,:) = xt';
        xlag= xt;
    end
    clear xt;
    
    Nsim = zeros( T,Sims);
    Ysim  = zeros( T,Sims);
    FBNsim = zeros(T,Sims);

    parfor i = 1:Sims
        % initialize:
        nmat = repmat(ngrid',[N 1]);
        nmatalpha = nmat.^alpha;
        nlag  = nss;
        nlagm = repmat(nlag,[1 nobs]);
        xlag  = xss;
        Nsimi=zeros(T,1);
        Ysimi=zeros(T,1);
        FBNsimv_normi = zeros(T,1);
        for t = 1:T
            %% Draw agg productivity
            pt  = psim(t,1);
            
            %% Draw IDIOSYNCRATIC productivity
          
            xt   = xsim(t,:)';
            xtm  = repmat(xt,[1 nobs]);
            %% Interpolate the EV function:
            pos2 = find( pgrid >pt,1,'first');
            pos1 = pos2 -1;
            plo  = pgrid( pos1);
            phi  = pgrid( pos2);
            
            EVt      = EV(pos1,:,:) + ((pt-plo)/(phi-plo))*( EV(pos2,:,:) -EV(pos1,:,:)+10^-10);
            EVt      = squeeze(EVt);
            %%Iterpolate over xt
            EVt = interp1(xgrid,EVt,xt);
            
            % Hiring:
            % As in steady state sims, interpolate value of the firm given
            % n(t) at simulated values of p and x. Then choose optimal n
            % on discrete grid.
            % Hiring:
            vh =  pt.*xtm.*nmatalpha  -  W*nmat  - hiringcost*(nmat - nlagm).*(nmat>nlagm) -(conc)*abs(nmat - nlagm).^cone + beta*EVt;
            [~,npos] = max( vh,[],2 );
            nhire   = ngrid(npos);
            % Firing:
            vf =  pt*xtm.*nmatalpha  -  W*nmat  - firingcost*(nlagm-nmat).*(nmat<nlagm) -(conc)*abs(nmat - nlagm).^cone + beta*EVt;
            [~,npos] = max( vf , [],2);
            nfire  = ngrid(npos);
            % New Employment:
            nt     = nlag.*(nhire<=nlag).*(nlag<=nfire) + nhire.*(nlag<nhire) + nfire.*(nlag>nfire);
            
            [ht, fbdensity,nhatsim, fbdensity_,nhatsim_, fbdensity2,nhatsim2, lognhatsim] = ...
                flowbalance( N,bins,nbins,nt,nlag,nhire,nfire);
            
            FBNsimv_normi(t,1) = nhatsim2/sum(fbdensity2);
            Nsimi(t,1) = mean(nt);
            Ysimi(t,1) = pt*mean( xt.*(nt.^alpha) );
            
            % reset:
            
            nlag  = nt;
            nlagm = repmat(nlag,[1 nobs]);
            if t/5==floor(t/5); disp( [t,i]); end
        end
            Nsim(:,i) = Nsimi;
            Ysim(:,i) = Ysimi;
            FBNsim(:,i) = FBNsimv_normi;
        
    end
    
        
        %Estimate equation of motion for actual employment on lagged actual and
        %productivity
        ENsim=mean(Nsim,2);
        EYsim=mean(Ysim,2);
        EFBNsim=mean(FBNsim,2);
        
        yreg = log(ENsim(2:end,1));
        xreg=[ones(T-1,1) log(ENsim(1:end-1,1)) log(psim(2:end,1))];
        updatecoeff=inv(xreg'*xreg)*xreg'*yreg;
        epsupdate = yreg - xreg*updatecoeff;
        R2=1-(epsupdate'*epsupdate)/((yreg-mean(yreg))'*(yreg-mean(yreg)));
        empsmoothmatch=exp(empsmooth)*mean(ENsim);
        %Back out process for pirf new
        psimnew=exp((1/updatecoeff(3))*(log(empsmoothmatch(2:end)) - updatecoeff(1) - ...
            updatecoeff(2)*log(empsmoothmatch(1:end-1))));
        
        psiminit =exp((1/updatecoeff(3))*(log(empsmoothmatch(1)) - updatecoeff(1) - ...
            updatecoeff(2)*log(empsmoothmatch(1))));
        %Update shock process;
        yreg = log(psimnew(2:end,1));xreg=[ones(length(psimnew)-1,1) log(psimnew(1:end-1,1))];
        zupdatecoeff=inv(xreg'*xreg)*xreg'*yreg;
        epsnew = yreg - xreg*zupdatecoeff;
        Var_epsp_new = epsnew'*epsnew/(length(yreg) - size(xreg,2));
        rhop_new = zupdatecoeff(2);
        Var_logp_new = Var_epsp_new/(1-rhop_new^2);
        
        disp('New vs. Old');
        disp([rhop_new rhop])
        disp([Var_logp_new Var_logp])
        
        errz=max([((Var_logp_new - Var_logp)./Var_logp).^2 ((rhop_new - rhop)./rhop).^2])
        
        rhop = rhop_new;
        Var_logp=Var_logp_new;
        psim = [psiminit*ones(T-length(psimnew),1);psimnew];
        
        iter=iter+1;
        
        plot(1:length(ENsim),ENsim,1:length(ENsim),EFBNsim)
        dlmwrite('MatchEmp.txt',[ENsim(T-length(psimnew):end,1) EFBNsim(T-length(psimnew):end,1)]);
        disp([rhop Var_logp]);
end

%AR(1) of output per worker 
logyn = log(Ysim./Nsim);
yyn = logyn(2:end);
xyn = [ones(length(yyn),1) logyn(1:end-1)];
beta_yn = inv(xyn'*xyn)*xyn'*yyn;
resid_yn = yyn - xyn*beta_yn;
disp([beta_yn(2)]);
disp([std(resid_yn)]);

        
        
        
       





