function [J,ENirf,EFBNirf] = main_paramsearch(params)

format long g
rhop = params(1);
conc = params(2);
%% CALIBRATION
%%%%%%%%%%%%%%
beta  = 0.99;
alpha = 0.64;
En    = 20;	     % set mean (desired) establishment size
delta  = 0;      % attrition rate


%% IDIOSYNCRATIC PRODUCTIVITY SHOCK
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rhox       = 0.70;
Stdev_epsx = 0.15;
%Unconditional moments:
E_x        = 1;
Stdev_logx = Stdev_epsx/sqrt(1-rhox^2);
Var_logx   = Stdev_logx^2;
E_logx     = log(E_x) - 0.5*Var_logx;
mux        = (1-rhox)*E_logx;
Stdev_x    = sqrt( (exp(Var_logx)-1)*exp(2*E_logx + Var_logx) );
Skewn_x    = (exp(Var_logx)+2) * sqrt(exp(Var_logx)-1);
%Conditional moments (recognizing the bounds on x):
Fxmax    = 0.9999;
Fxmin    = 0.0001;
x1       = exp( E_logx + Stdev_logx*norminv(Fxmin) );
xN       = exp( E_logx + Stdev_logx*norminv(Fxmax) );
e_x      = exp( E_logx + 0.5*Var_logx )*((normcdf((log(xN)-E_logx-  Var_logx)/Stdev_logx)-normcdf((log(x1)-E_logx-  Var_logx)/Stdev_logx))/(Fxmax-Fxmin));
var_x    = exp( 2*(E_logx + Var_logx) )*((normcdf((log(xN)-E_logx-2*Var_logx)/Stdev_logx)-normcdf((log(x1)-E_logx-2*Var_logx)/Stdev_logx))/(Fxmax-Fxmin)) ...
    +  E_x^2  -  2*E_x*e_x;
stdev_x  = sqrt(var_x);
skewn_x  = exp(3*E_logx + 4.5*Var_logx)*((normcdf((log(xN)-E_logx-3*Var_logx)/Stdev_logx)-normcdf((log(x1)-E_logx-3*Var_logx)/Stdev_logx))/(Fxmax-Fxmin)) ...
    - 3*E_x*(var_x - (E_x^2) + 2*E_x*e_x)  +  3*(E_x^2)*e_x  - (E_x^3);
skewn_x  = skewn_x / (Stdev_x^3);
% Productivity grid: Legendre nodes
xobs    = 90;
[xgrid, xweights] = legendre(x1, xN, xobs);
pdfx    = (1./(xgrid*Stdev_logx*sqrt(2*pi))) .* exp(-0.5*( ((log(xgrid) - E_logx)/Stdev_logx).^2 ));
pdfx	= pdfx / (Fxmax-Fxmin);
xwghts	= xweights.*pdfx;

ENirf = zeros(12,1);
EYirf(:,1)  = zeros(12,1);
ENirf(:,1)  = zeros(12,1);
EFBNirf(:,1) = zeros(12,1);
       
        

%% AGGREGATE PRODUCTIVITY SHOCK
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
E_p        = 1;
Var_logp   = (0.024*(1-alpha))^2;
Stdev_logp = sqrt(Var_logp);
Stdev_epsp = Stdev_logp*sqrt(1-rhop^2);
E_logp     = log(E_p) - 0.5*Var_logp;
mup        = (1-rhop)*E_logp;
Stdev_p    = sqrt( (exp(Var_logp)-1)*exp(2*E_logp + Var_logp) );
Var_epsp   = Stdev_epsp^2;

% Productivity grid: Rouwenhorst transition matrix
pobs   = 11;
logp0  = -sqrt(pobs-1)*Stdev_logp;
logpT  =  sqrt(pobs-1)*Stdev_logp;
logpgrid_ = (logp0 : (logpT-logp0)/(pobs-1) : logpT)';
logpgrid  = logpgrid_+ E_logp;
pgrid  = exp(logpgrid);
prob1  = (1+rhop)/2;
prob2  = (1+rhop)/2;
Ptrans = [ prob1, 1-prob1; 1-prob2, prob2];
for j  = 1:pobs-2
    Np = j+2;
    Ptrans = prob1*[Ptrans, zeros(Np-1,1); zeros(1,Np-1), 0]  + ...
        (1-prob1)* [zeros(Np-1,1) Ptrans; 0 zeros(1,Np-1)] + ...
        (1-prob2)* [zeros(1,Np-1) 0; Ptrans zeros(Np-1,1)] + ...
        prob2*[0 zeros(1,Np-1); zeros(Np-1,1) Ptrans];
    Ptrans(2:end-1,:) = Ptrans(2:end-1,:)/2;
end


%% Employment grid
%  calculate E[ x^(1/(1-alpha)) ]:
Ex_a   = exp((1/(1-alpha))*E_logx  + 0.5*(((1/(1-alpha))*Stdev_logx)^2));
Ex_a   = Ex_a * ((normcdf((log(xgrid(end))-E_logx-(1/(1-alpha))*Var_logx)/Stdev_logx) - ...
    normcdf((log(xgrid(1  ))-E_logx-(1/(1-alpha))*Var_logx)/Stdev_logx)) / (Fxmax-Fxmin));
%  set wage so that average employment in frictionless model = En (this
%  does account for Jensen effects):
W  = alpha * ((Ex_a/En)^(1-alpha));
%  We need to span optimal choice, conditional on adjusting.
%  Again, use frictionless choices as guide and scale up
%  and down from there:
nmin   = 1; % cOnfirm that this is <=((alpha*xgrid(1   )*pgrid(1   )/W)^(1/(1-alpha)));
nmax   = ceil( ((alpha*xgrid(xobs)*pgrid(pobs)/W)^(1/(1-alpha))) );
ngrid  = (nmin :1: nmax)';
nobs   = length(ngrid);
N = 250000;
%Will calibrate conc to hit IRF of FB
cone     = 2;    %exponent on convex cost

linearcost = 0.0; 
hiringcost = linearcost*W;
firingcost = linearcost*W;
    
%  We need to span optimal choice, conditional on adjusting.
%  Again, use frictionless choices as guide and scale up
%  and down from there:
nmin   = 1; 
nmax   = ceil( ((alpha*xgrid(xobs)*pgrid(pobs)/W)^(1/(1-alpha))) );
ngrid  = (nmin :1: nmax)';
nobs   = length(ngrid);

%Solve steady state
[N0,PrNoadj0,xss,nss,En25,En75,nstart,nfinal,nbins,bins,xlag]=simconvex(conc,alpha,beta,xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights,linearcost,cone,W,E_x,N);
disp([PrNoadj0]);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% Policy Function Out of Steady State
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[nh,nf, v_no,v_adjh,v_adjf, vtomaxh,vtomaxf,EV] = policylinearconvexdiscrete(alpha,beta,W,hiringcost,firingcost,   pobs,pgrid,Ptrans,  xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights,conc,cone);
EV = reshape(EV,[pobs xobs nobs]);

T    = 100;
Sims = 24;
horizon = 12;
%   Path of aggregate TFP:
psim = zeros(T,Sims);
epsp = Stdev_epsp*randn(T,Sims);
for s= 1:Sims
    plag = 1;
    for t= 1:T
        logpsim  = -0.5*Var_logp + rhop*log(plag) + epsp(t,s);
        psim(t,s)= max(pgrid(1), min(pgrid(pobs),exp(logpsim)) );
        plag   = psim(t,s);
    end
end
%  Path of idiosyncratic TFP:
xsim  = zeros( T,N);
for t = 1:T
    xt  = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(xlag) + Stdev_epsx*randn(N,1) )));
    xsim(t,:) = xt';
    xlag= xt;
end
clear xt;

Nsim = zeros( T,Sims);
Ysim  = zeros( T,Sims);
YIRF  = zeros(horizon,Sims);
NIRF  = zeros(horizon,Sims);
FBNIRF = zeros(horizon,Sims);

lags  = 4;
lags2 = 3; %allow different lags for innovation. Must have lags2+1 >=lags
Lags2 = max(lags2, lags);

parfor i = 1:Sims
    % initialize:
    nmat = repmat(ngrid',[N 1]);
    nmatalpha = nmat.^alpha;
    nlag  = nss;
    nlagm = repmat(nlag,[1 nobs]);
    xlag  = xss;
    Nsimi=zeros(T,1);
    Ysimi=zeros(T,1);
    FBNsimv_normi= zeros(T,1);
        
    for t = 1:T
        %% Draw agg productivity
        pt  = psim(t,i);
        
        %% Draw IDIOSYNCRATIC productivity
        xt   = xsim(t,:)';
        xtm  = repmat(xt,[1 nobs]);
        %% Interpolate the EV function:
        pos2 = find( pgrid >pt,1,'first');
        pos1 = pos2 -1;
        plo  = pgrid( pos1);
        phi  = pgrid( pos2);
        
        EVt      = EV(pos1,:,:) + ((pt-plo)/(phi-plo))*( EV(pos2,:,:) -EV(pos1,:,:)+10^-10);
        EVt      = squeeze(EVt);
        %%Iterpolate over xt
        EVt = interp1(xgrid,EVt,xt);
        
        % Hiring:
        % As in steady state sims, interpolate value of the firm given
        % n(t) at simulated values of p and x. Then choose optimal n
        % on discrete grid.
        % Hiring:
        vh =  pt.*xtm.*nmatalpha  -  W*nmat  - hiringcost*(nmat - nlagm).*(nmat>nlagm) -(conc)*abs(nmat - nlagm).^cone + beta*EVt;
        [~,npos] = max( vh,[],2 );
        nhire   = ngrid(npos);
        % Firing:
        vf =  pt*xtm.*nmatalpha  -  W*nmat  - firingcost*(nlagm-nmat).*(nmat<nlagm) -(conc)*abs(nmat - nlagm).^cone + beta*EVt;
        [~,npos] = max( vf , [],2);
        nfire  = ngrid(npos);
        % New Employment:
        nt     = nlag.*(nhire<=nlag).*(nlag<=nfire) + nhire.*(nlag<nhire) + nfire.*(nlag>nfire);
 
        % stochastic steady state:
        [ht, fbdensity,nhatsim, fbdensity_,nhatsim_, fbdensity2,nhatsim2, lognhatsim] = ...
            flowbalance( N,bins,nbins,nt,nlag,nhire,nfire);
                
        FBNsimv_normi(t,1) = nhatsim2/sum(fbdensity2);
        Nsimi(t,1) = mean(nt);
        Ysimi(t,1) = pt*mean( xt.*(nt.^alpha) );
        
        % reset:
        xlag  = xt;
        nlag  = nt;
        nlagm = repmat(nlag,[1 nobs]);
  
    end
    % IRFs %%%%%%%%%%%%%%%%%%%%%
           Ndev=log(Nsimi);
           FBNdev=log(FBNsimv_normi);   
           YLdev = log(Ysimi./Nsimi);
         
        
            %% 0)   Estimate surprises to output per worker.
            ylmat =[];
            nmat = [];

            for j = 1:lags;
                ylmat = [ylmat, YLdev(lags-(j-1): end-j)];
                nmat  = [nmat,  Ndev( lags-(j-1): end-j)];
            end            
            ymatrix  = [ylmat, nmat];
            ymatrix  = ymatrix  -  repmat(mean(ymatrix),length(ymatrix),1); %DEMEAN.
            ylcoeff  = (ymatrix'*ymatrix)\(ymatrix'*YLdev(lags+1:end));
            ylresid  = YLdev(lags+1:end) - ymatrix*ylcoeff;

            %%  I)   Estimate impulse responses with surprises
            ylmat = [];
            fbnmat = [];
            nmat  = [];
            Ndev_ = Ndev( lags+1:end, 1);
            FBNdev_ = FBNdev(lags+1:end,1);        
           
            for j = 1:lags;
                ylmat = [ylmat,  ylresid(Lags2+1 -(j-1) : end-j+1)];   %include contemporaneous Y/L
                fbnmat = [fbnmat,  FBNdev_( Lags2   -(j-1) : end-j  )];
                nmat  = [nmat,   Ndev_(  Lags2   -(j-1) : end-j  )];
 
            end
            if lags2 >=lags;
                 % add additional lags of y/n
                for j = 1:lags2-lags+1;
                    ylmat = [ylmat, ylresid(Lags2-lags+2-j : end-lags+1-j)];
                end
            end
             % EMPLOYMENT:
            nmatrix  = [ylmat, nmat];  
            nmatrix  = nmatrix - repmat( mean(nmatrix), length(nmatrix),1);
            ncoeff   = (nmatrix'*nmatrix)\(nmatrix'*Ndev_(Lags2+1:end));
            nresid   = Ndev_(Lags2+1:end) - nmatrix*ncoeff;
       
            % FB EMPLOYMENT:
            fbnmatrix  = [ylmat, fbnmat];  
            fbnmatrix  = fbnmatrix - repmat( mean(fbnmatrix), length(fbnmatrix),1);
            fbncoeff   = (fbnmatrix'*fbnmatrix)\(fbnmatrix'*FBNdev_(Lags2+1:end));
            
           
             % IRFs:
            yirf  = zeros(horizon,1);
            nirf  = zeros(horizon,1);
            fbnirf = zeros(horizon,1);
            
            ysurp = 0.01;
            ylmat = [ysurp, zeros(1,lags2)];
            nmat  = zeros(1,lags);
            fbnmat = zeros(1,lags);
            
            ylmat2= zeros(1,lags);
            for t = 1:horizon;
                 %Employment:
                xmatrix = [ylmat,nmat];
                nirf(t) = xmatrix*ncoeff;
                xmatrixs = [ylmat,fbnmat];
                fbnirf(t) = xmatrixs*fbncoeff;
                
                 %Output per worker:
                if t==1;
                    yirf(t) = ysurp;
                else
                    xmatrix = [ylmat2,nmat];
                    yirf(t) = xmatrix*ylcoeff;
                end

                 % Reset:
                nmat  = [nirf(t),   nmat(1:lags-1)]; %reset n.
                fbnmat = [fbnirf(t), fbnmat(1:lags-1)];
                
                ylmat2= [yirf(t), ylmat2(1:lags-1)];

                if t <lags2;
                     %ylmat is vector of current and past residuals.
                    ylmat= [zeros(1,t), ysurp, zeros(1,lags2-t)];
                elseif t==lags2;
                    ylmat= [zeros(1,t), ysurp];
                else
                    ylmat= zeros(1,lags2+1);
                end
            end
            YIRF(:,i) =yirf;
            NIRF(:,i) =nirf;
            FBNIRF(:,i) =fbnirf;
           
        end
        clear xt nt prodt;
        EYirf(:,1)  = mean(YIRF,2);
        ENirf(:,1)  = mean(NIRF,2);
        EFBNirf(:,1) = mean(FBNIRF,2);

 
datairf_actual =  [0.0014059;
0.0043808;
0.0071306;
0.0084176;
0.0091438;
0.0092912;
0.0090231;
0.0084545;
0.0076857;
0.0067995;
0.0058621;
0.0049251];   

datairf_fb = [0.004501;
0.007457;
0.008540;
0.009272;
0.009040;
0.008474;
0.007789;
0.006977;
0.006170;
0.005402;
0.004685;
0.004039];

J = sum((datairf_actual-ENirf(:,1)).^2);
        
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
