function [nh,nf, v_no,v_adjh,v_adjf, vtomaxh,vtomaxf,EV] = policylinearconvexdiscrete_bod(alpha,beta,W,hiringcost,firingcost,   pobs,pgrid,Ptrans,  xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights,conc,cone)
    
%%Create matrices as pobs x xobs*nobs(-1) x nobs
xmat   = repmat(xgrid * ones(1,nobs),[nobs 1]);
nch   = ones(xobs*nobs,1) * ngrid';             
nmat   = repmat(kron(ngrid,ones(xobs,1)),[1 nobs]);
nlag   = kron(ngrid,ones(xobs,1));

% Don't adjust:
%loop over productivity
profit_adjh = zeros(pobs,xobs*nobs);
profit_adjf = zeros(pobs,xobs*nobs);
profit_no   = zeros(pobs,xobs*nobs);
Nh_m = zeros(pobs,xobs*nobs);
Nf_m = zeros(pobs,xobs*nobs);
for i = 1:pobs
    pi = pgrid(i);
    
    profit_no(i,:)  = pi*xmat(:,1).*(nlag.^alpha)  -  W*nlag;
    
    % Adjust UP:
    profit_adjh1 = pi*xmat.*(nch.^alpha) -  W*nch  - hiringcost*(nch-nmat).*(nch >nmat) - (conc)*abs(nch - nmat).^cone;
    [profit_adjh(i,:),npos]   = max(profit_adjh1,[],2);
    Nh_m(i,:)  = ngrid(npos);
    
    % Adjust DOWN:
    profit_adjf1 = pi*xmat.*(nch.^alpha)  -  W*nch  - firingcost*(nmat-nch).*(nch <nmat) - (conc)*abs(nch - nmat).^cone;
    [profit_adjf(i,:),npos]   = max(profit_adjf1,[],2);
    Nf_m(i,:)  = ngrid(npos);
    
end

%% Start value iteration
v_adjh = profit_adjh; 
v_adjf = profit_adjf; 
v_no   = profit_no;   
v0 = zeros(pobs,xobs*nobs);
V_adjh = zeros(pobs,xobs*nobs);
V_adjf = V_adjh;
for i = 1:pobs;
    V_adjh(i,:) = (v_adjh(i,:)'.*(Nh_m(i,:)' >nlag) - 10e4*(Nh_m(i,:)' <=nlag))';
    V_adjf(i,:) = (v_adjf(i,:)'.*(Nf_m(i,:)' <nlag) - 10e4*(Nf_m(i,:)' >=nlag))';
    v0(i,:)     = max( [V_adjf(i,:)', v_no(i,:)', V_adjh(i,:)'],[], 2);
end


% For use in optimization,
profith = zeros(pobs,xobs*nobs,nobs);
profitf = zeros(pobs,xobs*nobs,nobs);
profit_no = zeros(pobs,xobs*nobs);
for i = 1:pobs;
    pi = pgrid(i);
    profith(i,:,:)  =  pi*xmat.*(nch.^alpha)  -  W*nch  - hiringcost*nch - (conc)*abs(nch - nmat).^cone;
    profitf(i,:,:)  =  pi*xmat.*(nch.^alpha)  -  W*nch  + firingcost*nch - (conc)*abs(nch - nmat).^cone;
    profit_no(i,:)  =  pi*xmat(:,1).*(nlag.^alpha)  -  W*nlag;
   
end

nh0 = Nh_m;
nf0 = Nf_m;
nf = nf0;
nh = nh0;
iter   = 0;
ptol   = 0.005;
perrcount  = 0;

while perrcount <5
    
    %% Iterate forward: for given v, compute E[v].
    EV     = evfunlinearconvex(nobs,xobs,xgrid,ngrid,xweights,  pobs,pgrid,Ptrans,  nf,nh,v0,v_no,v_adjh,v_adjf,  mux,rhox,Stdev_epsx); 
    
    v = zeros(pobs,xobs*nobs);
    v_adjh = v;
    v_adjf = v;
    nh     = v;
    nf     = v;
    for i = 1:pobs;
        EVi = reshape(squeeze(EV(i,:)),[xobs nobs]);
        EVi = repmat(EVi,[nobs 1]);
        EVi0 = squeeze(EV(i,:))';
        vtomaxh = squeeze(profith(i,:,:)) + beta*EVi;
        vtomaxf = squeeze(profitf(i,:,:)) + beta*EVi;
        
        % HIRING:
        [vmaxh_, pos]= max(vtomaxh,[],2);
        nnewh = ngrid(pos);
        vmaxh = vmaxh_ + hiringcost*nlag;
        % FIRING:
        [vmaxf_, pos]= max(vtomaxf,[],2);
        nnewf = ngrid(pos);
        vmaxf = vmaxf_ - firingcost*nlag;
        % NO Adjust
        v_no = squeeze(profit_no(i,:))' + beta*EVi0;
        
        v_adjh = vmaxh;
        v_adjf = vmaxf;
        Nh     = nnewh;
        Nf     = nnewf;
        V_adjh = v_adjh.*(Nh >nlag) - 10e6*(Nh <=nlag);
        V_adjf = v_adjf.*(Nf <nlag) - 10e6*(Nf >=nlag);
        v(i,:)      = max( [V_adjf,v_no, V_adjh], [],2 );
        nh(i,:) = Nh;
        nf(i,:) = Nf;
    end
    
    
    % Compare:
    iter   = iter +1;
    perr_h = max( max(abs(nh-nh0)) );
    perr_f = max( max(abs(nf-nf0)) );
    perr   = max( [perr_h,perr_f] );
    verr   = max(max(abs( v-v0)));
    %disp('Iteration,  dV,    max{dhiring,dfiring}');
    %disp( [iter,      verr,  perr] );
    perrcount = (perr <ptol).*(perrcount +1);
    nh0 = nh;
    nf0 = nf;
    v0  = v;

end

