function [N0,PrNoadj0,xss,nss,En25,En75,nstart,nfinal,nbins,bins,xlag] = simconvex(conc,alpha,beta,xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights,linearcost,cone,W,E_x,N)

hiringcost = linearcost*W;
firingcost = linearcost*W;  

%% First, full steady state
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
bargpower = 0;
threatpoint = W;
scaleprodfn = 1;
% Function written so that it can accommodate a SZ wage bargain. To
% shut off, set "0" for zero bargaining power,  insert W for the outside
% option, or threat point, and do not scale the production function (set
% this to 1).

[ssnh,ssnf, ssv_no,ssv_adjh,ssv_adjf, ssvh,ssvf,EVss] = policylinearconvexdiscretess(alpha,beta,  xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights,  bargpower,threatpoint,scaleprodfn, hiringcost,firingcost,conc,cone);

% Simulate invariant (steady-state) distribution of n.
rng( 508);
xlag  = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(E_x) + Stdev_epsx*randn(N,1) )));
nlag  = ceil( (alpha*xlag/(W+conc/2)).^(1/(1-alpha)) ); % round up.
T0 = 50;
Nsim  = zeros( T0,1);
PrNOadj = zeros( T0,1 );
stdDlogn = zeros(T0,1);
JR       = zeros(T0,1);
avecost  = zeros(T0,1);
convcost = zeros(T0,1);
lincost   = zeros(T0,1);
En25     = zeros(T0,1);
En75     = zeros(T0,1);
%Some matrices
nmat = repmat(ngrid',[N 1]);
nmatalpha = nmat.^alpha;
nlagm = repmat(nlag,[1 nobs]);
for t = 1:T0
    %% Draw IDIOSYNCRATIC productivity
    xt    = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(xlag) + Stdev_epsx*randn(N,1) )));
    xtm  = repmat(xt,[1 nobs]);
    
    EVt = interp1(xgrid,EVss,xt);
    
    % Hiring:
    vh =  scaleprodfn.*xtm.*nmatalpha  -  threatpoint*nmat  - hiringcost*(nmat - nlagm).*(nmat>nlagm) -(conc)*abs(nmat - nlagm).^cone + beta*EVt;
    [~,npos] = max( vh,[],2 );
    reseth   = ngrid(npos);
    adjustpos= (nlag <reseth);
    % Firing:
    vf =  scaleprodfn*xtm.*nmatalpha  -  threatpoint*nmat  - firingcost*(nlagm-nmat).*(nmat<nlagm) -(conc)*abs(nmat - nlagm).^cone + beta*EVt;
    [~,npos] = max( vf , [],2);
    resetf   = ngrid(npos);
    adjustneg= (nlag >resetf);
    % New Employment:
    adjust   = adjustneg + adjustpos;
    nsim     = reseth.*adjustpos + resetf.*adjustneg + nlag.*(1-adjust);
    
    
    %% Moments:
    Nsim(t) = mean( nsim );
    PrNOadj(t) = mean( (nsim==nlag ) );
    Dlogn     = log(nsim./nlag);
    stdDlogn(t) = std(Dlogn(adjust==1));
    gt   = 200*((nsim-nlag)./(nsim + nlag));
    Dn      = abs(nsim(adjust==1) - nlag(adjust==1));
    avecost(t) = mean(((conc).*Dn.^(cone))./(nsim(adjust==1)-nlag(adjust==1)))/W;
    convcost(t) = sum((conc).*Dn.^(cone));
    lincost(t)  = sum(hiringcost.*(nsim-nlag).*(nsim>nlag)) - sum(firingcost.*(nsim-nlag).*(nsim<nlag));
    JR(t)   = -mean(gt(adjustneg)) + mean(gt(adjustpos));
    En25(t) = prctile(nsim,25);
    En75(t) = prctile(nsim,75);
    
    % reset:
    xlag = xt;
    if t == T0; nlagf = nlag;
                reseth  = reseth;
                resetf = resetf;
    end
    
    nlag = nsim;

    nlagm = repmat(nlag,[1 nobs]);

end
t0 = 25;
N0  = mean(Nsim(end-t0:end));
PrNoadj0 = mean(PrNOadj(end-t0:end));
%disp('Mean employment'); disp( N0);
%disp('Prob of not adjusting'); disp(PrNoadj0);
xss = xlag;
nss = nlag;

En25 = mean(En25(end-t0:end));
En75 = mean(En75(end-t0:end));

nstart = max(ngrid(1),   min(nss)-10);
nfinal = min(ngrid(nobs),max(nss)+10);
nbins  = (nstart :1: nfinal)';
bins   = length(nbins);
