clear; clc;
plotflag = 0;
poolobj = gcp('nocreate');
delete(poolobj);
format long g
myPool = parpool(20);

%% CALIBRATION
%%%%%%%%%%%%%%
beta  = 0.99;
alpha = 0.64;
En    = 20;	     % set mean (desired) establishment size
delta  = 0;      % attrition rate

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Calibrate Measurement Error
%Share of establishments with missing employment data in a
%particular quarter
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
gamma_me = 0.05;

%% IDIOSYNCRATIC PRODUCTIVITY SHOCK
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rhox       = 0.70;
Stdev_epsx = 0.15;
%Unconditional moments:
E_x        = 1;
Stdev_logx = Stdev_epsx/sqrt(1-rhox^2);
Var_logx   = Stdev_logx^2;
E_logx     = log(E_x) - 0.5*Var_logx;
mux        = (1-rhox)*E_logx;
Stdev_x    = sqrt( (exp(Var_logx)-1)*exp(2*E_logx + Var_logx) );
Skewn_x    = (exp(Var_logx)+2) * sqrt(exp(Var_logx)-1);
%Conditional moments (recognizing the bounds on x):
Fxmax    = 0.9999;
Fxmin    = 0.0001;
x1       = exp( E_logx + Stdev_logx*norminv(Fxmin) );
xN       = exp( E_logx + Stdev_logx*norminv(Fxmax) );
e_x      = exp( E_logx + 0.5*Var_logx )*((normcdf((log(xN)-E_logx-  Var_logx)/Stdev_logx)-normcdf((log(x1)-E_logx-  Var_logx)/Stdev_logx))/(Fxmax-Fxmin));
var_x    = exp( 2*(E_logx + Var_logx) )*((normcdf((log(xN)-E_logx-2*Var_logx)/Stdev_logx)-normcdf((log(x1)-E_logx-2*Var_logx)/Stdev_logx))/(Fxmax-Fxmin)) ...
    +  E_x^2  -  2*E_x*e_x;
stdev_x  = sqrt(var_x);
skewn_x  = exp(3*E_logx + 4.5*Var_logx)*((normcdf((log(xN)-E_logx-3*Var_logx)/Stdev_logx)-normcdf((log(x1)-E_logx-3*Var_logx)/Stdev_logx))/(Fxmax-Fxmin)) ...
    - 3*E_x*(var_x - (E_x^2) + 2*E_x*e_x)  +  3*(E_x^2)*e_x  - (E_x^3);
skewn_x  = skewn_x / (Stdev_x^3);
% Productivity grid: Legendre nodes
xobs    = 90;
[xgrid, xweights] = legendre(x1, xN, xobs);
pdfx    = (1./(xgrid*Stdev_logx*sqrt(2*pi))) .* exp(-0.5*( ((log(xgrid) - E_logx)/Stdev_logx).^2 ));
pdfx	= pdfx / (Fxmax-Fxmin);
xwghts	= xweights.*pdfx;

%Start with the parameters of Figure 10
load('Fig10params','x');
rhop = [x(1)];
lincost = [x(2)];

ENirf = zeros(12,1);
EYirf(:,1)  = ENirf;
ENirf(:,1)  = ENirf;
EFBNirf(:,1) = ENirf;
Ephi1irf(:,1) = ENirf;
Ephi2irf(:,1) = ENirf;
Ephi3irf(:,1) = ENirf;
Etau1irf(:,1) = ENirf;
Etau2irf(:,1) = ENirf;
Etau3irf(:,1) = ENirf;

%% AGGREGATE PRODUCTIVITY SHOCK
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
E_p        = 1;
Var_logp   = (0.024*(1-alpha))^2;
Stdev_logp = sqrt(Var_logp);
Stdev_epsp = Stdev_logp*sqrt(1-rhop^2);
E_logp     = log(E_p) - 0.5*Var_logp;
mup        = (1-rhop)*E_logp;
Stdev_p    = sqrt( (exp(Var_logp)-1)*exp(2*E_logp + Var_logp) );
Var_epsp   = Stdev_epsp^2;

% Productivity grid: Rouwenhorst transition matrix
pobs   = 11;
logp0  = -sqrt(pobs-1)*Stdev_logp;
logpT  =  sqrt(pobs-1)*Stdev_logp;
logpgrid_ = (logp0 : (logpT-logp0)/(pobs-1) : logpT)';
logpgrid  = logpgrid_+ E_logp;
pgrid  = exp(logpgrid);
prob1  = (1+rhop)/2;
prob2  = (1+rhop)/2;
Ptrans = [ prob1, 1-prob1; 1-prob2, prob2];
for j  = 1:pobs-2
    Np = j+2;
    Ptrans = prob1*[Ptrans, zeros(Np-1,1); zeros(1,Np-1), 0]  + ...
        (1-prob1)* [zeros(Np-1,1) Ptrans; 0 zeros(1,Np-1)] + ...
        (1-prob2)* [zeros(1,Np-1) 0; Ptrans zeros(Np-1,1)] + ...
        prob2*[0 zeros(1,Np-1); zeros(Np-1,1) Ptrans];
    Ptrans(2:end-1,:) = Ptrans(2:end-1,:)/2;
end



%% Employment grid
%  calculate E[ x^(1/(1-alpha)) ]:
Ex_a   = exp((1/(1-alpha))*E_logx  + 0.5*(((1/(1-alpha))*Stdev_logx)^2));
Ex_a   = Ex_a * ((normcdf((log(xgrid(end))-E_logx-(1/(1-alpha))*Var_logx)/Stdev_logx) - ...
    normcdf((log(xgrid(1  ))-E_logx-(1/(1-alpha))*Var_logx)/Stdev_logx)) / (Fxmax-Fxmin));
%  set wage so that average employment in frictionless model = En (this
%  does account for Jensen effects):
W  = alpha * ((Ex_a/En)^(1-alpha));
hiringcost = lincost*W;
firingcost = lincost*W;
%Shut down convex cost parameters
conc     = 0; %constant in front of convex cost
cone     = 0;    %exponent on convex cost
%  We need to span optimal choice, conditional on adjusting.
%  Again, use frictionless choices as guide and scale up
%  and down from there:
nmin   = 1; 
nmax   = ceil( ((alpha*xgrid(xobs)*pgrid(pobs)/W)^(1/(1-alpha))) );
ngrid  = (nmin :1: nmax)';
nobs   = length(ngrid);


%% First, full steady state
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
bargpower = 0;
threatpoint = W;
scaleprodfn = 1;
% Function written so that it can accommodate a SZ wage bargain. To
% shut off, set "0" for zero bargaining power,  insert W for the outside
% option, or threat point, and do not scale the production function (set
% this to 1).

[ssnh,ssnf, ssv_no,ssv_adjh,ssv_adjf, ssvh,ssvf,EVss] = policylinearconvexdiscretess(alpha,beta,  xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights,  bargpower,threatpoint,scaleprodfn, hiringcost,firingcost,conc,cone);

% Simulate invariant (steady-state) distribution of n.
rng( 508);
N     = 250000;
xlag  = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(E_x) + Stdev_epsx*randn(N,1) )));
nlag  = ceil( (alpha*xlag/(W+conc/2)).^(1/(1-alpha)) ); % round up.
T0 = 50;
Nsim  = zeros( T0,1);
PrNOadj = zeros( T0,1 );
stdDlogn = zeros(T0,1);
JR       = zeros(T0,1);
avecost  = zeros(T0,1);
convcost = zeros(T0,1);
lincost   = zeros(T0,1);
En25     = zeros(T0,1);
En75     = zeros(T0,1);
%Some matrices
nmat = repmat(ngrid',[N 1]);
nmatalpha = nmat.^alpha;
nlagm = repmat(nlag,[1 nobs]);
for t = 1:T0
    %% Draw IDIOSYNCRATIC productivity
    xt    = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(xlag) + Stdev_epsx*randn(N,1) )));
    xtm  = repmat(xt,[1 nobs]);
    
    EVt = interp1(xgrid,EVss,xt);
    
    % Hiring:
    vh =  scaleprodfn.*xtm.*nmatalpha  -  threatpoint*nmat  - hiringcost*(nmat - nlagm).*(nmat>nlagm) -(conc)*abs(nmat - nlagm).^cone + beta*EVt;
    [~,npos] = max( vh,[],2 );
    reseth   = ngrid(npos);
    adjustpos= (nlag <reseth);
    % Firing:
    vf =  scaleprodfn*xtm.*nmatalpha  -  threatpoint*nmat  - firingcost*(nlagm-nmat).*(nmat<nlagm) -(conc)*abs(nmat - nlagm).^cone + beta*EVt;
    [~,npos] = max( vf , [],2);
    resetf   = ngrid(npos);
    adjustneg= (nlag >resetf);
    % New Employment:
    adjust   = adjustneg + adjustpos;
    nsim     = reseth.*adjustpos + resetf.*adjustneg + nlag.*(1-adjust);
    
    
    %% Moments:
    Nsim(t) = mean( nsim );
    PrNOadj(t) = mean( (nsim==nlag ) );
    Dlogn     = log(nsim./nlag);
    stdDlogn(t) = std(Dlogn(adjust==1));
    gt   = 200*((nsim-nlag)./(nsim + nlag));
    Dn      = abs(nsim(adjust==1) - nlag(adjust==1));
    avecost(t) = mean(((conc).*Dn.^(cone))./(nsim(adjust==1)-nlag(adjust==1)))/W;
    convcost(t) = sum((conc).*Dn.^(cone));
    lincost(t)  = sum(hiringcost.*(nsim-nlag).*(nsim>nlag)) - sum(firingcost.*(nsim-nlag).*(nsim<nlag));
    trialconv = (conc).*((nsim-nlag)).^(cone);triallin = hiringcost.*(nsim-nlag).*(nsim>nlag) - firingcost.*(nsim-nlag).*(nsim<nlag);
    JR(t)   = -mean(gt(adjustneg)) + mean(gt(adjustpos));
    En25(t) = prctile(nsim,25);
    En75(t) = prctile(nsim,75);
    
    % reset:
    xlag = xt;
    nlag = nsim;
    nlagm = repmat(nlag,[1 nobs]);
    if t/5==floor(t/5); disp( [t,mean(Nsim(5:t))] ); end
end
t0 = 25;
N0  = mean(Nsim(end-t0:end));
PrNoadj0 = mean(PrNOadj(end-t0:end));
disp('Mean employment'); disp( N0);
disp('Prob of not adjusting'); disp(PrNoadj0);
xss = xlag;
nss = nlag;

En25 = mean(En25(end-t0:end));
En75 = mean(En75(end-t0:end));

nstart = max(ngrid(1),   min(nss)-10);
nfinal = min(ngrid(nobs),max(nss)+10);
nbins  = (nstart :1: nfinal)';
bins   = length(nbins);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% Policy Function Out of Steady State
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[nh,nf, v_no,v_adjh,v_adjf, vtomaxh,vtomaxf,EV] = policylinearconvexdiscrete(alpha,beta,W,hiringcost,firingcost,   pobs,pgrid,Ptrans,  xobs,xgrid,Stdev_epsx,rhox,mux,   nobs,ngrid,xweights,conc,cone);
EV = reshape(EV,[pobs xobs nobs]);

T    = 100;
Sims = 24;
horizon = 12;
%   Path of aggregate TFP:
psim = zeros(T,Sims);
epsp = Stdev_epsp*randn(T,Sims);
for s= 1:Sims
    plag = 1;
    for t= 1:T
        logpsim  = -0.5*Var_logp + rhop*log(plag) + epsp(t,s);
        psim(t,s)= max(pgrid(1), min(pgrid(pobs),exp(logpsim)) );
        plag   = psim(t,s);
    end
end
%  Path of idiosyncratic TFP:
xsim  = zeros( T,N);
for t = 1:T
    xt  = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(xlag) + Stdev_epsx*randn(N,1) )));
    xsim(t,:) = xt';
    xlag= xt;
end
clear xt;

Nsim = zeros( T,Sims);
Ysim  = zeros( T,Sims);
YIRF  = zeros(horizon,Sims);
NIRF  = zeros(horizon,Sims);
FBNIRF = zeros(horizon,Sims);

lags  = 4;
lags2 = 3; %allow different lags for innovation. Must have lags2+1 >=lags
Lags2 = max(lags2, lags);

parfor i = 1:Sims
    % initialize:
    nmat = repmat(ngrid',[N 1]);
    nmatalpha = nmat.^alpha;
    nlag  = nss;
    nlagm = repmat(nlag,[1 nobs]);
    xlag  = xss;
    Nsimi=zeros(T,1);
    Nsimi_me = zeros(T,1);
    Ysimi=zeros(T,1);
    FBNsimv_normi= zeros(T,1);
    inflows25pi = zeros(T,1);
    inflows75pi= zeros(T,1);
    inflowsmidpi = zeros(T,1);
    phi25pi = zeros(T,1);
    phi75pi = zeros(T,1);
    phimidpi = zeros(T,1);
    nlag_me = nss;
    nlag_me2 = nss;
    nlag_me3 = nss;
    nlag_me4 = nss;
    nlag_me5 = nss;
    dn4      = 1;        
    for t = 1:T
        %% Draw agg productivity
        pt  = psim(t,i);
        
        %% Draw IDIOSYNCRATIC productivity
        %xt  = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(xlag) + Stdev_epsx*randn(N,1) )));
        xt   = xsim(t,:)';
        xtm  = repmat(xt,[1 nobs]);
        %% Interpolate the EV function:
        pos2 = find( pgrid >pt,1,'first');
        pos1 = pos2 -1;
        plo  = pgrid( pos1);
        phi  = pgrid( pos2);
        
        EVt      = EV(pos1,:,:) + ((pt-plo)/(phi-plo))*( EV(pos2,:,:) -EV(pos1,:,:)+10^-10);
        EVt      = squeeze(EVt);
        %%Iterpolate over xt
        EVt = interp1(xgrid,EVt,xt);
        
        % Hiring:
        % As in steady state sims, interpolate value of the firm given
        % n(t) at simulated values of p and x. Then choose optimal n
        % on discrete grid.
        % Hiring:
        vh =  pt.*xtm.*nmatalpha  -  W*nmat  - hiringcost*(nmat - nlagm).*(nmat>nlagm) -(conc)*abs(nmat - nlagm).^cone + beta*EVt;
        [~,npos] = max( vh,[],2 );
        nhire   = ngrid(npos);
        % Firing:
        vf =  pt*xtm.*nmatalpha  -  W*nmat  - firingcost*(nlagm-nmat).*(nmat<nlagm) -(conc)*abs(nmat - nlagm).^cone + beta*EVt;
        [~,npos] = max( vf , [],2);
        nfire  = ngrid(npos);
        % New Employment:
        nt     = nlag.*(nhire<=nlag).*(nlag<=nfire) + nhire.*(nlag<nhire) + nfire.*(nlag>nfire);
 
        me1       = (rand(N,1)<gamma_me);
         
        nt_me  = min(max(round(nt.*dn4),1),ngrid(end)).*(me1==1) + nt.*(me1==0);
         
        nhire_me = nhire.*(nt_me<=nt) + nt_me.*(nt_me>nt);
        nfire_me = nfire.*(nt_me>=nt) + nt_me.*(nt_me<nt);       
          
        [ht, fbdensity,nhatsim, fbdensity_,nhatsim_, fbdensity2,nhatsim2, lognhatsim,...
            inflows25p,inflows75p,inflows_midp,phi_25p,phi_75p,phi_midp] = ...
            flowbalance_flows( N,bins,nbins,nt_me,nlag_me,nhire_me,nfire_me,En25,En75);
        
        FBNsimv_normi(t,1) = nhatsim2/sum(fbdensity2);
        Nsimi(t,1) = mean(nt);
        Nsimi_me(t,1) = mean(nt_me);
        Ysimi(t,1) = pt*mean( xt.*(nt.^alpha) );
        inflows25pi(t,1) = inflows25p;
        inflows75pi(t,1) = inflows75p;
        inflowsmidpi(t,1) = inflows_midp;
        phi25pi(t,1) = phi_25p;
        phi75pi(t,1) = phi_75p;
        phimidpi(t,1) = phi_midp;
        
        % reset:
        xlag  = xt;
        nlag  = nt;
        nlagm = repmat(nlag,[1 nobs]);
        nlag_me5 = nlag_me4;
        nlag_me4 = nlag_me3;
        nlag_me3 = nlag_me2;
        nlag_me2 = nlag_me;
        nlag_me = nt_me;
        dn4 = nlag_me4./nlag_me5;
        
        
        if t/5==floor(t/5); disp( [t,i]); end
    end
    % IRFs %%%%%%%%%%%%%%%%%%%%%
           Ndev=log(Nsimi_me);
           FBNdev=log(FBNsimv_normi);   
           YLdev = log(Ysimi./Nsimi_me);
           
           phi1=log(phi25pi);
           phi2=log(phimidpi);
           phi3=log(phi75pi);
           tau1=log(inflows25pi);
           tau2=log(inflowsmidpi);
           tau3=log(inflows75pi);
           
            %% 0)   Estimate surprises to output per worker.
            ylmat =[];
            nmat = [];

            for j = 1:lags;
                ylmat = [ylmat, YLdev(lags-(j-1): end-j)];
                nmat  = [nmat,  Ndev( lags-(j-1): end-j)];
            end            
            ymatrix  = [ylmat, nmat];
            ymatrix  = ymatrix  -  repmat(mean(ymatrix),length(ymatrix),1); %DEMEAN.
            ylcoeff  = (ymatrix'*ymatrix)\(ymatrix'*YLdev(lags+1:end));
            ylresid  = YLdev(lags+1:end) - ymatrix*ylcoeff;

            %%  I)   Estimate impulse responses with surprises
            ylmat = [];
            fbnmat = [];
            nmat  = [];
            Ndev_ = Ndev( lags+1:end, 1);
            FBNdev_ = FBNdev(lags+1:end,1);
            
            phi1mat=[];phi2mat=[];phi3mat=[];tau1mat=[];tau2mat=[];tau3mat=[];
            phi1_  = phi1(lags+1:end,1);
            phi2_  = phi2(lags+1:end,1);
            phi3_  = phi3(lags+1:end,1);
            tau1_  = tau1(lags+1:end,1);
            tau2_  = tau2(lags+1:end,1);
            tau3_  = tau3(lags+1:end,1);
            
            for j = 1:lags;
                ylmat = [ylmat,  ylresid(Lags2+1 -(j-1) : end-j+1)];   %include contemporaneous Y/L
                fbnmat = [fbnmat,  FBNdev_( Lags2   -(j-1) : end-j  )];
                nmat  = [nmat,   Ndev_(  Lags2   -(j-1) : end-j  )];
                phi1mat = [phi1mat, phi1_(Lags2 - (j-1) : end-j  )];
                phi2mat = [phi2mat, phi2_(Lags2 - (j-1) : end-j  )];
                phi3mat = [phi3mat, phi3_(Lags2 - (j-1) : end-j  )];
                tau1mat = [tau1mat, tau1_(Lags2 - (j-1) : end-j  )];
                tau2mat = [tau2mat, tau2_(Lags2 - (j-1) : end-j  )];
                tau3mat = [tau3mat, tau3_(Lags2 - (j-1) : end-j  )];
            end
            if lags2 >=lags;
                 % add additional lags of y/n
                for j = 1:lags2-lags+1;
                    ylmat = [ylmat, ylresid(Lags2-lags+2-j : end-lags+1-j)];
                end
            end
             % EMPLOYMENT:
            nmatrix  = [ylmat, nmat];  
            nmatrix  = nmatrix - repmat( mean(nmatrix), length(nmatrix),1);
            ncoeff   = (nmatrix'*nmatrix)\(nmatrix'*Ndev_(Lags2+1:end));
            nresid   = Ndev_(Lags2+1:end) - nmatrix*ncoeff;
       
            % FB EMPLOYMENT:
            fbnmatrix  = [ylmat, fbnmat];  
            fbnmatrix  = fbnmatrix - repmat( mean(fbnmatrix), length(fbnmatrix),1);
            fbncoeff   = (fbnmatrix'*fbnmatrix)\(fbnmatrix'*FBNdev_(Lags2+1:end));
            
            %% Phi1
            phi1matrix  = [ylmat, phi1mat]; %ones(T-lags-Lags2,1),
            phi1matrix  = phi1matrix - repmat( mean(phi1matrix), length(phi1matrix),1);
            phi1coeff   = (phi1matrix'*phi1matrix)\(phi1matrix'*phi1_(Lags2+1:end));
            
            %% Phi2
            phi2matrix  = [ylmat, phi2mat]; %ones(T-lags-Lags2,1),
            phi2matrix  = phi2matrix - repmat( mean(phi2matrix), length(phi2matrix),1);
            phi2coeff   = (phi2matrix'*phi2matrix)\(phi2matrix'*phi2_(Lags2+1:end));
            
            %% Phi3
            phi3matrix  = [ylmat, phi3mat]; %ones(T-lags-Lags2,1),
            phi3matrix  = phi3matrix - repmat( mean(phi3matrix), length(phi3matrix),1);
            phi3coeff   = (phi3matrix'*phi3matrix)\(phi3matrix'*phi3_(Lags2+1:end));
            
            %% tau1
            tau1matrix  = [ylmat, tau1mat]; %ones(T-lags-Lags2,1),
            tau1matrix  = tau1matrix - repmat( mean(tau1matrix), length(tau1matrix),1);
            tau1coeff   = (tau1matrix'*tau1matrix)\(tau1matrix'*tau1_(Lags2+1:end));
            
            %% tau2
            tau2matrix  = [ylmat, tau2mat]; %ones(T-lags-Lags2,1),
            tau2matrix  = tau2matrix - repmat( mean(tau2matrix), length(tau2matrix),1);
            tau2coeff   = (tau2matrix'*tau2matrix)\(tau2matrix'*tau2_(Lags2+1:end));
            
            %% tau3
            tau3matrix  = [ylmat, tau3mat]; %ones(T-lags-Lags2,1),
            tau3matrix  = tau3matrix - repmat( mean(tau3matrix), length(tau3matrix),1);
            tau3coeff   = (tau3matrix'*tau3matrix)\(tau3matrix'*tau3_(Lags2+1:end));
            
            
             % IRFs:
            yirf  = zeros(horizon,1);
            nirf  = zeros(horizon,1);
            fbnirf = zeros(horizon,1);
            phi1irf = zeros(horizon,1);
            phi2irf = zeros(horizon,1);
            phi3irf = zeros(horizon,1);
            tau1irf = zeros(horizon,1);
            tau2irf = zeros(horizon,1);
            tau3irf = zeros(horizon,1);
            
            ysurp = 0.01;
            ylmat = [ysurp, zeros(1,lags2)];
            nmat  = zeros(1,lags);
            fbnmat = zeros(1,lags);
            phi1mat = zeros(1,lags);
            phi2mat = zeros(1,lags);
            phi3mat = zeros(1,lags);
            tau1mat = zeros(1,lags);
            tau2mat = zeros(1,lags);
            tau3mat = zeros(1,lags);
            
            ylmat2= zeros(1,lags);
            for t = 1:horizon;
                 %Employment:
                xmatrix = [ylmat,nmat];
                nirf(t) = xmatrix*ncoeff;
                xmatrixs = [ylmat,fbnmat];
                fbnirf(t) = xmatrixs*fbncoeff;
                xmatrixflow = [ylmat,phi1mat];
                phi1irf(t) = xmatrixflow*phi1coeff;
                xmatrixflow = [ylmat,phi2mat];
                phi2irf(t) = xmatrixflow*phi2coeff;
                xmatrixflow = [ylmat,phi3mat];
                phi3irf(t) = xmatrixflow*phi3coeff;
                xmatrixflow = [ylmat,tau1mat];
                tau1irf(t) = xmatrixflow*tau1coeff;
                xmatrixflow = [ylmat,tau2mat];
                tau2irf(t) = xmatrixflow*tau2coeff;
                xmatrixflow = [ylmat,tau3mat];
                tau3irf(t) = xmatrixflow*tau3coeff;
                
                 %Output per worker:
                if t==1;
                    yirf(t) = ysurp;
                else
                    xmatrix = [ylmat2,nmat];
                    yirf(t) = xmatrix*ylcoeff;
                end

                 % Reset:
                nmat  = [nirf(t),   nmat(1:lags-1)]; %reset n.
                fbnmat = [fbnirf(t), fbnmat(1:lags-1)];
                phi1mat = [phi1irf(t),phi1mat(1:lags-1)];
                phi2mat = [phi2irf(t),phi2mat(1:lags-1)];
                phi3mat = [phi3irf(t),phi3mat(1:lags-1)];
                tau1mat = [tau1irf(t),tau1mat(1:lags-1)];
                tau2mat = [tau2irf(t),tau2mat(1:lags-1)];
                tau3mat = [tau3irf(t),tau3mat(1:lags-1)];                
                
                ylmat2= [yirf(t), ylmat2(1:lags-1)];

                if t <lags2;
                     %ylmat is vector of current and past residuals.
                    ylmat= [zeros(1,t), ysurp, zeros(1,lags2-t)];
                elseif t==lags2;
                    ylmat= [zeros(1,t), ysurp];
                else
                    ylmat= zeros(1,lags2+1);
                end
            end
            YIRF(:,i) =yirf;
            NIRF(:,i) =nirf;
            FBNIRF(:,i) =fbnirf;
            PHI1IRF(:,i) = phi1irf;
            PHI2IRF(:,i) = phi2irf;
            PHI3IRF(:,i) = phi3irf;
            TAU1IRF(:,i) = tau1irf;
            TAU2IRF(:,i) = tau2irf;
            TAU3IRF(:,i) = tau3irf;
           
        end
        clear xt nt prodt;
        EYirf(:,1)  = mean(YIRF,2);
        ENirf(:,1)  = mean(NIRF,2);
        EFBNirf(:,1) = mean(FBNIRF,2);
        Ephi1irf(:,1) = mean(PHI1IRF,2);
        Ephi2irf(:,1) = mean(PHI2IRF,2);
        Ephi3irf(:,1) = mean(PHI3IRF,2);
        Etau1irf(:,1) = mean(TAU1IRF,2);
        Etau2irf(:,1) = mean(TAU2IRF,2);
        Etau3irf(:,1) = mean(TAU3IRF,2);
    
    
   
        dlmwrite('IRF_measerr.txt', [ENirf,EFBNirf])

    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
